# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main_window_ui.ui'
#
# Created by: PyQt5 UI code generator 5.11.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setWindowModality(QtCore.Qt.NonModal)
        MainWindow.setEnabled(True)
        MainWindow.resize(791, 641)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(0, 45))
        MainWindow.setMaximumSize(QtCore.QSize(16777179, 16777215))
        MainWindow.setSizeIncrement(QtCore.QSize(9, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        MainWindow.setFont(font)
        MainWindow.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        MainWindow.setFocusPolicy(QtCore.Qt.NoFocus)
        MainWindow.setContextMenuPolicy(QtCore.Qt.PreventContextMenu)
        MainWindow.setToolTipDuration(6)
        MainWindow.setAutoFillBackground(True)
        self.le_mountStatus = QtWidgets.QLineEdit(MainWindow)
        self.le_mountStatus.setEnabled(False)
        self.le_mountStatus.setGeometry(QtCore.QRect(20, 40, 221, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.le_mountStatus.setFont(font)
        self.le_mountStatus.setMouseTracking(False)
        self.le_mountStatus.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_mountStatus.setAcceptDrops(False)
        self.le_mountStatus.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_mountStatus.setReadOnly(True)
        self.le_mountStatus.setObjectName("le_mountStatus")
        self.mainTabWidget = QtWidgets.QTabWidget(MainWindow)
        self.mainTabWidget.setEnabled(True)
        self.mainTabWidget.setGeometry(QtCore.QRect(5, 150, 781, 486))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mainTabWidget.sizePolicy().hasHeightForWidth())
        self.mainTabWidget.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mainTabWidget.setFont(font)
        self.mainTabWidget.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.mainTabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.mainTabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.mainTabWidget.setElideMode(QtCore.Qt.ElideRight)
        self.mainTabWidget.setTabsClosable(False)
        self.mainTabWidget.setMovable(False)
        self.mainTabWidget.setTabBarAutoHide(False)
        self.mainTabWidget.setObjectName("mainTabWidget")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.le_telescopeRA = QtWidgets.QLineEdit(self.tab_2)
        self.le_telescopeRA.setEnabled(False)
        self.le_telescopeRA.setGeometry(QtCore.QRect(20, 150, 201, 41))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(28)
        font.setBold(True)
        font.setWeight(75)
        self.le_telescopeRA.setFont(font)
        self.le_telescopeRA.setMouseTracking(False)
        self.le_telescopeRA.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_telescopeRA.setAcceptDrops(False)
        self.le_telescopeRA.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_telescopeRA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_telescopeRA.setReadOnly(True)
        self.le_telescopeRA.setProperty("large", True)
        self.le_telescopeRA.setObjectName("le_telescopeRA")
        self.le_telescopeAzimut = QtWidgets.QLineEdit(self.tab_2)
        self.le_telescopeAzimut.setEnabled(False)
        self.le_telescopeAzimut.setGeometry(QtCore.QRect(20, 80, 201, 41))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(28)
        font.setBold(True)
        font.setWeight(75)
        self.le_telescopeAzimut.setFont(font)
        self.le_telescopeAzimut.setMouseTracking(False)
        self.le_telescopeAzimut.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_telescopeAzimut.setAcceptDrops(False)
        self.le_telescopeAzimut.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_telescopeAzimut.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_telescopeAzimut.setReadOnly(True)
        self.le_telescopeAzimut.setProperty("large", True)
        self.le_telescopeAzimut.setObjectName("le_telescopeAzimut")
        self.le_telescopeDEC = QtWidgets.QLineEdit(self.tab_2)
        self.le_telescopeDEC.setEnabled(False)
        self.le_telescopeDEC.setGeometry(QtCore.QRect(240, 150, 201, 41))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(28)
        font.setBold(True)
        font.setWeight(75)
        self.le_telescopeDEC.setFont(font)
        self.le_telescopeDEC.setMouseTracking(False)
        self.le_telescopeDEC.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_telescopeDEC.setAcceptDrops(False)
        self.le_telescopeDEC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_telescopeDEC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_telescopeDEC.setReadOnly(True)
        self.le_telescopeDEC.setProperty("large", True)
        self.le_telescopeDEC.setObjectName("le_telescopeDEC")
        self.le_telescopeAltitude = QtWidgets.QLineEdit(self.tab_2)
        self.le_telescopeAltitude.setEnabled(False)
        self.le_telescopeAltitude.setGeometry(QtCore.QRect(240, 80, 201, 41))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(28)
        font.setBold(True)
        font.setWeight(75)
        self.le_telescopeAltitude.setFont(font)
        self.le_telescopeAltitude.setMouseTracking(False)
        self.le_telescopeAltitude.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_telescopeAltitude.setAcceptDrops(False)
        self.le_telescopeAltitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_telescopeAltitude.setStyleSheet("")
        self.le_telescopeAltitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_telescopeAltitude.setReadOnly(True)
        self.le_telescopeAltitude.setProperty("large", True)
        self.le_telescopeAltitude.setObjectName("le_telescopeAltitude")
        self.label_9 = QtWidgets.QLabel(self.tab_2)
        self.label_9.setGeometry(QtCore.QRect(20, 60, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.label_10 = QtWidgets.QLabel(self.tab_2)
        self.label_10.setGeometry(QtCore.QRect(240, 130, 131, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_10.setFont(font)
        self.label_10.setObjectName("label_10")
        self.label_8 = QtWidgets.QLabel(self.tab_2)
        self.label_8.setGeometry(QtCore.QRect(240, 60, 121, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.label_11 = QtWidgets.QLabel(self.tab_2)
        self.label_11.setGeometry(QtCore.QRect(20, 130, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_11.setFont(font)
        self.label_11.setObjectName("label_11")
        self.label_17 = QtWidgets.QLabel(self.tab_2)
        self.label_17.setGeometry(QtCore.QRect(20, 250, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_17.setFont(font)
        self.label_17.setObjectName("label_17")
        self.le_meridianLimitGuide = QtWidgets.QLineEdit(self.tab_2)
        self.le_meridianLimitGuide.setEnabled(True)
        self.le_meridianLimitGuide.setGeometry(QtCore.QRect(140, 340, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_meridianLimitGuide.setFont(font)
        self.le_meridianLimitGuide.setMouseTracking(False)
        self.le_meridianLimitGuide.setAcceptDrops(False)
        self.le_meridianLimitGuide.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_meridianLimitGuide.setInputMask("")
        self.le_meridianLimitGuide.setCursorPosition(2)
        self.le_meridianLimitGuide.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_meridianLimitGuide.setReadOnly(True)
        self.le_meridianLimitGuide.setObjectName("le_meridianLimitGuide")
        self.le_telescopePierSide = QtWidgets.QLineEdit(self.tab_2)
        self.le_telescopePierSide.setEnabled(False)
        self.le_telescopePierSide.setGeometry(QtCore.QRect(370, 250, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_telescopePierSide.setFont(font)
        self.le_telescopePierSide.setMouseTracking(False)
        self.le_telescopePierSide.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_telescopePierSide.setAcceptDrops(False)
        self.le_telescopePierSide.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_telescopePierSide.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_telescopePierSide.setReadOnly(True)
        self.le_telescopePierSide.setObjectName("le_telescopePierSide")
        self.label_18 = QtWidgets.QLabel(self.tab_2)
        self.label_18.setGeometry(QtCore.QRect(250, 250, 71, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_18.setFont(font)
        self.label_18.setObjectName("label_18")
        self.label_21 = QtWidgets.QLabel(self.tab_2)
        self.label_21.setGeometry(QtCore.QRect(20, 280, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_21.setFont(font)
        self.label_21.setObjectName("label_21")
        self.le_telescopeDualTrack = QtWidgets.QLineEdit(self.tab_2)
        self.le_telescopeDualTrack.setEnabled(False)
        self.le_telescopeDualTrack.setGeometry(QtCore.QRect(140, 280, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_telescopeDualTrack.setFont(font)
        self.le_telescopeDualTrack.setMouseTracking(False)
        self.le_telescopeDualTrack.setAcceptDrops(False)
        self.le_telescopeDualTrack.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_telescopeDualTrack.setInputMask("")
        self.le_telescopeDualTrack.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_telescopeDualTrack.setReadOnly(True)
        self.le_telescopeDualTrack.setObjectName("le_telescopeDualTrack")
        self.btn_setDualTracking = QtWidgets.QPushButton(self.tab_2)
        self.btn_setDualTracking.setGeometry(QtCore.QRect(190, 280, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_setDualTracking.setFont(font)
        self.btn_setDualTracking.setObjectName("btn_setDualTracking")
        self.label_52 = QtWidgets.QLabel(self.tab_2)
        self.label_52.setGeometry(QtCore.QRect(250, 340, 91, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_52.setFont(font)
        self.label_52.setWordWrap(False)
        self.label_52.setObjectName("label_52")
        self.le_timeToFlip = QtWidgets.QLineEdit(self.tab_2)
        self.le_timeToFlip.setEnabled(False)
        self.le_timeToFlip.setGeometry(QtCore.QRect(370, 340, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_timeToFlip.setFont(font)
        self.le_timeToFlip.setMouseTracking(False)
        self.le_timeToFlip.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_timeToFlip.setAcceptDrops(False)
        self.le_timeToFlip.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_timeToFlip.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_timeToFlip.setReadOnly(True)
        self.le_timeToFlip.setObjectName("le_timeToFlip")
        self.btn_setUnattendedFlip = QtWidgets.QPushButton(self.tab_2)
        self.btn_setUnattendedFlip.setGeometry(QtCore.QRect(190, 310, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_setUnattendedFlip.setFont(font)
        self.btn_setUnattendedFlip.setObjectName("btn_setUnattendedFlip")
        self.le_telescopeUnattendedFlip = QtWidgets.QLineEdit(self.tab_2)
        self.le_telescopeUnattendedFlip.setEnabled(False)
        self.le_telescopeUnattendedFlip.setGeometry(QtCore.QRect(140, 310, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_telescopeUnattendedFlip.setFont(font)
        self.le_telescopeUnattendedFlip.setMouseTracking(False)
        self.le_telescopeUnattendedFlip.setAcceptDrops(False)
        self.le_telescopeUnattendedFlip.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_telescopeUnattendedFlip.setInputMask("")
        self.le_telescopeUnattendedFlip.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_telescopeUnattendedFlip.setReadOnly(True)
        self.le_telescopeUnattendedFlip.setObjectName("le_telescopeUnattendedFlip")
        self.label_23 = QtWidgets.QLabel(self.tab_2)
        self.label_23.setGeometry(QtCore.QRect(20, 310, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_23.setFont(font)
        self.label_23.setObjectName("label_23")
        self.label_61 = QtWidgets.QLabel(self.tab_2)
        self.label_61.setGeometry(QtCore.QRect(415, 340, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_61.setFont(font)
        self.label_61.setAlignment(QtCore.Qt.AlignCenter)
        self.label_61.setWordWrap(False)
        self.label_61.setObjectName("label_61")
        self.label_62 = QtWidgets.QLabel(self.tab_2)
        self.label_62.setGeometry(QtCore.QRect(90, 250, 31, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_62.setFont(font)
        self.label_62.setAlignment(QtCore.Qt.AlignCenter)
        self.label_62.setWordWrap(False)
        self.label_62.setObjectName("label_62")
        self.label_80 = QtWidgets.QLabel(self.tab_2)
        self.label_80.setGeometry(QtCore.QRect(20, 220, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_80.setFont(font)
        self.label_80.setObjectName("label_80")
        self.line_12 = QtWidgets.QFrame(self.tab_2)
        self.line_12.setGeometry(QtCore.QRect(20, 240, 421, 1))
        self.line_12.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_12.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_12.setObjectName("line_12")
        self.line_13 = QtWidgets.QFrame(self.tab_2)
        self.line_13.setGeometry(QtCore.QRect(20, 40, 421, 1))
        self.line_13.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_13.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_13.setObjectName("line_13")
        self.btn_startTracking = QtWidgets.QPushButton(self.tab_2)
        self.btn_startTracking.setGeometry(QtCore.QRect(480, 220, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_startTracking.setFont(font)
        self.btn_startTracking.setObjectName("btn_startTracking")
        self.btn_mountPark = QtWidgets.QPushButton(self.tab_2)
        self.btn_mountPark.setGeometry(QtCore.QRect(480, 350, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_mountPark.setFont(font)
        self.btn_mountPark.setObjectName("btn_mountPark")
        self.btn_mountUnpark = QtWidgets.QPushButton(self.tab_2)
        self.btn_mountUnpark.setGeometry(QtCore.QRect(620, 350, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_mountUnpark.setFont(font)
        self.btn_mountUnpark.setObjectName("btn_mountUnpark")
        self.line_16 = QtWidgets.QFrame(self.tab_2)
        self.line_16.setGeometry(QtCore.QRect(480, 340, 271, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_16.setFont(font)
        self.line_16.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_16.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_16.setObjectName("line_16")
        self.label_83 = QtWidgets.QLabel(self.tab_2)
        self.label_83.setGeometry(QtCore.QRect(480, 20, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_83.setFont(font)
        self.label_83.setObjectName("label_83")
        self.label_84 = QtWidgets.QLabel(self.tab_2)
        self.label_84.setGeometry(QtCore.QRect(480, 190, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_84.setFont(font)
        self.label_84.setObjectName("label_84")
        self.btn_mountPos4 = QtWidgets.QPushButton(self.tab_2)
        self.btn_mountPos4.setGeometry(QtCore.QRect(620, 90, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_mountPos4.setFont(font)
        self.btn_mountPos4.setObjectName("btn_mountPos4")
        self.line_15 = QtWidgets.QFrame(self.tab_2)
        self.line_15.setGeometry(QtCore.QRect(480, 210, 271, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_15.setFont(font)
        self.line_15.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_15.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_15.setObjectName("line_15")
        self.label_85 = QtWidgets.QLabel(self.tab_2)
        self.label_85.setGeometry(QtCore.QRect(480, 320, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_85.setFont(font)
        self.label_85.setObjectName("label_85")
        self.btn_mountPos2 = QtWidgets.QPushButton(self.tab_2)
        self.btn_mountPos2.setGeometry(QtCore.QRect(620, 50, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_mountPos2.setFont(font)
        self.btn_mountPos2.setObjectName("btn_mountPos2")
        self.btn_mountPos3 = QtWidgets.QPushButton(self.tab_2)
        self.btn_mountPos3.setGeometry(QtCore.QRect(480, 90, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_mountPos3.setFont(font)
        self.btn_mountPos3.setObjectName("btn_mountPos3")
        self.btn_mountPos1 = QtWidgets.QPushButton(self.tab_2)
        self.btn_mountPos1.setGeometry(QtCore.QRect(480, 50, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_mountPos1.setFont(font)
        self.btn_mountPos1.setObjectName("btn_mountPos1")
        self.btn_stopTracking = QtWidgets.QPushButton(self.tab_2)
        self.btn_stopTracking.setGeometry(QtCore.QRect(620, 220, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_stopTracking.setFont(font)
        self.btn_stopTracking.setObjectName("btn_stopTracking")
        self.line_14 = QtWidgets.QFrame(self.tab_2)
        self.line_14.setGeometry(QtCore.QRect(480, 40, 271, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_14.setFont(font)
        self.line_14.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_14.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_14.setObjectName("line_14")
        self.btn_stop = QtWidgets.QPushButton(self.tab_2)
        self.btn_stop.setGeometry(QtCore.QRect(480, 390, 271, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_stop.setFont(font)
        self.btn_stop.setObjectName("btn_stop")
        self.btn_flipMount = QtWidgets.QPushButton(self.tab_2)
        self.btn_flipMount.setGeometry(QtCore.QRect(250, 280, 191, 51))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_flipMount.setFont(font)
        self.btn_flipMount.setObjectName("btn_flipMount")
        self.btn_setTrackingLunar = QtWidgets.QPushButton(self.tab_2)
        self.btn_setTrackingLunar.setGeometry(QtCore.QRect(480, 260, 91, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_setTrackingLunar.setFont(font)
        self.btn_setTrackingLunar.setObjectName("btn_setTrackingLunar")
        self.btn_setTrackingSolar = QtWidgets.QPushButton(self.tab_2)
        self.btn_setTrackingSolar.setGeometry(QtCore.QRect(580, 260, 81, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_setTrackingSolar.setFont(font)
        self.btn_setTrackingSolar.setObjectName("btn_setTrackingSolar")
        self.btn_setTrackingSideral = QtWidgets.QPushButton(self.tab_2)
        self.btn_setTrackingSideral.setGeometry(QtCore.QRect(670, 260, 81, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_setTrackingSideral.setFont(font)
        self.btn_setTrackingSideral.setObjectName("btn_setTrackingSideral")
        self.label_34 = QtWidgets.QLabel(self.tab_2)
        self.label_34.setGeometry(QtCore.QRect(20, 340, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_34.setFont(font)
        self.label_34.setObjectName("label_34")
        self.label_65 = QtWidgets.QLabel(self.tab_2)
        self.label_65.setGeometry(QtCore.QRect(20, 370, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_65.setFont(font)
        self.label_65.setObjectName("label_65")
        self.le_meridianLimitSlew = QtWidgets.QLineEdit(self.tab_2)
        self.le_meridianLimitSlew.setGeometry(QtCore.QRect(140, 370, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_meridianLimitSlew.setFont(font)
        self.le_meridianLimitSlew.setMouseTracking(False)
        self.le_meridianLimitSlew.setAcceptDrops(False)
        self.le_meridianLimitSlew.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_meridianLimitSlew.setInputMask("")
        self.le_meridianLimitSlew.setCursorPosition(2)
        self.le_meridianLimitSlew.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_meridianLimitSlew.setReadOnly(True)
        self.le_meridianLimitSlew.setObjectName("le_meridianLimitSlew")
        self.label_88 = QtWidgets.QLabel(self.tab_2)
        self.label_88.setGeometry(QtCore.QRect(180, 340, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_88.setFont(font)
        self.label_88.setAlignment(QtCore.Qt.AlignCenter)
        self.label_88.setWordWrap(False)
        self.label_88.setObjectName("label_88")
        self.label_89 = QtWidgets.QLabel(self.tab_2)
        self.label_89.setGeometry(QtCore.QRect(180, 370, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_89.setFont(font)
        self.label_89.setAlignment(QtCore.Qt.AlignCenter)
        self.label_89.setWordWrap(False)
        self.label_89.setObjectName("label_89")
        self.le_slewRate = QtWidgets.QLineEdit(self.tab_2)
        self.le_slewRate.setGeometry(QtCore.QRect(140, 250, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_slewRate.setFont(font)
        self.le_slewRate.setMouseTracking(False)
        self.le_slewRate.setAcceptDrops(False)
        self.le_slewRate.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_slewRate.setInputMask("")
        self.le_slewRate.setText("")
        self.le_slewRate.setCursorPosition(0)
        self.le_slewRate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_slewRate.setReadOnly(False)
        self.le_slewRate.setObjectName("le_slewRate")
        self.label_90 = QtWidgets.QLabel(self.tab_2)
        self.label_90.setGeometry(QtCore.QRect(250, 370, 101, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_90.setFont(font)
        self.label_90.setWordWrap(False)
        self.label_90.setObjectName("label_90")
        self.le_timeToMeridian = QtWidgets.QLineEdit(self.tab_2)
        self.le_timeToMeridian.setEnabled(False)
        self.le_timeToMeridian.setGeometry(QtCore.QRect(370, 370, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_timeToMeridian.setFont(font)
        self.le_timeToMeridian.setMouseTracking(False)
        self.le_timeToMeridian.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_timeToMeridian.setAcceptDrops(False)
        self.le_timeToMeridian.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_timeToMeridian.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_timeToMeridian.setReadOnly(True)
        self.le_timeToMeridian.setObjectName("le_timeToMeridian")
        self.label_91 = QtWidgets.QLabel(self.tab_2)
        self.label_91.setGeometry(QtCore.QRect(415, 370, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_91.setFont(font)
        self.label_91.setAlignment(QtCore.Qt.AlignCenter)
        self.label_91.setWordWrap(False)
        self.label_91.setObjectName("label_91")
        self.btn_mountPos6 = QtWidgets.QPushButton(self.tab_2)
        self.btn_mountPos6.setGeometry(QtCore.QRect(620, 130, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_mountPos6.setFont(font)
        self.btn_mountPos6.setObjectName("btn_mountPos6")
        self.btn_mountPos5 = QtWidgets.QPushButton(self.tab_2)
        self.btn_mountPos5.setGeometry(QtCore.QRect(480, 130, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_mountPos5.setFont(font)
        self.btn_mountPos5.setObjectName("btn_mountPos5")
        self.label_20 = QtWidgets.QLabel(self.tab_2)
        self.label_20.setGeometry(QtCore.QRect(250, 400, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_20.setFont(font)
        self.label_20.setObjectName("label_20")
        self.le_horizonLimitHigh = QtWidgets.QLineEdit(self.tab_2)
        self.le_horizonLimitHigh.setGeometry(QtCore.QRect(370, 400, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_horizonLimitHigh.setFont(font)
        self.le_horizonLimitHigh.setMouseTracking(False)
        self.le_horizonLimitHigh.setAcceptDrops(False)
        self.le_horizonLimitHigh.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_horizonLimitHigh.setInputMask("")
        self.le_horizonLimitHigh.setText("")
        self.le_horizonLimitHigh.setCursorPosition(0)
        self.le_horizonLimitHigh.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_horizonLimitHigh.setReadOnly(False)
        self.le_horizonLimitHigh.setObjectName("le_horizonLimitHigh")
        self.label_4 = QtWidgets.QLabel(self.tab_2)
        self.label_4.setGeometry(QtCore.QRect(20, 400, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.le_horizonLimitLow = QtWidgets.QLineEdit(self.tab_2)
        self.le_horizonLimitLow.setGeometry(QtCore.QRect(140, 400, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_horizonLimitLow.setFont(font)
        self.le_horizonLimitLow.setMouseTracking(False)
        self.le_horizonLimitLow.setAcceptDrops(False)
        self.le_horizonLimitLow.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_horizonLimitLow.setInputMask("")
        self.le_horizonLimitLow.setText("")
        self.le_horizonLimitLow.setMaxLength(32767)
        self.le_horizonLimitLow.setFrame(True)
        self.le_horizonLimitLow.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_horizonLimitLow.setReadOnly(False)
        self.le_horizonLimitLow.setClearButtonEnabled(False)
        self.le_horizonLimitLow.setObjectName("le_horizonLimitLow")
        self.line_51 = QtWidgets.QFrame(self.tab_2)
        self.line_51.setGeometry(QtCore.QRect(450, 10, 21, 431))
        self.line_51.setStyleSheet("")
        self.line_51.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_51.setLineWidth(2)
        self.line_51.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_51.setObjectName("line_51")
        self.label_98 = QtWidgets.QLabel(self.tab_2)
        self.label_98.setGeometry(QtCore.QRect(20, 20, 141, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_98.setFont(font)
        self.label_98.setObjectName("label_98")
        self.label_104 = QtWidgets.QLabel(self.tab_2)
        self.label_104.setGeometry(QtCore.QRect(180, 400, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_104.setFont(font)
        self.label_104.setAlignment(QtCore.Qt.AlignCenter)
        self.label_104.setWordWrap(False)
        self.label_104.setObjectName("label_104")
        self.label_109 = QtWidgets.QLabel(self.tab_2)
        self.label_109.setGeometry(QtCore.QRect(415, 400, 26, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_109.setFont(font)
        self.label_109.setAlignment(QtCore.Qt.AlignCenter)
        self.label_109.setWordWrap(False)
        self.label_109.setObjectName("label_109")
        self.line_15.raise_()
        self.line_16.raise_()
        self.line_14.raise_()
        self.line_12.raise_()
        self.line_13.raise_()
        self.line_51.raise_()
        self.le_telescopeRA.raise_()
        self.le_telescopeAzimut.raise_()
        self.le_telescopeDEC.raise_()
        self.le_telescopeAltitude.raise_()
        self.label_9.raise_()
        self.label_10.raise_()
        self.label_8.raise_()
        self.label_11.raise_()
        self.label_17.raise_()
        self.le_meridianLimitGuide.raise_()
        self.le_telescopePierSide.raise_()
        self.label_18.raise_()
        self.label_21.raise_()
        self.le_telescopeDualTrack.raise_()
        self.btn_setDualTracking.raise_()
        self.label_52.raise_()
        self.le_timeToFlip.raise_()
        self.btn_setUnattendedFlip.raise_()
        self.le_telescopeUnattendedFlip.raise_()
        self.label_23.raise_()
        self.label_61.raise_()
        self.label_62.raise_()
        self.label_80.raise_()
        self.btn_startTracking.raise_()
        self.btn_mountPark.raise_()
        self.btn_mountUnpark.raise_()
        self.label_83.raise_()
        self.label_84.raise_()
        self.btn_mountPos4.raise_()
        self.label_85.raise_()
        self.btn_mountPos2.raise_()
        self.btn_mountPos3.raise_()
        self.btn_mountPos1.raise_()
        self.btn_stopTracking.raise_()
        self.btn_stop.raise_()
        self.btn_flipMount.raise_()
        self.btn_setTrackingLunar.raise_()
        self.btn_setTrackingSolar.raise_()
        self.btn_setTrackingSideral.raise_()
        self.label_34.raise_()
        self.label_65.raise_()
        self.le_meridianLimitSlew.raise_()
        self.label_88.raise_()
        self.label_89.raise_()
        self.le_slewRate.raise_()
        self.label_90.raise_()
        self.le_timeToMeridian.raise_()
        self.label_91.raise_()
        self.btn_mountPos6.raise_()
        self.btn_mountPos5.raise_()
        self.label_20.raise_()
        self.le_horizonLimitHigh.raise_()
        self.label_4.raise_()
        self.le_horizonLimitLow.raise_()
        self.label_98.raise_()
        self.label_104.raise_()
        self.label_109.raise_()
        self.mainTabWidget.addTab(self.tab_2, "")
        self.tab_4 = QtWidgets.QWidget()
        self.tab_4.setObjectName("tab_4")
        self.le_siteElevation = QtWidgets.QLineEdit(self.tab_4)
        self.le_siteElevation.setEnabled(False)
        self.le_siteElevation.setGeometry(QtCore.QRect(160, 110, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_siteElevation.setFont(font)
        self.le_siteElevation.setMouseTracking(False)
        self.le_siteElevation.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_siteElevation.setAcceptDrops(False)
        self.le_siteElevation.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_siteElevation.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_siteElevation.setReadOnly(True)
        self.le_siteElevation.setObjectName("le_siteElevation")
        self.label_5 = QtWidgets.QLabel(self.tab_4)
        self.label_5.setGeometry(QtCore.QRect(10, 50, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.le_siteLongitude = QtWidgets.QLineEdit(self.tab_4)
        self.le_siteLongitude.setEnabled(False)
        self.le_siteLongitude.setGeometry(QtCore.QRect(160, 50, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_siteLongitude.setFont(font)
        self.le_siteLongitude.setMouseTracking(False)
        self.le_siteLongitude.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_siteLongitude.setAcceptDrops(False)
        self.le_siteLongitude.setToolTip("")
        self.le_siteLongitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_siteLongitude.setMaxLength(32634)
        self.le_siteLongitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_siteLongitude.setReadOnly(True)
        self.le_siteLongitude.setCursorMoveStyle(QtCore.Qt.VisualMoveStyle)
        self.le_siteLongitude.setObjectName("le_siteLongitude")
        self.label_6 = QtWidgets.QLabel(self.tab_4)
        self.label_6.setGeometry(QtCore.QRect(10, 80, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.le_localSiderealTime = QtWidgets.QLineEdit(self.tab_4)
        self.le_localSiderealTime.setEnabled(False)
        self.le_localSiderealTime.setGeometry(QtCore.QRect(160, 140, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_localSiderealTime.setFont(font)
        self.le_localSiderealTime.setMouseTracking(False)
        self.le_localSiderealTime.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_localSiderealTime.setAcceptDrops(False)
        self.le_localSiderealTime.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_localSiderealTime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_localSiderealTime.setReadOnly(True)
        self.le_localSiderealTime.setObjectName("le_localSiderealTime")
        self.label_7 = QtWidgets.QLabel(self.tab_4)
        self.label_7.setGeometry(QtCore.QRect(10, 140, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.le_siteLatitude = QtWidgets.QLineEdit(self.tab_4)
        self.le_siteLatitude.setEnabled(False)
        self.le_siteLatitude.setGeometry(QtCore.QRect(160, 80, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_siteLatitude.setFont(font)
        self.le_siteLatitude.setMouseTracking(False)
        self.le_siteLatitude.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_siteLatitude.setAcceptDrops(False)
        self.le_siteLatitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_siteLatitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_siteLatitude.setReadOnly(True)
        self.le_siteLatitude.setObjectName("le_siteLatitude")
        self.label_12 = QtWidgets.QLabel(self.tab_4)
        self.label_12.setGeometry(QtCore.QRect(10, 110, 111, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_12.setFont(font)
        self.label_12.setObjectName("label_12")
        self.label_58 = QtWidgets.QLabel(self.tab_4)
        self.label_58.setGeometry(QtCore.QRect(10, 20, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_58.setFont(font)
        self.label_58.setObjectName("label_58")
        self.line_8 = QtWidgets.QFrame(self.tab_4)
        self.line_8.setGeometry(QtCore.QRect(10, 40, 301, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_8.setFont(font)
        self.line_8.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_8.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_8.setObjectName("line_8")
        self.label_16 = QtWidgets.QLabel(self.tab_4)
        self.label_16.setGeometry(QtCore.QRect(10, 240, 211, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_16.setFont(font)
        self.label_16.setObjectName("label_16")
        self.le_telescopeTempDECMotor = QtWidgets.QLineEdit(self.tab_4)
        self.le_telescopeTempDECMotor.setEnabled(False)
        self.le_telescopeTempDECMotor.setGeometry(QtCore.QRect(220, 240, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_telescopeTempDECMotor.setFont(font)
        self.le_telescopeTempDECMotor.setMouseTracking(False)
        self.le_telescopeTempDECMotor.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_telescopeTempDECMotor.setAcceptDrops(False)
        self.le_telescopeTempDECMotor.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_telescopeTempDECMotor.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_telescopeTempDECMotor.setReadOnly(True)
        self.le_telescopeTempDECMotor.setObjectName("le_telescopeTempDECMotor")
        self.label_55 = QtWidgets.QLabel(self.tab_4)
        self.label_55.setGeometry(QtCore.QRect(10, 210, 221, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_55.setFont(font)
        self.label_55.setObjectName("label_55")
        self.line_5 = QtWidgets.QFrame(self.tab_4)
        self.line_5.setGeometry(QtCore.QRect(10, 230, 301, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_5.setFont(font)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setObjectName("line_5")
        self.label_64 = QtWidgets.QLabel(self.tab_4)
        self.label_64.setGeometry(QtCore.QRect(290, 240, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_64.setFont(font)
        self.label_64.setObjectName("label_64")
        self.line_7 = QtWidgets.QFrame(self.tab_4)
        self.line_7.setGeometry(QtCore.QRect(350, 40, 401, 1))
        self.line_7.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_7.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_7.setObjectName("line_7")
        self.le_windSpeed = QtWidgets.QLineEdit(self.tab_4)
        self.le_windSpeed.setEnabled(False)
        self.le_windSpeed.setGeometry(QtCore.QRect(650, 140, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_windSpeed.setFont(font)
        self.le_windSpeed.setMouseTracking(False)
        self.le_windSpeed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_windSpeed.setAcceptDrops(False)
        self.le_windSpeed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_windSpeed.setMaxLength(8)
        self.le_windSpeed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_windSpeed.setReadOnly(True)
        self.le_windSpeed.setObjectName("le_windSpeed")
        self.label_32 = QtWidgets.QLabel(self.tab_4)
        self.label_32.setGeometry(QtCore.QRect(580, 140, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_32.setFont(font)
        self.label_32.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_32.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_32.setObjectName("label_32")
        self.label_73 = QtWidgets.QLabel(self.tab_4)
        self.label_73.setGeometry(QtCore.QRect(490, 80, 31, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_73.setFont(font)
        self.label_73.setAlignment(QtCore.Qt.AlignCenter)
        self.label_73.setWordWrap(False)
        self.label_73.setObjectName("label_73")
        self.le_temperature = QtWidgets.QLineEdit(self.tab_4)
        self.le_temperature.setEnabled(False)
        self.le_temperature.setGeometry(QtCore.QRect(420, 50, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_temperature.setFont(font)
        self.le_temperature.setMouseTracking(False)
        self.le_temperature.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_temperature.setAcceptDrops(False)
        self.le_temperature.setToolTip("")
        self.le_temperature.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_temperature.setMaxLength(8)
        self.le_temperature.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_temperature.setReadOnly(True)
        self.le_temperature.setObjectName("le_temperature")
        self.label_25 = QtWidgets.QLabel(self.tab_4)
        self.label_25.setGeometry(QtCore.QRect(350, 110, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_25.setFont(font)
        self.label_25.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_25.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_25.setObjectName("label_25")
        self.label_31 = QtWidgets.QLabel(self.tab_4)
        self.label_31.setGeometry(QtCore.QRect(350, 200, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_31.setFont(font)
        self.label_31.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_31.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_31.setObjectName("label_31")
        self.label_70 = QtWidgets.QLabel(self.tab_4)
        self.label_70.setGeometry(QtCore.QRect(490, 50, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_70.setFont(font)
        self.label_70.setAlignment(QtCore.Qt.AlignCenter)
        self.label_70.setWordWrap(False)
        self.label_70.setObjectName("label_70")
        self.label_68 = QtWidgets.QLabel(self.tab_4)
        self.label_68.setGeometry(QtCore.QRect(490, 200, 31, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_68.setFont(font)
        self.label_68.setAlignment(QtCore.Qt.AlignCenter)
        self.label_68.setWordWrap(False)
        self.label_68.setObjectName("label_68")
        self.label_26 = QtWidgets.QLabel(self.tab_4)
        self.label_26.setGeometry(QtCore.QRect(350, 140, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_26.setFont(font)
        self.label_26.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_26.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_26.setObjectName("label_26")
        self.le_refractionTemperature = QtWidgets.QLineEdit(self.tab_4)
        self.le_refractionTemperature.setEnabled(False)
        self.le_refractionTemperature.setGeometry(QtCore.QRect(650, 50, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_refractionTemperature.setFont(font)
        self.le_refractionTemperature.setMouseTracking(False)
        self.le_refractionTemperature.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_refractionTemperature.setAcceptDrops(False)
        self.le_refractionTemperature.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_refractionTemperature.setMaxLength(8)
        self.le_refractionTemperature.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_refractionTemperature.setReadOnly(True)
        self.le_refractionTemperature.setObjectName("le_refractionTemperature")
        self.le_humidity = QtWidgets.QLineEdit(self.tab_4)
        self.le_humidity.setEnabled(False)
        self.le_humidity.setGeometry(QtCore.QRect(420, 140, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_humidity.setFont(font)
        self.le_humidity.setMouseTracking(False)
        self.le_humidity.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_humidity.setAcceptDrops(False)
        self.le_humidity.setToolTip("")
        self.le_humidity.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_humidity.setMaxLength(8)
        self.le_humidity.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_humidity.setReadOnly(True)
        self.le_humidity.setObjectName("le_humidity")
        self.le_cloudCover = QtWidgets.QLineEdit(self.tab_4)
        self.le_cloudCover.setEnabled(False)
        self.le_cloudCover.setGeometry(QtCore.QRect(420, 170, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_cloudCover.setFont(font)
        self.le_cloudCover.setMouseTracking(False)
        self.le_cloudCover.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_cloudCover.setAcceptDrops(False)
        self.le_cloudCover.setToolTip("")
        self.le_cloudCover.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_cloudCover.setMaxLength(8)
        self.le_cloudCover.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_cloudCover.setReadOnly(True)
        self.le_cloudCover.setObjectName("le_cloudCover")
        self.label_67 = QtWidgets.QLabel(self.tab_4)
        self.label_67.setGeometry(QtCore.QRect(720, 140, 41, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_67.setFont(font)
        self.label_67.setAlignment(QtCore.Qt.AlignCenter)
        self.label_67.setWordWrap(False)
        self.label_67.setObjectName("label_67")
        self.btn_setRefractionParameters = QtWidgets.QPushButton(self.tab_4)
        self.btn_setRefractionParameters.setGeometry(QtCore.QRect(650, 345, 101, 91))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_setRefractionParameters.setFont(font)
        self.btn_setRefractionParameters.setFlat(False)
        self.btn_setRefractionParameters.setObjectName("btn_setRefractionParameters")
        self.le_dewPoint = QtWidgets.QLineEdit(self.tab_4)
        self.le_dewPoint.setEnabled(False)
        self.le_dewPoint.setGeometry(QtCore.QRect(420, 110, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_dewPoint.setFont(font)
        self.le_dewPoint.setMouseTracking(False)
        self.le_dewPoint.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_dewPoint.setAcceptDrops(False)
        self.le_dewPoint.setToolTip("")
        self.le_dewPoint.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_dewPoint.setMaxLength(8)
        self.le_dewPoint.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_dewPoint.setReadOnly(True)
        self.le_dewPoint.setObjectName("le_dewPoint")
        self.label_71 = QtWidgets.QLabel(self.tab_4)
        self.label_71.setGeometry(QtCore.QRect(490, 110, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_71.setFont(font)
        self.label_71.setAlignment(QtCore.Qt.AlignCenter)
        self.label_71.setWordWrap(False)
        self.label_71.setObjectName("label_71")
        self.le_windDirection = QtWidgets.QLineEdit(self.tab_4)
        self.le_windDirection.setEnabled(False)
        self.le_windDirection.setGeometry(QtCore.QRect(650, 170, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_windDirection.setFont(font)
        self.le_windDirection.setMouseTracking(False)
        self.le_windDirection.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_windDirection.setAcceptDrops(False)
        self.le_windDirection.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_windDirection.setMaxLength(8)
        self.le_windDirection.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_windDirection.setReadOnly(True)
        self.le_windDirection.setObjectName("le_windDirection")
        self.label_33 = QtWidgets.QLabel(self.tab_4)
        self.label_33.setGeometry(QtCore.QRect(580, 170, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_33.setFont(font)
        self.label_33.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_33.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_33.setObjectName("label_33")
        self.label_30 = QtWidgets.QLabel(self.tab_4)
        self.label_30.setGeometry(QtCore.QRect(350, 170, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_30.setFont(font)
        self.label_30.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_30.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_30.setObjectName("label_30")
        self.label_24 = QtWidgets.QLabel(self.tab_4)
        self.label_24.setGeometry(QtCore.QRect(350, 50, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_24.setFont(font)
        self.label_24.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_24.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_24.setObjectName("label_24")
        self.label_22 = QtWidgets.QLabel(self.tab_4)
        self.label_22.setGeometry(QtCore.QRect(350, 270, 291, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_22.setFont(font)
        self.label_22.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_22.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_22.setObjectName("label_22")
        self.label_27 = QtWidgets.QLabel(self.tab_4)
        self.label_27.setGeometry(QtCore.QRect(350, 80, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_27.setFont(font)
        self.label_27.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_27.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_27.setObjectName("label_27")
        self.label_57 = QtWidgets.QLabel(self.tab_4)
        self.label_57.setGeometry(QtCore.QRect(350, 20, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_57.setFont(font)
        self.label_57.setObjectName("label_57")
        self.line_10 = QtWidgets.QFrame(self.tab_4)
        self.line_10.setGeometry(QtCore.QRect(350, 260, 401, 1))
        self.line_10.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_10.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_10.setObjectName("line_10")
        self.le_refractionPressure = QtWidgets.QLineEdit(self.tab_4)
        self.le_refractionPressure.setEnabled(False)
        self.le_refractionPressure.setGeometry(QtCore.QRect(650, 80, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_refractionPressure.setFont(font)
        self.le_refractionPressure.setMouseTracking(False)
        self.le_refractionPressure.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_refractionPressure.setAcceptDrops(False)
        self.le_refractionPressure.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_refractionPressure.setMaxLength(8)
        self.le_refractionPressure.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_refractionPressure.setReadOnly(True)
        self.le_refractionPressure.setObjectName("le_refractionPressure")
        self.le_rainRate = QtWidgets.QLineEdit(self.tab_4)
        self.le_rainRate.setEnabled(False)
        self.le_rainRate.setGeometry(QtCore.QRect(420, 200, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_rainRate.setFont(font)
        self.le_rainRate.setMouseTracking(False)
        self.le_rainRate.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_rainRate.setAcceptDrops(False)
        self.le_rainRate.setToolTip("")
        self.le_rainRate.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_rainRate.setMaxLength(8)
        self.le_rainRate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_rainRate.setReadOnly(True)
        self.le_rainRate.setObjectName("le_rainRate")
        self.label_66 = QtWidgets.QLabel(self.tab_4)
        self.label_66.setGeometry(QtCore.QRect(720, 170, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_66.setFont(font)
        self.label_66.setAlignment(QtCore.Qt.AlignCenter)
        self.label_66.setWordWrap(False)
        self.label_66.setObjectName("label_66")
        self.le_pressure = QtWidgets.QLineEdit(self.tab_4)
        self.le_pressure.setEnabled(False)
        self.le_pressure.setGeometry(QtCore.QRect(420, 80, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_pressure.setFont(font)
        self.le_pressure.setMouseTracking(False)
        self.le_pressure.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_pressure.setAcceptDrops(False)
        self.le_pressure.setToolTip("")
        self.le_pressure.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_pressure.setMaxLength(8)
        self.le_pressure.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_pressure.setReadOnly(True)
        self.le_pressure.setObjectName("le_pressure")
        self.le_refractionStatus = QtWidgets.QLineEdit(self.tab_4)
        self.le_refractionStatus.setEnabled(False)
        self.le_refractionStatus.setGeometry(QtCore.QRect(650, 270, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_refractionStatus.setFont(font)
        self.le_refractionStatus.setMouseTracking(False)
        self.le_refractionStatus.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_refractionStatus.setAcceptDrops(False)
        self.le_refractionStatus.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_refractionStatus.setAlignment(QtCore.Qt.AlignCenter)
        self.le_refractionStatus.setReadOnly(True)
        self.le_refractionStatus.setObjectName("le_refractionStatus")
        self.label_13 = QtWidgets.QLabel(self.tab_4)
        self.label_13.setGeometry(QtCore.QRect(650, 20, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_13.setFont(font)
        self.label_13.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_13.setObjectName("label_13")
        self.label_72 = QtWidgets.QLabel(self.tab_4)
        self.label_72.setGeometry(QtCore.QRect(490, 140, 31, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_72.setFont(font)
        self.label_72.setAlignment(QtCore.Qt.AlignCenter)
        self.label_72.setWordWrap(False)
        self.label_72.setObjectName("label_72")
        self.label_69 = QtWidgets.QLabel(self.tab_4)
        self.label_69.setGeometry(QtCore.QRect(490, 170, 31, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_69.setFont(font)
        self.label_69.setAlignment(QtCore.Qt.AlignCenter)
        self.label_69.setWordWrap(False)
        self.label_69.setObjectName("label_69")
        self.label_78 = QtWidgets.QLabel(self.tab_4)
        self.label_78.setGeometry(QtCore.QRect(720, 80, 31, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_78.setFont(font)
        self.label_78.setAlignment(QtCore.Qt.AlignCenter)
        self.label_78.setWordWrap(False)
        self.label_78.setObjectName("label_78")
        self.label_82 = QtWidgets.QLabel(self.tab_4)
        self.label_82.setGeometry(QtCore.QRect(280, 110, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_82.setFont(font)
        self.label_82.setAlignment(QtCore.Qt.AlignCenter)
        self.label_82.setWordWrap(False)
        self.label_82.setObjectName("label_82")
        self.label_86 = QtWidgets.QLabel(self.tab_4)
        self.label_86.setGeometry(QtCore.QRect(350, 240, 201, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_86.setFont(font)
        self.label_86.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_86.setObjectName("label_86")
        self.btn_setRefractionCorrection = QtWidgets.QPushButton(self.tab_4)
        self.btn_setRefractionCorrection.setGeometry(QtCore.QRect(720, 270, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_setRefractionCorrection.setFont(font)
        self.btn_setRefractionCorrection.setObjectName("btn_setRefractionCorrection")
        self.label_105 = QtWidgets.QLabel(self.tab_4)
        self.label_105.setGeometry(QtCore.QRect(280, 80, 31, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_105.setFont(font)
        self.label_105.setAlignment(QtCore.Qt.AlignCenter)
        self.label_105.setWordWrap(False)
        self.label_105.setObjectName("label_105")
        self.label_106 = QtWidgets.QLabel(self.tab_4)
        self.label_106.setGeometry(QtCore.QRect(280, 50, 31, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_106.setFont(font)
        self.label_106.setAlignment(QtCore.Qt.AlignCenter)
        self.label_106.setWordWrap(False)
        self.label_106.setObjectName("label_106")
        self.label_79 = QtWidgets.QLabel(self.tab_4)
        self.label_79.setGeometry(QtCore.QRect(720, 50, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_79.setFont(font)
        self.label_79.setAlignment(QtCore.Qt.AlignCenter)
        self.label_79.setWordWrap(False)
        self.label_79.setObjectName("label_79")
        self.label_142 = QtWidgets.QLabel(self.tab_4)
        self.label_142.setGeometry(QtCore.QRect(580, 200, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_142.setFont(font)
        self.label_142.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_142.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_142.setObjectName("label_142")
        self.le_SQR = QtWidgets.QLineEdit(self.tab_4)
        self.le_SQR.setEnabled(False)
        self.le_SQR.setGeometry(QtCore.QRect(650, 200, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_SQR.setFont(font)
        self.le_SQR.setMouseTracking(False)
        self.le_SQR.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_SQR.setAcceptDrops(False)
        self.le_SQR.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_SQR.setMaxLength(8)
        self.le_SQR.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_SQR.setReadOnly(True)
        self.le_SQR.setObjectName("le_SQR")
        self.line_53 = QtWidgets.QFrame(self.tab_4)
        self.line_53.setGeometry(QtCore.QRect(320, 10, 21, 431))
        self.line_53.setStyleSheet("")
        self.line_53.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_53.setLineWidth(2)
        self.line_53.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_53.setObjectName("line_53")
        self.label_87 = QtWidgets.QLabel(self.tab_4)
        self.label_87.setGeometry(QtCore.QRect(720, 200, 41, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_87.setFont(font)
        self.label_87.setAlignment(QtCore.Qt.AlignCenter)
        self.label_87.setWordWrap(False)
        self.label_87.setObjectName("label_87")
        self.label_63 = QtWidgets.QLabel(self.tab_4)
        self.label_63.setGeometry(QtCore.QRect(10, 170, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_63.setFont(font)
        self.label_63.setObjectName("label_63")
        self.le_JulianDate = QtWidgets.QLineEdit(self.tab_4)
        self.le_JulianDate.setEnabled(False)
        self.le_JulianDate.setGeometry(QtCore.QRect(160, 170, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_JulianDate.setFont(font)
        self.le_JulianDate.setMouseTracking(False)
        self.le_JulianDate.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_JulianDate.setAcceptDrops(False)
        self.le_JulianDate.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_JulianDate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_JulianDate.setReadOnly(True)
        self.le_JulianDate.setObjectName("le_JulianDate")
        self.label_151 = QtWidgets.QLabel(self.tab_4)
        self.label_151.setGeometry(QtCore.QRect(30, 310, 111, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_151.setFont(font)
        self.label_151.setObjectName("label_151")
        self.le_UTCDataExpirationDate = QtWidgets.QLineEdit(self.tab_4)
        self.le_UTCDataExpirationDate.setEnabled(False)
        self.le_UTCDataExpirationDate.setGeometry(QtCore.QRect(160, 340, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_UTCDataExpirationDate.setFont(font)
        self.le_UTCDataExpirationDate.setMouseTracking(False)
        self.le_UTCDataExpirationDate.setAcceptDrops(False)
        self.le_UTCDataExpirationDate.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_UTCDataExpirationDate.setInputMask("")
        self.le_UTCDataExpirationDate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_UTCDataExpirationDate.setReadOnly(True)
        self.le_UTCDataExpirationDate.setObjectName("le_UTCDataExpirationDate")
        self.label_152 = QtWidgets.QLabel(self.tab_4)
        self.label_152.setGeometry(QtCore.QRect(30, 340, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_152.setFont(font)
        self.label_152.setObjectName("label_152")
        self.le_UTCDataValid = QtWidgets.QLineEdit(self.tab_4)
        self.le_UTCDataValid.setEnabled(False)
        self.le_UTCDataValid.setGeometry(QtCore.QRect(160, 310, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_UTCDataValid.setFont(font)
        self.le_UTCDataValid.setMouseTracking(False)
        self.le_UTCDataValid.setAcceptDrops(False)
        self.le_UTCDataValid.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_UTCDataValid.setInputMask("")
        self.le_UTCDataValid.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_UTCDataValid.setReadOnly(True)
        self.le_UTCDataValid.setObjectName("le_UTCDataValid")
        self.line_76 = QtWidgets.QFrame(self.tab_4)
        self.line_76.setGeometry(QtCore.QRect(10, 300, 301, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_76.setFont(font)
        self.line_76.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_76.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_76.setObjectName("line_76")
        self.label_74 = QtWidgets.QLabel(self.tab_4)
        self.label_74.setGeometry(QtCore.QRect(10, 280, 271, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_74.setFont(font)
        self.label_74.setObjectName("label_74")
        self.label_135 = QtWidgets.QLabel(self.tab_4)
        self.label_135.setGeometry(QtCore.QRect(350, 305, 211, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_135.setFont(font)
        self.label_135.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_135.setObjectName("label_135")
        self.line_77 = QtWidgets.QFrame(self.tab_4)
        self.line_77.setGeometry(QtCore.QRect(350, 325, 401, 1))
        self.line_77.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_77.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_77.setObjectName("line_77")
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab_4)
        self.groupBox_3.setGeometry(QtCore.QRect(350, 340, 266, 96))
        self.groupBox_3.setObjectName("groupBox_3")
        self.checkAutoRefractionContinous = QtWidgets.QRadioButton(self.groupBox_3)
        self.checkAutoRefractionContinous.setGeometry(QtCore.QRect(15, 45, 226, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkAutoRefractionContinous.setFont(font)
        self.checkAutoRefractionContinous.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkAutoRefractionContinous.setChecked(False)
        self.checkAutoRefractionContinous.setObjectName("checkAutoRefractionContinous")
        self.checkAutoRefractionNotTracking = QtWidgets.QRadioButton(self.groupBox_3)
        self.checkAutoRefractionNotTracking.setGeometry(QtCore.QRect(15, 65, 241, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkAutoRefractionNotTracking.setFont(font)
        self.checkAutoRefractionNotTracking.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkAutoRefractionNotTracking.setChecked(False)
        self.checkAutoRefractionNotTracking.setObjectName("checkAutoRefractionNotTracking")
        self.checkAutoRefractionNone = QtWidgets.QRadioButton(self.groupBox_3)
        self.checkAutoRefractionNone.setGeometry(QtCore.QRect(15, 25, 191, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkAutoRefractionNone.setFont(font)
        self.checkAutoRefractionNone.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkAutoRefractionNone.setChecked(True)
        self.checkAutoRefractionNone.setObjectName("checkAutoRefractionNone")
        self.line_10.raise_()
        self.line_76.raise_()
        self.line_7.raise_()
        self.line_8.raise_()
        self.line_5.raise_()
        self.line_77.raise_()
        self.le_siteElevation.raise_()
        self.label_5.raise_()
        self.le_siteLongitude.raise_()
        self.label_6.raise_()
        self.le_localSiderealTime.raise_()
        self.label_7.raise_()
        self.le_siteLatitude.raise_()
        self.label_12.raise_()
        self.label_58.raise_()
        self.label_16.raise_()
        self.le_telescopeTempDECMotor.raise_()
        self.label_55.raise_()
        self.label_64.raise_()
        self.le_windSpeed.raise_()
        self.label_32.raise_()
        self.label_73.raise_()
        self.le_temperature.raise_()
        self.label_25.raise_()
        self.label_31.raise_()
        self.label_70.raise_()
        self.label_68.raise_()
        self.label_26.raise_()
        self.le_refractionTemperature.raise_()
        self.le_humidity.raise_()
        self.le_cloudCover.raise_()
        self.label_67.raise_()
        self.btn_setRefractionParameters.raise_()
        self.le_dewPoint.raise_()
        self.label_71.raise_()
        self.le_windDirection.raise_()
        self.label_33.raise_()
        self.label_30.raise_()
        self.label_24.raise_()
        self.label_22.raise_()
        self.label_27.raise_()
        self.label_57.raise_()
        self.le_refractionPressure.raise_()
        self.le_rainRate.raise_()
        self.label_66.raise_()
        self.le_pressure.raise_()
        self.le_refractionStatus.raise_()
        self.label_13.raise_()
        self.label_72.raise_()
        self.label_69.raise_()
        self.label_78.raise_()
        self.label_82.raise_()
        self.label_86.raise_()
        self.btn_setRefractionCorrection.raise_()
        self.label_105.raise_()
        self.label_106.raise_()
        self.label_79.raise_()
        self.label_142.raise_()
        self.le_SQR.raise_()
        self.line_53.raise_()
        self.label_87.raise_()
        self.label_63.raise_()
        self.le_JulianDate.raise_()
        self.label_151.raise_()
        self.le_UTCDataExpirationDate.raise_()
        self.label_152.raise_()
        self.le_UTCDataValid.raise_()
        self.label_74.raise_()
        self.label_135.raise_()
        self.groupBox_3.raise_()
        self.mainTabWidget.addTab(self.tab_4, "")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.checkSortPoints = QtWidgets.QCheckBox(self.tab)
        self.checkSortPoints.setGeometry(QtCore.QRect(420, 260, 211, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkSortPoints.setFont(font)
        self.checkSortPoints.setObjectName("checkSortPoints")
        self.checkDeletePointsHorizonMask = QtWidgets.QCheckBox(self.tab)
        self.checkDeletePointsHorizonMask.setGeometry(QtCore.QRect(420, 230, 211, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkDeletePointsHorizonMask.setFont(font)
        self.checkDeletePointsHorizonMask.setObjectName("checkDeletePointsHorizonMask")
        self.label_122 = QtWidgets.QLabel(self.tab)
        self.label_122.setGeometry(QtCore.QRect(665, 180, 26, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_122.setFont(font)
        self.label_122.setAlignment(QtCore.Qt.AlignCenter)
        self.label_122.setWordWrap(False)
        self.label_122.setObjectName("label_122")
        self.checkUseMinimumHorizonLine = QtWidgets.QCheckBox(self.tab)
        self.checkUseMinimumHorizonLine.setGeometry(QtCore.QRect(420, 155, 211, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkUseMinimumHorizonLine.setFont(font)
        self.checkUseMinimumHorizonLine.setChecked(False)
        self.checkUseMinimumHorizonLine.setObjectName("checkUseMinimumHorizonLine")
        self.checkUseFileHorizonLine = QtWidgets.QCheckBox(self.tab)
        self.checkUseFileHorizonLine.setGeometry(QtCore.QRect(420, 125, 211, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkUseFileHorizonLine.setFont(font)
        self.checkUseFileHorizonLine.setChecked(False)
        self.checkUseFileHorizonLine.setObjectName("checkUseFileHorizonLine")
        self.le_horizonPointsFileName = QtWidgets.QLineEdit(self.tab)
        self.le_horizonPointsFileName.setEnabled(False)
        self.le_horizonPointsFileName.setGeometry(QtCore.QRect(410, 50, 231, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_horizonPointsFileName.setFont(font)
        self.le_horizonPointsFileName.setMouseTracking(False)
        self.le_horizonPointsFileName.setAcceptDrops(False)
        self.le_horizonPointsFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_horizonPointsFileName.setText("")
        self.le_horizonPointsFileName.setReadOnly(False)
        self.le_horizonPointsFileName.setObjectName("le_horizonPointsFileName")
        self.line_58 = QtWidgets.QFrame(self.tab)
        self.line_58.setGeometry(QtCore.QRect(410, 40, 336, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_58.setFont(font)
        self.line_58.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_58.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_58.setObjectName("line_58")
        self.label_201 = QtWidgets.QLabel(self.tab)
        self.label_201.setGeometry(QtCore.QRect(470, 180, 146, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_201.setFont(font)
        self.label_201.setObjectName("label_201")
        self.label_194 = QtWidgets.QLabel(self.tab)
        self.label_194.setGeometry(QtCore.QRect(410, 20, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_194.setFont(font)
        self.label_194.setObjectName("label_194")
        self.altitudeMinimumHorizon = QtWidgets.QDoubleSpinBox(self.tab)
        self.altitudeMinimumHorizon.setGeometry(QtCore.QRect(615, 180, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(True)
        font.setWeight(75)
        self.altitudeMinimumHorizon.setFont(font)
        self.altitudeMinimumHorizon.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeMinimumHorizon.setDecimals(0)
        self.altitudeMinimumHorizon.setMinimum(0.0)
        self.altitudeMinimumHorizon.setMaximum(90.0)
        self.altitudeMinimumHorizon.setSingleStep(1.0)
        self.altitudeMinimumHorizon.setProperty("value", 20.0)
        self.altitudeMinimumHorizon.setObjectName("altitudeMinimumHorizon")
        self.label_164 = QtWidgets.QLabel(self.tab)
        self.label_164.setGeometry(QtCore.QRect(335, 50, 16, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_164.setFont(font)
        self.label_164.setObjectName("label_164")
        self.label_165 = QtWidgets.QLabel(self.tab)
        self.label_165.setGeometry(QtCore.QRect(335, 195, 16, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_165.setFont(font)
        self.label_165.setObjectName("label_165")
        self.settlingTime = QtWidgets.QDoubleSpinBox(self.tab)
        self.settlingTime.setGeometry(QtCore.QRect(255, 195, 71, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.settlingTime.setFont(font)
        self.settlingTime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.settlingTime.setDecimals(0)
        self.settlingTime.setMinimum(1.0)
        self.settlingTime.setMaximum(100.0)
        self.settlingTime.setSingleStep(1.0)
        self.settlingTime.setProperty("value", 1.0)
        self.settlingTime.setObjectName("settlingTime")
        self.label_172 = QtWidgets.QLabel(self.tab)
        self.label_172.setGeometry(QtCore.QRect(20, 165, 181, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_172.setFont(font)
        self.label_172.setObjectName("label_172")
        self.checkFastDownload = QtWidgets.QCheckBox(self.tab)
        self.checkFastDownload.setGeometry(QtCore.QRect(20, 140, 291, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkFastDownload.setFont(font)
        self.checkFastDownload.setChecked(False)
        self.checkFastDownload.setObjectName("checkFastDownload")
        self.label_40 = QtWidgets.QLabel(self.tab)
        self.label_40.setGeometry(QtCore.QRect(20, 80, 131, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_40.setFont(font)
        self.label_40.setObjectName("label_40")
        self.label_163 = QtWidgets.QLabel(self.tab)
        self.label_163.setGeometry(QtCore.QRect(20, 20, 231, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_163.setFont(font)
        self.label_163.setObjectName("label_163")
        self.label_38 = QtWidgets.QLabel(self.tab)
        self.label_38.setGeometry(QtCore.QRect(20, 195, 111, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_38.setFont(font)
        self.label_38.setObjectName("label_38")
        self.checkDoSubframe = QtWidgets.QCheckBox(self.tab)
        self.checkDoSubframe.setGeometry(QtCore.QRect(20, 110, 111, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkDoSubframe.setFont(font)
        self.checkDoSubframe.setChecked(False)
        self.checkDoSubframe.setObjectName("checkDoSubframe")
        self.checkKeepImages = QtWidgets.QCheckBox(self.tab)
        self.checkKeepImages.setGeometry(QtCore.QRect(20, 225, 231, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkKeepImages.setFont(font)
        self.checkKeepImages.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkKeepImages.setChecked(False)
        self.checkKeepImages.setObjectName("checkKeepImages")
        self.line_25 = QtWidgets.QFrame(self.tab)
        self.line_25.setGeometry(QtCore.QRect(20, 40, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_25.setFont(font)
        self.line_25.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_25.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_25.setObjectName("line_25")
        self.cameraBin = QtWidgets.QDoubleSpinBox(self.tab)
        self.cameraBin.setGeometry(QtCore.QRect(255, 80, 71, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.cameraBin.setFont(font)
        self.cameraBin.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.cameraBin.setDecimals(0)
        self.cameraBin.setMinimum(1.0)
        self.cameraBin.setMaximum(9.0)
        self.cameraBin.setSingleStep(1.0)
        self.cameraBin.setProperty("value", 1.0)
        self.cameraBin.setObjectName("cameraBin")
        self.scaleSubframe = QtWidgets.QDoubleSpinBox(self.tab)
        self.scaleSubframe.setGeometry(QtCore.QRect(255, 110, 71, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.scaleSubframe.setFont(font)
        self.scaleSubframe.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.scaleSubframe.setDecimals(0)
        self.scaleSubframe.setMinimum(10.0)
        self.scaleSubframe.setMaximum(100.0)
        self.scaleSubframe.setSingleStep(10.0)
        self.scaleSubframe.setProperty("value", 100.0)
        self.scaleSubframe.setObjectName("scaleSubframe")
        self.cameraExposure = QtWidgets.QDoubleSpinBox(self.tab)
        self.cameraExposure.setGeometry(QtCore.QRect(255, 50, 71, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.cameraExposure.setFont(font)
        self.cameraExposure.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.cameraExposure.setDecimals(0)
        self.cameraExposure.setMinimum(1.0)
        self.cameraExposure.setMaximum(300.0)
        self.cameraExposure.setSingleStep(1.0)
        self.cameraExposure.setProperty("value", 1.0)
        self.cameraExposure.setObjectName("cameraExposure")
        self.label_173 = QtWidgets.QLabel(self.tab)
        self.label_173.setGeometry(QtCore.QRect(335, 110, 16, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_173.setFont(font)
        self.label_173.setObjectName("label_173")
        self.isoSetting = QtWidgets.QDoubleSpinBox(self.tab)
        self.isoSetting.setGeometry(QtCore.QRect(255, 165, 71, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.isoSetting.setFont(font)
        self.isoSetting.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.isoSetting.setDecimals(0)
        self.isoSetting.setMinimum(100.0)
        self.isoSetting.setMaximum(128000.0)
        self.isoSetting.setSingleStep(100.0)
        self.isoSetting.setProperty("value", 800.0)
        self.isoSetting.setObjectName("isoSetting")
        self.label_39 = QtWidgets.QLabel(self.tab)
        self.label_39.setGeometry(QtCore.QRect(20, 50, 161, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_39.setFont(font)
        self.label_39.setObjectName("label_39")
        self.line_41 = QtWidgets.QFrame(self.tab)
        self.line_41.setGeometry(QtCore.QRect(370, 10, 21, 431))
        self.line_41.setStyleSheet("")
        self.line_41.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_41.setLineWidth(2)
        self.line_41.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_41.setObjectName("line_41")
        self.btn_saveHorizonMaskAs = QtWidgets.QPushButton(self.tab)
        self.btn_saveHorizonMaskAs.setEnabled(True)
        self.btn_saveHorizonMaskAs.setGeometry(QtCore.QRect(570, 80, 71, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_saveHorizonMaskAs.setFont(font)
        self.btn_saveHorizonMaskAs.setStyleSheet("")
        self.btn_saveHorizonMaskAs.setObjectName("btn_saveHorizonMaskAs")
        self.btn_loadHorizonMask = QtWidgets.QPushButton(self.tab)
        self.btn_loadHorizonMask.setGeometry(QtCore.QRect(410, 80, 71, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_loadHorizonMask.setFont(font)
        self.btn_loadHorizonMask.setStyleSheet("")
        self.btn_loadHorizonMask.setObjectName("btn_loadHorizonMask")
        self.btn_saveHorizonMask = QtWidgets.QPushButton(self.tab)
        self.btn_saveHorizonMask.setEnabled(True)
        self.btn_saveHorizonMask.setGeometry(QtCore.QRect(490, 80, 71, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_saveHorizonMask.setFont(font)
        self.btn_saveHorizonMask.setStyleSheet("")
        self.btn_saveHorizonMask.setObjectName("btn_saveHorizonMask")
        self.label_99 = QtWidgets.QLabel(self.tab)
        self.label_99.setGeometry(QtCore.QRect(410, 310, 191, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_99.setFont(font)
        self.label_99.setObjectName("label_99")
        self.line_29 = QtWidgets.QFrame(self.tab)
        self.line_29.setGeometry(QtCore.QRect(410, 330, 336, 1))
        self.line_29.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_29.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_29.setObjectName("line_29")
        self.le_analyseFileName = QtWidgets.QLineEdit(self.tab)
        self.le_analyseFileName.setEnabled(False)
        self.le_analyseFileName.setGeometry(QtCore.QRect(410, 340, 226, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.le_analyseFileName.setFont(font)
        self.le_analyseFileName.setMouseTracking(False)
        self.le_analyseFileName.setAcceptDrops(False)
        self.le_analyseFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_analyseFileName.setText("")
        self.le_analyseFileName.setReadOnly(False)
        self.le_analyseFileName.setObjectName("le_analyseFileName")
        self.btn_loadAnalyseData = QtWidgets.QPushButton(self.tab)
        self.btn_loadAnalyseData.setGeometry(QtCore.QRect(410, 370, 71, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_loadAnalyseData.setFont(font)
        self.btn_loadAnalyseData.setStyleSheet("")
        self.btn_loadAnalyseData.setObjectName("btn_loadAnalyseData")
        self.label_166 = QtWidgets.QLabel(self.tab)
        self.label_166.setGeometry(QtCore.QRect(20, 295, 171, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_166.setFont(font)
        self.label_166.setObjectName("label_166")
        self.pixelSize = QtWidgets.QDoubleSpinBox(self.tab)
        self.pixelSize.setGeometry(QtCore.QRect(255, 295, 71, 26))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.pixelSize.setFont(font)
        self.pixelSize.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.pixelSize.setDecimals(1)
        self.pixelSize.setMaximum(25.0)
        self.pixelSize.setSingleStep(0.1)
        self.pixelSize.setProperty("value", 3.7)
        self.pixelSize.setObjectName("pixelSize")
        self.label_167 = QtWidgets.QLabel(self.tab)
        self.label_167.setGeometry(QtCore.QRect(330, 295, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_167.setFont(font)
        self.label_167.setObjectName("label_167")
        self.label_168 = QtWidgets.QLabel(self.tab)
        self.label_168.setGeometry(QtCore.QRect(20, 325, 156, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_168.setFont(font)
        self.label_168.setObjectName("label_168")
        self.focalLength = QtWidgets.QDoubleSpinBox(self.tab)
        self.focalLength.setGeometry(QtCore.QRect(255, 325, 71, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.focalLength.setFont(font)
        self.focalLength.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.focalLength.setDecimals(0)
        self.focalLength.setMaximum(5000.0)
        self.focalLength.setSingleStep(1.0)
        self.focalLength.setProperty("value", 582.0)
        self.focalLength.setObjectName("focalLength")
        self.label_169 = QtWidgets.QLabel(self.tab)
        self.label_169.setGeometry(QtCore.QRect(330, 325, 26, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_169.setFont(font)
        self.label_169.setObjectName("label_169")
        self.label_218 = QtWidgets.QLabel(self.tab)
        self.label_218.setGeometry(QtCore.QRect(20, 265, 231, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_218.setFont(font)
        self.label_218.setObjectName("label_218")
        self.line_36 = QtWidgets.QFrame(self.tab)
        self.line_36.setGeometry(QtCore.QRect(20, 285, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_36.setFont(font)
        self.line_36.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_36.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_36.setObjectName("line_36")
        self.astrometryDownsampling = QtWidgets.QDoubleSpinBox(self.tab)
        self.astrometryDownsampling.setGeometry(QtCore.QRect(255, 355, 71, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.astrometryDownsampling.setFont(font)
        self.astrometryDownsampling.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.astrometryDownsampling.setDecimals(0)
        self.astrometryDownsampling.setMinimum(2.0)
        self.astrometryDownsampling.setMaximum(6.0)
        self.astrometryDownsampling.setSingleStep(1.0)
        self.astrometryDownsampling.setProperty("value", 2.0)
        self.astrometryDownsampling.setObjectName("astrometryDownsampling")
        self.label_219 = QtWidgets.QLabel(self.tab)
        self.label_219.setGeometry(QtCore.QRect(20, 355, 201, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_219.setFont(font)
        self.label_219.setObjectName("label_219")
        self.label_221 = QtWidgets.QLabel(self.tab)
        self.label_221.setGeometry(QtCore.QRect(20, 385, 201, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_221.setFont(font)
        self.label_221.setObjectName("label_221")
        self.astrometryRadius = QtWidgets.QDoubleSpinBox(self.tab)
        self.astrometryRadius.setGeometry(QtCore.QRect(255, 385, 71, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.astrometryRadius.setFont(font)
        self.astrometryRadius.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.astrometryRadius.setDecimals(1)
        self.astrometryRadius.setMinimum(0.0)
        self.astrometryRadius.setMaximum(5.0)
        self.astrometryRadius.setSingleStep(0.1)
        self.astrometryRadius.setProperty("value", 3.0)
        self.astrometryRadius.setObjectName("astrometryRadius")
        self.label_222 = QtWidgets.QLabel(self.tab)
        self.label_222.setGeometry(QtCore.QRect(330, 385, 26, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_222.setFont(font)
        self.label_222.setObjectName("label_222")
        self.checkUseBlindSolving = QtWidgets.QCheckBox(self.tab)
        self.checkUseBlindSolving.setGeometry(QtCore.QRect(20, 415, 231, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkUseBlindSolving.setFont(font)
        self.checkUseBlindSolving.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkUseBlindSolving.setChecked(False)
        self.checkUseBlindSolving.setObjectName("checkUseBlindSolving")
        self.mainTabWidget.addTab(self.tab, "")
        self.tab_15 = QtWidgets.QWidget()
        self.tab_15.setObjectName("tab_15")
        self.label_116 = QtWidgets.QLabel(self.tab_15)
        self.label_116.setGeometry(QtCore.QRect(10, 120, 151, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_116.setFont(font)
        self.label_116.setObjectName("label_116")
        self.label_171 = QtWidgets.QLabel(self.tab_15)
        self.label_171.setGeometry(QtCore.QRect(170, 190, 51, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_171.setFont(font)
        self.label_171.setObjectName("label_171")
        self.label_170 = QtWidgets.QLabel(self.tab_15)
        self.label_170.setGeometry(QtCore.QRect(15, 190, 51, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_170.setFont(font)
        self.label_170.setObjectName("label_170")
        self.line_27 = QtWidgets.QFrame(self.tab_15)
        self.line_27.setGeometry(QtCore.QRect(10, 140, 271, 1))
        self.line_27.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_27.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_27.setObjectName("line_27")
        self.azimuthBase = QtWidgets.QDoubleSpinBox(self.tab_15)
        self.azimuthBase.setGeometry(QtCore.QRect(230, 190, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.azimuthBase.setFont(font)
        self.azimuthBase.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthBase.setDecimals(0)
        self.azimuthBase.setMaximum(120.0)
        self.azimuthBase.setSingleStep(1.0)
        self.azimuthBase.setProperty("value", 50.0)
        self.azimuthBase.setObjectName("azimuthBase")
        self.btn_runInitialModel = QtWidgets.QPushButton(self.tab_15)
        self.btn_runInitialModel.setGeometry(QtCore.QRect(325, 50, 201, 31))
        self.btn_runInitialModel.setObjectName("btn_runInitialModel")
        self.line_35 = QtWidgets.QFrame(self.tab_15)
        self.line_35.setGeometry(QtCore.QRect(10, 40, 266, 1))
        self.line_35.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_35.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_35.setObjectName("line_35")
        self.btn_generateInitialPoints = QtWidgets.QPushButton(self.tab_15)
        self.btn_generateInitialPoints.setGeometry(QtCore.QRect(10, 150, 271, 31))
        self.btn_generateInitialPoints.setObjectName("btn_generateInitialPoints")
        self.altitudeBase = QtWidgets.QDoubleSpinBox(self.tab_15)
        self.altitudeBase.setGeometry(QtCore.QRect(65, 190, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeBase.setFont(font)
        self.altitudeBase.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeBase.setDecimals(0)
        self.altitudeBase.setMinimum(20.0)
        self.altitudeBase.setMaximum(70.0)
        self.altitudeBase.setSingleStep(1.0)
        self.altitudeBase.setProperty("value", 50.0)
        self.altitudeBase.setObjectName("altitudeBase")
        self.btn_cancelInitialModel = QtWidgets.QPushButton(self.tab_15)
        self.btn_cancelInitialModel.setGeometry(QtCore.QRect(325, 90, 201, 31))
        self.btn_cancelInitialModel.setStyleSheet("")
        self.btn_cancelInitialModel.setObjectName("btn_cancelInitialModel")
        self.label = QtWidgets.QLabel(self.tab_15)
        self.label.setGeometry(QtCore.QRect(20, 320, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.le_alignErrorAz = QtWidgets.QLineEdit(self.tab_15)
        self.le_alignErrorAz.setEnabled(False)
        self.le_alignErrorAz.setGeometry(QtCore.QRect(140, 410, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignErrorAz.setFont(font)
        self.le_alignErrorAz.setMouseTracking(False)
        self.le_alignErrorAz.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignErrorAz.setAcceptDrops(False)
        self.le_alignErrorAz.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignErrorAz.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignErrorAz.setReadOnly(True)
        self.le_alignErrorAz.setObjectName("le_alignErrorAz")
        self.le_alignErrorPosAngle = QtWidgets.QLineEdit(self.tab_15)
        self.le_alignErrorPosAngle.setEnabled(False)
        self.le_alignErrorPosAngle.setGeometry(QtCore.QRect(410, 350, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignErrorPosAngle.setFont(font)
        self.le_alignErrorPosAngle.setMouseTracking(False)
        self.le_alignErrorPosAngle.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignErrorPosAngle.setAcceptDrops(False)
        self.le_alignErrorPosAngle.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignErrorPosAngle.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignErrorPosAngle.setReadOnly(True)
        self.le_alignErrorPosAngle.setObjectName("le_alignErrorPosAngle")
        self.le_alignKnobTurnAz = QtWidgets.QLineEdit(self.tab_15)
        self.le_alignKnobTurnAz.setEnabled(False)
        self.le_alignKnobTurnAz.setGeometry(QtCore.QRect(670, 50, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignKnobTurnAz.setFont(font)
        self.le_alignKnobTurnAz.setMouseTracking(False)
        self.le_alignKnobTurnAz.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignKnobTurnAz.setAcceptDrops(False)
        self.le_alignKnobTurnAz.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignKnobTurnAz.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignKnobTurnAz.setReadOnly(True)
        self.le_alignKnobTurnAz.setObjectName("le_alignKnobTurnAz")
        self.le_alignErrorPolar = QtWidgets.QLineEdit(self.tab_15)
        self.le_alignErrorPolar.setEnabled(False)
        self.le_alignErrorPolar.setGeometry(QtCore.QRect(140, 380, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignErrorPolar.setFont(font)
        self.le_alignErrorPolar.setMouseTracking(False)
        self.le_alignErrorPolar.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignErrorPolar.setAcceptDrops(False)
        self.le_alignErrorPolar.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignErrorPolar.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignErrorPolar.setReadOnly(True)
        self.le_alignErrorPolar.setObjectName("le_alignErrorPolar")
        self.label_14 = QtWidgets.QLabel(self.tab_15)
        self.label_14.setGeometry(QtCore.QRect(20, 350, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_14.setFont(font)
        self.label_14.setObjectName("label_14")
        self.label_153 = QtWidgets.QLabel(self.tab_15)
        self.label_153.setGeometry(QtCore.QRect(290, 380, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_153.setFont(font)
        self.label_153.setObjectName("label_153")
        self.le_alignNumberTerms = QtWidgets.QLineEdit(self.tab_15)
        self.le_alignNumberTerms.setEnabled(False)
        self.le_alignNumberTerms.setGeometry(QtCore.QRect(410, 320, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignNumberTerms.setFont(font)
        self.le_alignNumberTerms.setMouseTracking(False)
        self.le_alignNumberTerms.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignNumberTerms.setAcceptDrops(False)
        self.le_alignNumberTerms.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignNumberTerms.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignNumberTerms.setReadOnly(True)
        self.le_alignNumberTerms.setObjectName("le_alignNumberTerms")
        self.le_alignKnobTurnAlt = QtWidgets.QLineEdit(self.tab_15)
        self.le_alignKnobTurnAlt.setEnabled(False)
        self.le_alignKnobTurnAlt.setGeometry(QtCore.QRect(670, 260, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignKnobTurnAlt.setFont(font)
        self.le_alignKnobTurnAlt.setMouseTracking(False)
        self.le_alignKnobTurnAlt.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignKnobTurnAlt.setAcceptDrops(False)
        self.le_alignKnobTurnAlt.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignKnobTurnAlt.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignKnobTurnAlt.setReadOnly(True)
        self.le_alignKnobTurnAlt.setObjectName("le_alignKnobTurnAlt")
        self.label_156 = QtWidgets.QLabel(self.tab_15)
        self.label_156.setGeometry(QtCore.QRect(575, 50, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_156.setFont(font)
        self.label_156.setObjectName("label_156")
        self.label_158 = QtWidgets.QLabel(self.tab_15)
        self.label_158.setGeometry(QtCore.QRect(20, 410, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_158.setFont(font)
        self.label_158.setObjectName("label_158")
        self.le_alignNumberStars = QtWidgets.QLineEdit(self.tab_15)
        self.le_alignNumberStars.setEnabled(False)
        self.le_alignNumberStars.setGeometry(QtCore.QRect(140, 320, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignNumberStars.setFont(font)
        self.le_alignNumberStars.setMouseTracking(False)
        self.le_alignNumberStars.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignNumberStars.setAcceptDrops(False)
        self.le_alignNumberStars.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignNumberStars.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignNumberStars.setReadOnly(True)
        self.le_alignNumberStars.setObjectName("le_alignNumberStars")
        self.le_alignErrorRMS = QtWidgets.QLineEdit(self.tab_15)
        self.le_alignErrorRMS.setEnabled(False)
        self.le_alignErrorRMS.setGeometry(QtCore.QRect(140, 350, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignErrorRMS.setFont(font)
        self.le_alignErrorRMS.setMouseTracking(False)
        self.le_alignErrorRMS.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignErrorRMS.setAcceptDrops(False)
        self.le_alignErrorRMS.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignErrorRMS.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignErrorRMS.setReadOnly(True)
        self.le_alignErrorRMS.setObjectName("le_alignErrorRMS")
        self.label_155 = QtWidgets.QLabel(self.tab_15)
        self.label_155.setGeometry(QtCore.QRect(20, 280, 291, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_155.setFont(font)
        self.label_155.setObjectName("label_155")
        self.le_alignErrorOrtho = QtWidgets.QLineEdit(self.tab_15)
        self.le_alignErrorOrtho.setEnabled(False)
        self.le_alignErrorOrtho.setGeometry(QtCore.QRect(410, 380, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignErrorOrtho.setFont(font)
        self.le_alignErrorOrtho.setMouseTracking(False)
        self.le_alignErrorOrtho.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignErrorOrtho.setAcceptDrops(False)
        self.le_alignErrorOrtho.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignErrorOrtho.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignErrorOrtho.setReadOnly(True)
        self.le_alignErrorOrtho.setObjectName("le_alignErrorOrtho")
        self.label_159 = QtWidgets.QLabel(self.tab_15)
        self.label_159.setGeometry(QtCore.QRect(290, 410, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_159.setFont(font)
        self.label_159.setObjectName("label_159")
        self.label_15 = QtWidgets.QLabel(self.tab_15)
        self.label_15.setGeometry(QtCore.QRect(20, 380, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_15.setFont(font)
        self.label_15.setObjectName("label_15")
        self.le_alignErrorAlt = QtWidgets.QLineEdit(self.tab_15)
        self.le_alignErrorAlt.setEnabled(False)
        self.le_alignErrorAlt.setGeometry(QtCore.QRect(410, 410, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignErrorAlt.setFont(font)
        self.le_alignErrorAlt.setMouseTracking(False)
        self.le_alignErrorAlt.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignErrorAlt.setAcceptDrops(False)
        self.le_alignErrorAlt.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignErrorAlt.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignErrorAlt.setReadOnly(True)
        self.le_alignErrorAlt.setObjectName("le_alignErrorAlt")
        self.label_154 = QtWidgets.QLabel(self.tab_15)
        self.label_154.setGeometry(QtCore.QRect(290, 320, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_154.setFont(font)
        self.label_154.setObjectName("label_154")
        self.label_157 = QtWidgets.QLabel(self.tab_15)
        self.label_157.setGeometry(QtCore.QRect(575, 260, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_157.setFont(font)
        self.label_157.setObjectName("label_157")
        self.line_54 = QtWidgets.QFrame(self.tab_15)
        self.line_54.setGeometry(QtCore.QRect(20, 300, 506, 1))
        self.line_54.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_54.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_54.setObjectName("line_54")
        self.label_160 = QtWidgets.QLabel(self.tab_15)
        self.label_160.setGeometry(QtCore.QRect(290, 350, 71, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_160.setFont(font)
        self.label_160.setObjectName("label_160")
        self.line_63 = QtWidgets.QFrame(self.tab_15)
        self.line_63.setGeometry(QtCore.QRect(295, 10, 21, 251))
        self.line_63.setStyleSheet("")
        self.line_63.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_63.setLineWidth(2)
        self.line_63.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_63.setObjectName("line_63")
        self.btn_plateSolveSync = QtWidgets.QPushButton(self.tab_15)
        self.btn_plateSolveSync.setEnabled(True)
        self.btn_plateSolveSync.setGeometry(QtCore.QRect(325, 215, 201, 31))
        self.btn_plateSolveSync.setObjectName("btn_plateSolveSync")
        self.label_206 = QtWidgets.QLabel(self.tab_15)
        self.label_206.setGeometry(QtCore.QRect(120, 220, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_206.setFont(font)
        self.label_206.setObjectName("label_206")
        self.numberBase = QtWidgets.QDoubleSpinBox(self.tab_15)
        self.numberBase.setGeometry(QtCore.QRect(230, 220, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberBase.setFont(font)
        self.numberBase.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberBase.setDecimals(0)
        self.numberBase.setMinimum(3.0)
        self.numberBase.setMaximum(10.0)
        self.numberBase.setSingleStep(1.0)
        self.numberBase.setProperty("value", 3.0)
        self.numberBase.setObjectName("numberBase")
        self.line_64 = QtWidgets.QFrame(self.tab_15)
        self.line_64.setGeometry(QtCore.QRect(535, 10, 21, 431))
        self.line_64.setStyleSheet("")
        self.line_64.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_64.setLineWidth(2)
        self.line_64.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_64.setObjectName("line_64")
        self.picAZ = QtWidgets.QLabel(self.tab_15)
        self.picAZ.setGeometry(QtCore.QRect(590, 80, 140, 140))
        self.picAZ.setFrameShape(QtWidgets.QFrame.Box)
        self.picAZ.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picAZ.setProperty("iconpicture", True)
        self.picAZ.setObjectName("picAZ")
        self.line_62 = QtWidgets.QFrame(self.tab_15)
        self.line_62.setGeometry(QtCore.QRect(325, 40, 201, 1))
        self.line_62.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_62.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_62.setObjectName("line_62")
        self.label_145 = QtWidgets.QLabel(self.tab_15)
        self.label_145.setGeometry(QtCore.QRect(325, 20, 131, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_145.setFont(font)
        self.label_145.setObjectName("label_145")
        self.line_65 = QtWidgets.QFrame(self.tab_15)
        self.line_65.setGeometry(QtCore.QRect(565, 40, 186, 1))
        self.line_65.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_65.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_65.setObjectName("line_65")
        self.label_207 = QtWidgets.QLabel(self.tab_15)
        self.label_207.setGeometry(QtCore.QRect(565, 20, 151, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_207.setFont(font)
        self.label_207.setObjectName("label_207")
        self.picALT = QtWidgets.QLabel(self.tab_15)
        self.picALT.setGeometry(QtCore.QRect(590, 290, 140, 140))
        self.picALT.setFrameShape(QtWidgets.QFrame.Box)
        self.picALT.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picALT.setProperty("iconpicture", True)
        self.picALT.setObjectName("picALT")
        self.line_66 = QtWidgets.QFrame(self.tab_15)
        self.line_66.setGeometry(QtCore.QRect(325, 205, 201, 1))
        self.line_66.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_66.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_66.setObjectName("line_66")
        self.label_208 = QtWidgets.QLabel(self.tab_15)
        self.label_208.setGeometry(QtCore.QRect(325, 185, 131, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_208.setFont(font)
        self.label_208.setObjectName("label_208")
        self.btn_showInitialModelPoints = QtWidgets.QPushButton(self.tab_15)
        self.btn_showInitialModelPoints.setGeometry(QtCore.QRect(10, 80, 61, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_showInitialModelPoints.setFont(font)
        self.btn_showInitialModelPoints.setStyleSheet("")
        self.btn_showInitialModelPoints.setObjectName("btn_showInitialModelPoints")
        self.btn_saveInitialModelPoints = QtWidgets.QPushButton(self.tab_15)
        self.btn_saveInitialModelPoints.setEnabled(True)
        self.btn_saveInitialModelPoints.setGeometry(QtCore.QRect(150, 80, 61, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_saveInitialModelPoints.setFont(font)
        self.btn_saveInitialModelPoints.setStyleSheet("")
        self.btn_saveInitialModelPoints.setObjectName("btn_saveInitialModelPoints")
        self.btn_saveInitialModelPointsAs = QtWidgets.QPushButton(self.tab_15)
        self.btn_saveInitialModelPointsAs.setEnabled(True)
        self.btn_saveInitialModelPointsAs.setGeometry(QtCore.QRect(220, 80, 61, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_saveInitialModelPointsAs.setFont(font)
        self.btn_saveInitialModelPointsAs.setStyleSheet("")
        self.btn_saveInitialModelPointsAs.setObjectName("btn_saveInitialModelPointsAs")
        self.le_modelInitialPointsFileName = QtWidgets.QLineEdit(self.tab_15)
        self.le_modelInitialPointsFileName.setEnabled(False)
        self.le_modelInitialPointsFileName.setGeometry(QtCore.QRect(10, 50, 271, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_modelInitialPointsFileName.setFont(font)
        self.le_modelInitialPointsFileName.setMouseTracking(False)
        self.le_modelInitialPointsFileName.setAcceptDrops(False)
        self.le_modelInitialPointsFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_modelInitialPointsFileName.setText("")
        self.le_modelInitialPointsFileName.setReadOnly(False)
        self.le_modelInitialPointsFileName.setObjectName("le_modelInitialPointsFileName")
        self.label_147 = QtWidgets.QLabel(self.tab_15)
        self.label_147.setGeometry(QtCore.QRect(10, 20, 196, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_147.setFont(font)
        self.label_147.setObjectName("label_147")
        self.btn_loadInitialModelPoints = QtWidgets.QPushButton(self.tab_15)
        self.btn_loadInitialModelPoints.setGeometry(QtCore.QRect(80, 80, 61, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_loadInitialModelPoints.setFont(font)
        self.btn_loadInitialModelPoints.setStyleSheet("")
        self.btn_loadInitialModelPoints.setObjectName("btn_loadInitialModelPoints")
        self.line_54.raise_()
        self.line_27.raise_()
        self.line_35.raise_()
        self.label_116.raise_()
        self.label_171.raise_()
        self.label_170.raise_()
        self.azimuthBase.raise_()
        self.btn_runInitialModel.raise_()
        self.btn_generateInitialPoints.raise_()
        self.altitudeBase.raise_()
        self.btn_cancelInitialModel.raise_()
        self.label.raise_()
        self.le_alignErrorAz.raise_()
        self.le_alignErrorPosAngle.raise_()
        self.le_alignKnobTurnAz.raise_()
        self.le_alignErrorPolar.raise_()
        self.label_14.raise_()
        self.label_153.raise_()
        self.le_alignNumberTerms.raise_()
        self.le_alignKnobTurnAlt.raise_()
        self.label_156.raise_()
        self.label_158.raise_()
        self.le_alignNumberStars.raise_()
        self.le_alignErrorRMS.raise_()
        self.label_155.raise_()
        self.le_alignErrorOrtho.raise_()
        self.label_159.raise_()
        self.label_15.raise_()
        self.le_alignErrorAlt.raise_()
        self.label_154.raise_()
        self.label_157.raise_()
        self.label_160.raise_()
        self.line_63.raise_()
        self.btn_plateSolveSync.raise_()
        self.label_206.raise_()
        self.numberBase.raise_()
        self.line_64.raise_()
        self.picAZ.raise_()
        self.line_62.raise_()
        self.label_145.raise_()
        self.line_65.raise_()
        self.label_207.raise_()
        self.picALT.raise_()
        self.line_66.raise_()
        self.label_208.raise_()
        self.btn_showInitialModelPoints.raise_()
        self.btn_saveInitialModelPoints.raise_()
        self.btn_saveInitialModelPointsAs.raise_()
        self.le_modelInitialPointsFileName.raise_()
        self.label_147.raise_()
        self.btn_loadInitialModelPoints.raise_()
        self.mainTabWidget.addTab(self.tab_15, "")
        self.tab_6 = QtWidgets.QWidget()
        self.tab_6.setObjectName("tab_6")
        self.btn_generateGridPoints = QtWidgets.QPushButton(self.tab_6)
        self.btn_generateGridPoints.setGeometry(QtCore.QRect(10, 155, 351, 31))
        self.btn_generateGridPoints.setObjectName("btn_generateGridPoints")
        self.label_108 = QtWidgets.QLabel(self.tab_6)
        self.label_108.setGeometry(QtCore.QRect(10, 125, 171, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_108.setFont(font)
        self.label_108.setObjectName("label_108")
        self.label_179 = QtWidgets.QLabel(self.tab_6)
        self.label_179.setGeometry(QtCore.QRect(190, 195, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_179.setFont(font)
        self.label_179.setObjectName("label_179")
        self.label_178 = QtWidgets.QLabel(self.tab_6)
        self.label_178.setGeometry(QtCore.QRect(10, 195, 91, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_178.setFont(font)
        self.label_178.setObjectName("label_178")
        self.btn_generateNormalPoints = QtWidgets.QPushButton(self.tab_6)
        self.btn_generateNormalPoints.setGeometry(QtCore.QRect(135, 310, 111, 31))
        self.btn_generateNormalPoints.setObjectName("btn_generateNormalPoints")
        self.btn_generateMaxPoints = QtWidgets.QPushButton(self.tab_6)
        self.btn_generateMaxPoints.setGeometry(QtCore.QRect(10, 310, 116, 31))
        self.btn_generateMaxPoints.setObjectName("btn_generateMaxPoints")
        self.numberGridPointsRow = QtWidgets.QDoubleSpinBox(self.tab_6)
        self.numberGridPointsRow.setGeometry(QtCore.QRect(110, 195, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberGridPointsRow.setFont(font)
        self.numberGridPointsRow.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberGridPointsRow.setDecimals(0)
        self.numberGridPointsRow.setMinimum(3.0)
        self.numberGridPointsRow.setMaximum(10.0)
        self.numberGridPointsRow.setSingleStep(1.0)
        self.numberGridPointsRow.setProperty("value", 5.0)
        self.numberGridPointsRow.setObjectName("numberGridPointsRow")
        self.line_34 = QtWidgets.QFrame(self.tab_6)
        self.line_34.setGeometry(QtCore.QRect(10, 40, 351, 1))
        self.line_34.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_34.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_34.setObjectName("line_34")
        self.numberGridPointsCol = QtWidgets.QDoubleSpinBox(self.tab_6)
        self.numberGridPointsCol.setGeometry(QtCore.QRect(290, 195, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberGridPointsCol.setFont(font)
        self.numberGridPointsCol.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberGridPointsCol.setDecimals(0)
        self.numberGridPointsCol.setMinimum(5.0)
        self.numberGridPointsCol.setMaximum(15.0)
        self.numberGridPointsCol.setSingleStep(1.0)
        self.numberGridPointsCol.setProperty("value", 10.0)
        self.numberGridPointsCol.setObjectName("numberGridPointsCol")
        self.btn_runFullModel = QtWidgets.QPushButton(self.tab_6)
        self.btn_runFullModel.setGeometry(QtCore.QRect(400, 175, 351, 31))
        self.btn_runFullModel.setObjectName("btn_runFullModel")
        self.line_26 = QtWidgets.QFrame(self.tab_6)
        self.line_26.setGeometry(QtCore.QRect(400, 40, 351, 1))
        self.line_26.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_26.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_26.setObjectName("line_26")
        self.btn_cancelFullModel = QtWidgets.QPushButton(self.tab_6)
        self.btn_cancelFullModel.setGeometry(QtCore.QRect(400, 215, 351, 31))
        self.btn_cancelFullModel.setStyleSheet("")
        self.btn_cancelFullModel.setObjectName("btn_cancelFullModel")
        self.line_28 = QtWidgets.QFrame(self.tab_6)
        self.line_28.setGeometry(QtCore.QRect(10, 145, 351, 1))
        self.line_28.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_28.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_28.setObjectName("line_28")
        self.btn_generateDSOPoints = QtWidgets.QPushButton(self.tab_6)
        self.btn_generateDSOPoints.setGeometry(QtCore.QRect(400, 50, 351, 31))
        self.btn_generateDSOPoints.setObjectName("btn_generateDSOPoints")
        self.line_40 = QtWidgets.QFrame(self.tab_6)
        self.line_40.setGeometry(QtCore.QRect(370, 10, 21, 431))
        self.line_40.setStyleSheet("")
        self.line_40.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_40.setLineWidth(2)
        self.line_40.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_40.setObjectName("line_40")
        self.line_42 = QtWidgets.QFrame(self.tab_6)
        self.line_42.setGeometry(QtCore.QRect(10, 300, 351, 1))
        self.line_42.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_42.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_42.setObjectName("line_42")
        self.numberHoursDSO = QtWidgets.QDoubleSpinBox(self.tab_6)
        self.numberHoursDSO.setGeometry(QtCore.QRect(450, 90, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberHoursDSO.setFont(font)
        self.numberHoursDSO.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberHoursDSO.setDecimals(0)
        self.numberHoursDSO.setMinimum(1.0)
        self.numberHoursDSO.setMaximum(15.0)
        self.numberHoursDSO.setSingleStep(1.0)
        self.numberHoursDSO.setProperty("value", 10.0)
        self.numberHoursDSO.setObjectName("numberHoursDSO")
        self.label_181 = QtWidgets.QLabel(self.tab_6)
        self.label_181.setGeometry(QtCore.QRect(410, 90, 41, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_181.setFont(font)
        self.label_181.setObjectName("label_181")
        self.label_188 = QtWidgets.QLabel(self.tab_6)
        self.label_188.setGeometry(QtCore.QRect(520, 90, 41, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_188.setFont(font)
        self.label_188.setObjectName("label_188")
        self.numberPointsDSO = QtWidgets.QDoubleSpinBox(self.tab_6)
        self.numberPointsDSO.setGeometry(QtCore.QRect(560, 90, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberPointsDSO.setFont(font)
        self.numberPointsDSO.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberPointsDSO.setDecimals(0)
        self.numberPointsDSO.setMinimum(5.0)
        self.numberPointsDSO.setMaximum(50.0)
        self.numberPointsDSO.setSingleStep(1.0)
        self.numberPointsDSO.setProperty("value", 10.0)
        self.numberPointsDSO.setObjectName("numberPointsDSO")
        self.label_189 = QtWidgets.QLabel(self.tab_6)
        self.label_189.setGeometry(QtCore.QRect(630, 90, 61, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_189.setFont(font)
        self.label_189.setObjectName("label_189")
        self.numberHoursPreview = QtWidgets.QDoubleSpinBox(self.tab_6)
        self.numberHoursPreview.setGeometry(QtCore.QRect(690, 90, 41, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberHoursPreview.setFont(font)
        self.numberHoursPreview.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberHoursPreview.setDecimals(0)
        self.numberHoursPreview.setMinimum(-12.0)
        self.numberHoursPreview.setMaximum(12.0)
        self.numberHoursPreview.setSingleStep(1.0)
        self.numberHoursPreview.setProperty("value", 0.0)
        self.numberHoursPreview.setObjectName("numberHoursPreview")
        self.label_180 = QtWidgets.QLabel(self.tab_6)
        self.label_180.setGeometry(QtCore.QRect(190, 225, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_180.setFont(font)
        self.label_180.setObjectName("label_180")
        self.altitudeMax = QtWidgets.QDoubleSpinBox(self.tab_6)
        self.altitudeMax.setGeometry(QtCore.QRect(290, 225, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.altitudeMax.setFont(font)
        self.altitudeMax.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeMax.setDecimals(0)
        self.altitudeMax.setMinimum(50.0)
        self.altitudeMax.setMaximum(90.0)
        self.altitudeMax.setSingleStep(5.0)
        self.altitudeMax.setProperty("value", 80.0)
        self.altitudeMax.setObjectName("altitudeMax")
        self.altitudeMin = QtWidgets.QDoubleSpinBox(self.tab_6)
        self.altitudeMin.setGeometry(QtCore.QRect(110, 225, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.altitudeMin.setFont(font)
        self.altitudeMin.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeMin.setDecimals(0)
        self.altitudeMin.setMinimum(20.0)
        self.altitudeMin.setMaximum(45.0)
        self.altitudeMin.setSingleStep(5.0)
        self.altitudeMin.setProperty("value", 30.0)
        self.altitudeMin.setObjectName("altitudeMin")
        self.label_190 = QtWidgets.QLabel(self.tab_6)
        self.label_190.setGeometry(QtCore.QRect(10, 225, 91, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_190.setFont(font)
        self.label_190.setObjectName("label_190")
        self.label_137 = QtWidgets.QLabel(self.tab_6)
        self.label_137.setGeometry(QtCore.QRect(340, 195, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_137.setFont(font)
        self.label_137.setAlignment(QtCore.Qt.AlignCenter)
        self.label_137.setWordWrap(False)
        self.label_137.setObjectName("label_137")
        self.label_144 = QtWidgets.QLabel(self.tab_6)
        self.label_144.setGeometry(QtCore.QRect(340, 230, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_144.setFont(font)
        self.label_144.setAlignment(QtCore.Qt.AlignCenter)
        self.label_144.setWordWrap(False)
        self.label_144.setObjectName("label_144")
        self.label_202 = QtWidgets.QLabel(self.tab_6)
        self.label_202.setGeometry(QtCore.QRect(160, 195, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_202.setFont(font)
        self.label_202.setAlignment(QtCore.Qt.AlignCenter)
        self.label_202.setWordWrap(False)
        self.label_202.setObjectName("label_202")
        self.label_203 = QtWidgets.QLabel(self.tab_6)
        self.label_203.setGeometry(QtCore.QRect(160, 230, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_203.setFont(font)
        self.label_203.setAlignment(QtCore.Qt.AlignCenter)
        self.label_203.setWordWrap(False)
        self.label_203.setObjectName("label_203")
        self.btn_generateMinPoints = QtWidgets.QPushButton(self.tab_6)
        self.btn_generateMinPoints.setGeometry(QtCore.QRect(255, 310, 106, 31))
        self.btn_generateMinPoints.setObjectName("btn_generateMinPoints")
        self.label_204 = QtWidgets.QLabel(self.tab_6)
        self.label_204.setGeometry(QtCore.QRect(740, 90, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_204.setFont(font)
        self.label_204.setObjectName("label_204")
        self.line_38 = QtWidgets.QFrame(self.tab_6)
        self.line_38.setGeometry(QtCore.QRect(400, 165, 351, 1))
        self.line_38.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_38.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_38.setObjectName("line_38")
        self.label_138 = QtWidgets.QLabel(self.tab_6)
        self.label_138.setGeometry(QtCore.QRect(400, 145, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_138.setFont(font)
        self.label_138.setObjectName("label_138")
        self.label_113 = QtWidgets.QLabel(self.tab_6)
        self.label_113.setGeometry(QtCore.QRect(10, 280, 311, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_113.setFont(font)
        self.label_113.setObjectName("label_113")
        self.label_136 = QtWidgets.QLabel(self.tab_6)
        self.label_136.setGeometry(QtCore.QRect(400, 20, 311, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_136.setFont(font)
        self.label_136.setObjectName("label_136")
        self.label_213 = QtWidgets.QLabel(self.tab_6)
        self.label_213.setGeometry(QtCore.QRect(10, 20, 186, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_213.setFont(font)
        self.label_213.setObjectName("label_213")
        self.le_modelFullPointsFileName = QtWidgets.QLineEdit(self.tab_6)
        self.le_modelFullPointsFileName.setEnabled(False)
        self.le_modelFullPointsFileName.setGeometry(QtCore.QRect(10, 50, 351, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_modelFullPointsFileName.setFont(font)
        self.le_modelFullPointsFileName.setMouseTracking(False)
        self.le_modelFullPointsFileName.setAcceptDrops(False)
        self.le_modelFullPointsFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_modelFullPointsFileName.setText("")
        self.le_modelFullPointsFileName.setReadOnly(False)
        self.le_modelFullPointsFileName.setObjectName("le_modelFullPointsFileName")
        self.btn_showFullModelPoints = QtWidgets.QPushButton(self.tab_6)
        self.btn_showFullModelPoints.setGeometry(QtCore.QRect(10, 80, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_showFullModelPoints.setFont(font)
        self.btn_showFullModelPoints.setStyleSheet("")
        self.btn_showFullModelPoints.setObjectName("btn_showFullModelPoints")
        self.btn_saveFullModelPoints = QtWidgets.QPushButton(self.tab_6)
        self.btn_saveFullModelPoints.setEnabled(True)
        self.btn_saveFullModelPoints.setGeometry(QtCore.QRect(190, 80, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_saveFullModelPoints.setFont(font)
        self.btn_saveFullModelPoints.setStyleSheet("")
        self.btn_saveFullModelPoints.setObjectName("btn_saveFullModelPoints")
        self.btn_saveFullModelPointsAs = QtWidgets.QPushButton(self.tab_6)
        self.btn_saveFullModelPointsAs.setEnabled(True)
        self.btn_saveFullModelPointsAs.setGeometry(QtCore.QRect(280, 80, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_saveFullModelPointsAs.setFont(font)
        self.btn_saveFullModelPointsAs.setStyleSheet("")
        self.btn_saveFullModelPointsAs.setObjectName("btn_saveFullModelPointsAs")
        self.btn_loadFullModelPoints = QtWidgets.QPushButton(self.tab_6)
        self.btn_loadFullModelPoints.setGeometry(QtCore.QRect(100, 80, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_loadFullModelPoints.setFont(font)
        self.btn_loadFullModelPoints.setStyleSheet("")
        self.btn_loadFullModelPoints.setObjectName("btn_loadFullModelPoints")
        self.le_modelingTimeEstimated = QtWidgets.QLineEdit(self.tab_6)
        self.le_modelingTimeEstimated.setGeometry(QtCore.QRect(700, 345, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_modelingTimeEstimated.setFont(font)
        self.le_modelingTimeEstimated.setMouseTracking(False)
        self.le_modelingTimeEstimated.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_modelingTimeEstimated.setAcceptDrops(False)
        self.le_modelingTimeEstimated.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_modelingTimeEstimated.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_modelingTimeEstimated.setReadOnly(True)
        self.le_modelingTimeEstimated.setObjectName("le_modelingTimeEstimated")
        self.label_35 = QtWidgets.QLabel(self.tab_6)
        self.label_35.setGeometry(QtCore.QRect(580, 345, 116, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_35.setFont(font)
        self.label_35.setObjectName("label_35")
        self.bar_modelingStatusPercent = QtWidgets.QProgressBar(self.tab_6)
        self.bar_modelingStatusPercent.setGeometry(QtCore.QRect(400, 400, 351, 31))
        self.bar_modelingStatusPercent.setMaximum(1000)
        self.bar_modelingStatusPercent.setProperty("value", 0)
        self.bar_modelingStatusPercent.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.bar_modelingStatusPercent.setTextVisible(False)
        self.bar_modelingStatusPercent.setOrientation(QtCore.Qt.Horizontal)
        self.bar_modelingStatusPercent.setInvertedAppearance(False)
        self.bar_modelingStatusPercent.setTextDirection(QtWidgets.QProgressBar.TopToBottom)
        self.bar_modelingStatusPercent.setObjectName("bar_modelingStatusPercent")
        self.label_29 = QtWidgets.QLabel(self.tab_6)
        self.label_29.setGeometry(QtCore.QRect(410, 345, 81, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_29.setFont(font)
        self.label_29.setObjectName("label_29")
        self.label_216 = QtWidgets.QLabel(self.tab_6)
        self.label_216.setGeometry(QtCore.QRect(400, 315, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_216.setFont(font)
        self.label_216.setObjectName("label_216")
        self.line_45 = QtWidgets.QFrame(self.tab_6)
        self.line_45.setGeometry(QtCore.QRect(400, 335, 351, 1))
        self.line_45.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_45.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_45.setObjectName("line_45")
        self.btn_runBatchModel = QtWidgets.QPushButton(self.tab_6)
        self.btn_runBatchModel.setEnabled(True)
        self.btn_runBatchModel.setGeometry(QtCore.QRect(10, 400, 351, 31))
        self.btn_runBatchModel.setObjectName("btn_runBatchModel")
        self.line_39 = QtWidgets.QFrame(self.tab_6)
        self.line_39.setGeometry(QtCore.QRect(10, 390, 351, 1))
        self.line_39.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_39.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_39.setObjectName("line_39")
        self.label_205 = QtWidgets.QLabel(self.tab_6)
        self.label_205.setGeometry(QtCore.QRect(10, 370, 151, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_205.setFont(font)
        self.label_205.setObjectName("label_205")
        self.le_modelingTimeElapsed = QtWidgets.QLineEdit(self.tab_6)
        self.le_modelingTimeElapsed.setGeometry(QtCore.QRect(520, 345, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_modelingTimeElapsed.setFont(font)
        self.le_modelingTimeElapsed.setMouseTracking(False)
        self.le_modelingTimeElapsed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_modelingTimeElapsed.setAcceptDrops(False)
        self.le_modelingTimeElapsed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_modelingTimeElapsed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_modelingTimeElapsed.setReadOnly(True)
        self.le_modelingTimeElapsed.setObjectName("le_modelingTimeElapsed")
        self.label_42 = QtWidgets.QLabel(self.tab_6)
        self.label_42.setGeometry(QtCore.QRect(580, 370, 91, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_42.setFont(font)
        self.label_42.setObjectName("label_42")
        self.le_modelingTimeFinished = QtWidgets.QLineEdit(self.tab_6)
        self.le_modelingTimeFinished.setGeometry(QtCore.QRect(680, 370, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_modelingTimeFinished.setFont(font)
        self.le_modelingTimeFinished.setMouseTracking(False)
        self.le_modelingTimeFinished.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_modelingTimeFinished.setAcceptDrops(False)
        self.le_modelingTimeFinished.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_modelingTimeFinished.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_modelingTimeFinished.setReadOnly(True)
        self.le_modelingTimeFinished.setObjectName("le_modelingTimeFinished")
        self.le_modelingTimeActual = QtWidgets.QLineEdit(self.tab_6)
        self.le_modelingTimeActual.setGeometry(QtCore.QRect(500, 370, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_modelingTimeActual.setFont(font)
        self.le_modelingTimeActual.setMouseTracking(False)
        self.le_modelingTimeActual.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_modelingTimeActual.setAcceptDrops(False)
        self.le_modelingTimeActual.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_modelingTimeActual.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_modelingTimeActual.setReadOnly(True)
        self.le_modelingTimeActual.setObjectName("le_modelingTimeActual")
        self.label_60 = QtWidgets.QLabel(self.tab_6)
        self.label_60.setGeometry(QtCore.QRect(410, 370, 76, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_60.setFont(font)
        self.label_60.setObjectName("label_60")
        self.line_42.raise_()
        self.line_28.raise_()
        self.line_34.raise_()
        self.line_26.raise_()
        self.btn_generateGridPoints.raise_()
        self.label_108.raise_()
        self.label_179.raise_()
        self.label_178.raise_()
        self.btn_generateNormalPoints.raise_()
        self.btn_generateMaxPoints.raise_()
        self.numberGridPointsRow.raise_()
        self.numberGridPointsCol.raise_()
        self.btn_runFullModel.raise_()
        self.btn_cancelFullModel.raise_()
        self.btn_generateDSOPoints.raise_()
        self.line_40.raise_()
        self.numberHoursDSO.raise_()
        self.label_181.raise_()
        self.label_188.raise_()
        self.numberPointsDSO.raise_()
        self.label_189.raise_()
        self.numberHoursPreview.raise_()
        self.label_180.raise_()
        self.altitudeMax.raise_()
        self.altitudeMin.raise_()
        self.label_190.raise_()
        self.label_137.raise_()
        self.label_144.raise_()
        self.label_202.raise_()
        self.label_203.raise_()
        self.btn_generateMinPoints.raise_()
        self.label_204.raise_()
        self.line_38.raise_()
        self.label_138.raise_()
        self.label_113.raise_()
        self.label_136.raise_()
        self.label_213.raise_()
        self.le_modelFullPointsFileName.raise_()
        self.btn_showFullModelPoints.raise_()
        self.btn_saveFullModelPoints.raise_()
        self.btn_saveFullModelPointsAs.raise_()
        self.btn_loadFullModelPoints.raise_()
        self.le_modelingTimeEstimated.raise_()
        self.label_35.raise_()
        self.bar_modelingStatusPercent.raise_()
        self.label_29.raise_()
        self.label_216.raise_()
        self.line_45.raise_()
        self.btn_runBatchModel.raise_()
        self.line_39.raise_()
        self.label_205.raise_()
        self.le_modelingTimeElapsed.raise_()
        self.label_42.raise_()
        self.le_modelingTimeFinished.raise_()
        self.le_modelingTimeActual.raise_()
        self.label_60.raise_()
        self.mainTabWidget.addTab(self.tab_6, "")
        self.tab_8 = QtWidgets.QWidget()
        self.tab_8.setObjectName("tab_8")
        self.label_37 = QtWidgets.QLabel(self.tab_8)
        self.label_37.setGeometry(QtCore.QRect(615, 85, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_37.setFont(font)
        self.label_37.setObjectName("label_37")
        self.btn_runTargetRMSAlignment = QtWidgets.QPushButton(self.tab_8)
        self.btn_runTargetRMSAlignment.setGeometry(QtCore.QRect(475, 85, 121, 26))
        self.btn_runTargetRMSAlignment.setObjectName("btn_runTargetRMSAlignment")
        self.targetRMS = QtWidgets.QDoubleSpinBox(self.tab_8)
        self.targetRMS.setGeometry(QtCore.QRect(705, 85, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.targetRMS.setFont(font)
        self.targetRMS.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.targetRMS.setDecimals(1)
        self.targetRMS.setMaximum(100.0)
        self.targetRMS.setSingleStep(0.1)
        self.targetRMS.setProperty("value", 99.0)
        self.targetRMS.setObjectName("targetRMS")
        self.line_18 = QtWidgets.QFrame(self.tab_8)
        self.line_18.setGeometry(QtCore.QRect(475, 240, 281, 1))
        self.line_18.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_18.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_18.setObjectName("line_18")
        self.btn_deleteWorstPoint = QtWidgets.QPushButton(self.tab_8)
        self.btn_deleteWorstPoint.setGeometry(QtCore.QRect(475, 50, 121, 26))
        self.btn_deleteWorstPoint.setObjectName("btn_deleteWorstPoint")
        self.line_44 = QtWidgets.QFrame(self.tab_8)
        self.line_44.setGeometry(QtCore.QRect(475, 40, 281, 1))
        self.line_44.setAutoFillBackground(False)
        self.line_44.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_44.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_44.setObjectName("line_44")
        self.label_124 = QtWidgets.QLabel(self.tab_8)
        self.label_124.setGeometry(QtCore.QRect(475, 20, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_124.setFont(font)
        self.label_124.setObjectName("label_124")
        self.label_125 = QtWidgets.QLabel(self.tab_8)
        self.label_125.setGeometry(QtCore.QRect(475, 220, 251, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_125.setFont(font)
        self.label_125.setObjectName("label_125")
        self.line_46 = QtWidgets.QFrame(self.tab_8)
        self.line_46.setGeometry(QtCore.QRect(445, 10, 21, 431))
        self.line_46.setStyleSheet("")
        self.line_46.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_46.setLineWidth(2)
        self.line_46.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_46.setObjectName("line_46")
        self.model = QtWidgets.QWidget(self.tab_8)
        self.model.setEnabled(True)
        self.model.setGeometry(QtCore.QRect(5, 5, 446, 446))
        self.model.setAutoFillBackground(True)
        self.model.setObjectName("model")
        self.btn_cancelRunTargetRMSAlignment = QtWidgets.QPushButton(self.tab_8)
        self.btn_cancelRunTargetRMSAlignment.setGeometry(QtCore.QRect(475, 120, 121, 26))
        self.btn_cancelRunTargetRMSAlignment.setObjectName("btn_cancelRunTargetRMSAlignment")
        self.le_alignErrorRMS2 = QtWidgets.QLineEdit(self.tab_8)
        self.le_alignErrorRMS2.setEnabled(False)
        self.le_alignErrorRMS2.setGeometry(QtCore.QRect(705, 120, 36, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignErrorRMS2.setFont(font)
        self.le_alignErrorRMS2.setMouseTracking(False)
        self.le_alignErrorRMS2.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignErrorRMS2.setAcceptDrops(False)
        self.le_alignErrorRMS2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignErrorRMS2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignErrorRMS2.setReadOnly(True)
        self.le_alignErrorRMS2.setObjectName("le_alignErrorRMS2")
        self.label_75 = QtWidgets.QLabel(self.tab_8)
        self.label_75.setGeometry(QtCore.QRect(610, 120, 91, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_75.setFont(font)
        self.label_75.setObjectName("label_75")
        self.btn_clearModel = QtWidgets.QPushButton(self.tab_8)
        self.btn_clearModel.setGeometry(QtCore.QRect(475, 155, 121, 26))
        self.btn_clearModel.setObjectName("btn_clearModel")
        self.le_alignNumberStars2 = QtWidgets.QLineEdit(self.tab_8)
        self.le_alignNumberStars2.setEnabled(False)
        self.le_alignNumberStars2.setGeometry(QtCore.QRect(725, 155, 31, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignNumberStars2.setFont(font)
        self.le_alignNumberStars2.setMouseTracking(False)
        self.le_alignNumberStars2.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignNumberStars2.setAcceptDrops(False)
        self.le_alignNumberStars2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignNumberStars2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignNumberStars2.setReadOnly(True)
        self.le_alignNumberStars2.setObjectName("le_alignNumberStars2")
        self.label_76 = QtWidgets.QLabel(self.tab_8)
        self.label_76.setGeometry(QtCore.QRect(690, 155, 36, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_76.setFont(font)
        self.label_76.setObjectName("label_76")
        self.le_alignNumberTerms2 = QtWidgets.QLineEdit(self.tab_8)
        self.le_alignNumberTerms2.setEnabled(False)
        self.le_alignNumberTerms2.setGeometry(QtCore.QRect(650, 155, 31, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_alignNumberTerms2.setFont(font)
        self.le_alignNumberTerms2.setMouseTracking(False)
        self.le_alignNumberTerms2.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_alignNumberTerms2.setAcceptDrops(False)
        self.le_alignNumberTerms2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_alignNumberTerms2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_alignNumberTerms2.setReadOnly(True)
        self.le_alignNumberTerms2.setObjectName("le_alignNumberTerms2")
        self.label_209 = QtWidgets.QLabel(self.tab_8)
        self.label_209.setGeometry(QtCore.QRect(610, 155, 41, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_209.setFont(font)
        self.label_209.setObjectName("label_209")
        self.btn_reloadAlignmentModel = QtWidgets.QPushButton(self.tab_8)
        self.btn_reloadAlignmentModel.setGeometry(QtCore.QRect(610, 50, 146, 26))
        self.btn_reloadAlignmentModel.setObjectName("btn_reloadAlignmentModel")
        self.btn_saveModel = QtWidgets.QPushButton(self.tab_8)
        self.btn_saveModel.setGeometry(QtCore.QRect(675, 280, 81, 26))
        self.btn_saveModel.setObjectName("btn_saveModel")
        self.listModelName = QtWidgets.QListWidget(self.tab_8)
        self.listModelName.setGeometry(QtCore.QRect(475, 250, 191, 196))
        self.listModelName.setObjectName("listModelName")
        self.btn_loadModel = QtWidgets.QPushButton(self.tab_8)
        self.btn_loadModel.setGeometry(QtCore.QRect(675, 250, 81, 26))
        self.btn_loadModel.setObjectName("btn_loadModel")
        self.btn_deleteModel = QtWidgets.QPushButton(self.tab_8)
        self.btn_deleteModel.setGeometry(QtCore.QRect(675, 310, 81, 26))
        self.btn_deleteModel.setObjectName("btn_deleteModel")
        self.checkShowErrorValues = QtWidgets.QCheckBox(self.tab_8)
        self.checkShowErrorValues.setGeometry(QtCore.QRect(480, 190, 266, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowErrorValues.setFont(font)
        self.checkShowErrorValues.setChecked(False)
        self.checkShowErrorValues.setObjectName("checkShowErrorValues")
        self.line_18.raise_()
        self.line_44.raise_()
        self.label_37.raise_()
        self.btn_runTargetRMSAlignment.raise_()
        self.targetRMS.raise_()
        self.btn_deleteWorstPoint.raise_()
        self.label_124.raise_()
        self.label_125.raise_()
        self.line_46.raise_()
        self.model.raise_()
        self.btn_cancelRunTargetRMSAlignment.raise_()
        self.le_alignErrorRMS2.raise_()
        self.label_75.raise_()
        self.btn_clearModel.raise_()
        self.le_alignNumberStars2.raise_()
        self.label_76.raise_()
        self.le_alignNumberTerms2.raise_()
        self.label_209.raise_()
        self.btn_reloadAlignmentModel.raise_()
        self.btn_saveModel.raise_()
        self.listModelName.raise_()
        self.btn_loadModel.raise_()
        self.btn_deleteModel.raise_()
        self.checkShowErrorValues.raise_()
        self.mainTabWidget.addTab(self.tab_8, "")
        self.Analyse = QtWidgets.QWidget()
        self.Analyse.setObjectName("Analyse")
        self.label_107 = QtWidgets.QLabel(self.Analyse)
        self.label_107.setGeometry(QtCore.QRect(10, 20, 351, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_107.setFont(font)
        self.label_107.setObjectName("label_107")
        self.btn_cancelAnalyse = QtWidgets.QPushButton(self.Analyse)
        self.btn_cancelAnalyse.setEnabled(True)
        self.btn_cancelAnalyse.setGeometry(QtCore.QRect(10, 400, 351, 31))
        self.btn_cancelAnalyse.setObjectName("btn_cancelAnalyse")
        self.btn_runFlexure = QtWidgets.QPushButton(self.Analyse)
        self.btn_runFlexure.setEnabled(True)
        self.btn_runFlexure.setGeometry(QtCore.QRect(10, 130, 351, 31))
        self.btn_runFlexure.setObjectName("btn_runFlexure")
        self.line_33 = QtWidgets.QFrame(self.Analyse)
        self.line_33.setGeometry(QtCore.QRect(10, 40, 351, 1))
        self.line_33.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_33.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_33.setObjectName("line_33")
        self.label_174 = QtWidgets.QLabel(self.Analyse)
        self.label_174.setGeometry(QtCore.QRect(10, 50, 51, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_174.setFont(font)
        self.label_174.setObjectName("label_174")
        self.azimuthTimeChange = QtWidgets.QDoubleSpinBox(self.Analyse)
        self.azimuthTimeChange.setGeometry(QtCore.QRect(290, 50, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthTimeChange.setFont(font)
        self.azimuthTimeChange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthTimeChange.setDecimals(0)
        self.azimuthTimeChange.setMaximum(359.0)
        self.azimuthTimeChange.setSingleStep(1.0)
        self.azimuthTimeChange.setProperty("value", 0.0)
        self.azimuthTimeChange.setObjectName("azimuthTimeChange")
        self.label_175 = QtWidgets.QLabel(self.Analyse)
        self.label_175.setGeometry(QtCore.QRect(220, 50, 51, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_175.setFont(font)
        self.label_175.setObjectName("label_175")
        self.altitudeTimeChange = QtWidgets.QDoubleSpinBox(self.Analyse)
        self.altitudeTimeChange.setGeometry(QtCore.QRect(130, 50, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeTimeChange.setFont(font)
        self.altitudeTimeChange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeTimeChange.setDecimals(0)
        self.altitudeTimeChange.setMinimum(0.0)
        self.altitudeTimeChange.setMaximum(90.0)
        self.altitudeTimeChange.setSingleStep(1.0)
        self.altitudeTimeChange.setProperty("value", 45.0)
        self.altitudeTimeChange.setObjectName("altitudeTimeChange")
        self.label_176 = QtWidgets.QLabel(self.Analyse)
        self.label_176.setGeometry(QtCore.QRect(10, 85, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_176.setFont(font)
        self.label_176.setObjectName("label_176")
        self.numberRunsTimeChange = QtWidgets.QDoubleSpinBox(self.Analyse)
        self.numberRunsTimeChange.setGeometry(QtCore.QRect(130, 85, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberRunsTimeChange.setFont(font)
        self.numberRunsTimeChange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberRunsTimeChange.setDecimals(0)
        self.numberRunsTimeChange.setMinimum(10.0)
        self.numberRunsTimeChange.setMaximum(300.0)
        self.numberRunsTimeChange.setSingleStep(1.0)
        self.numberRunsTimeChange.setProperty("value", 30.0)
        self.numberRunsTimeChange.setObjectName("numberRunsTimeChange")
        self.label_111 = QtWidgets.QLabel(self.Analyse)
        self.label_111.setGeometry(QtCore.QRect(340, 50, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_111.setFont(font)
        self.label_111.setAlignment(QtCore.Qt.AlignCenter)
        self.label_111.setWordWrap(False)
        self.label_111.setObjectName("label_111")
        self.label_112 = QtWidgets.QLabel(self.Analyse)
        self.label_112.setGeometry(QtCore.QRect(180, 50, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_112.setFont(font)
        self.label_112.setAlignment(QtCore.Qt.AlignCenter)
        self.label_112.setWordWrap(False)
        self.label_112.setObjectName("label_112")
        self.label_177 = QtWidgets.QLabel(self.Analyse)
        self.label_177.setGeometry(QtCore.QRect(220, 85, 41, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_177.setFont(font)
        self.label_177.setObjectName("label_177")
        self.delayTimeFlexure = QtWidgets.QDoubleSpinBox(self.Analyse)
        self.delayTimeFlexure.setGeometry(QtCore.QRect(290, 85, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.delayTimeFlexure.setFont(font)
        self.delayTimeFlexure.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.delayTimeFlexure.setDecimals(0)
        self.delayTimeFlexure.setMinimum(1.0)
        self.delayTimeFlexure.setMaximum(300.0)
        self.delayTimeFlexure.setSingleStep(1.0)
        self.delayTimeFlexure.setProperty("value", 30.0)
        self.delayTimeFlexure.setObjectName("delayTimeFlexure")
        self.label_115 = QtWidgets.QLabel(self.Analyse)
        self.label_115.setGeometry(QtCore.QRect(340, 85, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_115.setFont(font)
        self.label_115.setAlignment(QtCore.Qt.AlignCenter)
        self.label_115.setWordWrap(False)
        self.label_115.setObjectName("label_115")
        self.label_118 = QtWidgets.QLabel(self.Analyse)
        self.label_118.setGeometry(QtCore.QRect(10, 180, 131, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_118.setFont(font)
        self.label_118.setObjectName("label_118")
        self.line_37 = QtWidgets.QFrame(self.Analyse)
        self.line_37.setGeometry(QtCore.QRect(10, 200, 291, 1))
        self.line_37.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_37.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_37.setObjectName("line_37")
        self.btn_runHysterese = QtWidgets.QPushButton(self.Analyse)
        self.btn_runHysterese.setEnabled(True)
        self.btn_runHysterese.setGeometry(QtCore.QRect(10, 330, 351, 31))
        self.btn_runHysterese.setObjectName("btn_runHysterese")
        self.label_182 = QtWidgets.QLabel(self.Analyse)
        self.label_182.setGeometry(QtCore.QRect(220, 245, 51, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_182.setFont(font)
        self.label_182.setObjectName("label_182")
        self.numberRunsHysterese = QtWidgets.QDoubleSpinBox(self.Analyse)
        self.numberRunsHysterese.setGeometry(QtCore.QRect(130, 280, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberRunsHysterese.setFont(font)
        self.numberRunsHysterese.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberRunsHysterese.setDecimals(0)
        self.numberRunsHysterese.setMinimum(1.0)
        self.numberRunsHysterese.setMaximum(50.0)
        self.numberRunsHysterese.setSingleStep(1.0)
        self.numberRunsHysterese.setProperty("value", 5.0)
        self.numberRunsHysterese.setObjectName("numberRunsHysterese")
        self.label_183 = QtWidgets.QLabel(self.Analyse)
        self.label_183.setGeometry(QtCore.QRect(10, 280, 91, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_183.setFont(font)
        self.label_183.setObjectName("label_183")
        self.label_184 = QtWidgets.QLabel(self.Analyse)
        self.label_184.setGeometry(QtCore.QRect(220, 280, 41, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_184.setFont(font)
        self.label_184.setObjectName("label_184")
        self.delayTimeHysterese = QtWidgets.QDoubleSpinBox(self.Analyse)
        self.delayTimeHysterese.setGeometry(QtCore.QRect(290, 280, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.delayTimeHysterese.setFont(font)
        self.delayTimeHysterese.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.delayTimeHysterese.setDecimals(0)
        self.delayTimeHysterese.setMinimum(1.0)
        self.delayTimeHysterese.setMaximum(300.0)
        self.delayTimeHysterese.setSingleStep(1.0)
        self.delayTimeHysterese.setProperty("value", 30.0)
        self.delayTimeHysterese.setObjectName("delayTimeHysterese")
        self.azimuthHysterese2 = QtWidgets.QDoubleSpinBox(self.Analyse)
        self.azimuthHysterese2.setGeometry(QtCore.QRect(290, 245, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthHysterese2.setFont(font)
        self.azimuthHysterese2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthHysterese2.setDecimals(0)
        self.azimuthHysterese2.setMaximum(359.0)
        self.azimuthHysterese2.setSingleStep(1.0)
        self.azimuthHysterese2.setProperty("value", 0.0)
        self.azimuthHysterese2.setObjectName("azimuthHysterese2")
        self.label_117 = QtWidgets.QLabel(self.Analyse)
        self.label_117.setGeometry(QtCore.QRect(340, 280, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_117.setFont(font)
        self.label_117.setAlignment(QtCore.Qt.AlignCenter)
        self.label_117.setWordWrap(False)
        self.label_117.setObjectName("label_117")
        self.label_114 = QtWidgets.QLabel(self.Analyse)
        self.label_114.setGeometry(QtCore.QRect(180, 245, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_114.setFont(font)
        self.label_114.setAlignment(QtCore.Qt.AlignCenter)
        self.label_114.setWordWrap(False)
        self.label_114.setObjectName("label_114")
        self.altitudeHysterese2 = QtWidgets.QDoubleSpinBox(self.Analyse)
        self.altitudeHysterese2.setGeometry(QtCore.QRect(130, 245, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeHysterese2.setFont(font)
        self.altitudeHysterese2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeHysterese2.setDecimals(0)
        self.altitudeHysterese2.setMinimum(0.0)
        self.altitudeHysterese2.setMaximum(90.0)
        self.altitudeHysterese2.setSingleStep(1.0)
        self.altitudeHysterese2.setProperty("value", 45.0)
        self.altitudeHysterese2.setObjectName("altitudeHysterese2")
        self.label_119 = QtWidgets.QLabel(self.Analyse)
        self.label_119.setGeometry(QtCore.QRect(340, 245, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_119.setFont(font)
        self.label_119.setAlignment(QtCore.Qt.AlignCenter)
        self.label_119.setWordWrap(False)
        self.label_119.setObjectName("label_119")
        self.label_185 = QtWidgets.QLabel(self.Analyse)
        self.label_185.setGeometry(QtCore.QRect(10, 210, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_185.setFont(font)
        self.label_185.setObjectName("label_185")
        self.label_120 = QtWidgets.QLabel(self.Analyse)
        self.label_120.setGeometry(QtCore.QRect(180, 210, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_120.setFont(font)
        self.label_120.setAlignment(QtCore.Qt.AlignCenter)
        self.label_120.setWordWrap(False)
        self.label_120.setObjectName("label_120")
        self.label_121 = QtWidgets.QLabel(self.Analyse)
        self.label_121.setGeometry(QtCore.QRect(340, 210, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_121.setFont(font)
        self.label_121.setAlignment(QtCore.Qt.AlignCenter)
        self.label_121.setWordWrap(False)
        self.label_121.setObjectName("label_121")
        self.azimuthHysterese1 = QtWidgets.QDoubleSpinBox(self.Analyse)
        self.azimuthHysterese1.setGeometry(QtCore.QRect(290, 210, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthHysterese1.setFont(font)
        self.azimuthHysterese1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthHysterese1.setDecimals(0)
        self.azimuthHysterese1.setMaximum(359.0)
        self.azimuthHysterese1.setSingleStep(1.0)
        self.azimuthHysterese1.setProperty("value", 0.0)
        self.azimuthHysterese1.setObjectName("azimuthHysterese1")
        self.altitudeHysterese1 = QtWidgets.QDoubleSpinBox(self.Analyse)
        self.altitudeHysterese1.setGeometry(QtCore.QRect(130, 210, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeHysterese1.setFont(font)
        self.altitudeHysterese1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeHysterese1.setDecimals(0)
        self.altitudeHysterese1.setMinimum(0.0)
        self.altitudeHysterese1.setMaximum(90.0)
        self.altitudeHysterese1.setSingleStep(1.0)
        self.altitudeHysterese1.setProperty("value", 45.0)
        self.altitudeHysterese1.setObjectName("altitudeHysterese1")
        self.label_186 = QtWidgets.QLabel(self.Analyse)
        self.label_186.setGeometry(QtCore.QRect(220, 210, 51, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_186.setFont(font)
        self.label_186.setObjectName("label_186")
        self.line_43 = QtWidgets.QFrame(self.Analyse)
        self.line_43.setGeometry(QtCore.QRect(10, 190, 351, 21))
        self.line_43.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_43.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_43.setObjectName("line_43")
        self.label_187 = QtWidgets.QLabel(self.Analyse)
        self.label_187.setGeometry(QtCore.QRect(10, 245, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_187.setFont(font)
        self.label_187.setObjectName("label_187")
        self.line_52 = QtWidgets.QFrame(self.Analyse)
        self.line_52.setGeometry(QtCore.QRect(370, 10, 21, 431))
        self.line_52.setStyleSheet("")
        self.line_52.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_52.setLineWidth(2)
        self.line_52.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_52.setObjectName("line_52")
        self.btn_split = QtWidgets.QPushButton(self.Analyse)
        self.btn_split.setEnabled(True)
        self.btn_split.setGeometry(QtCore.QRect(400, 40, 351, 31))
        self.btn_split.setObjectName("btn_split")
        self.line_33.raise_()
        self.line_43.raise_()
        self.line_52.raise_()
        self.label_107.raise_()
        self.btn_cancelAnalyse.raise_()
        self.btn_runFlexure.raise_()
        self.label_174.raise_()
        self.azimuthTimeChange.raise_()
        self.label_175.raise_()
        self.altitudeTimeChange.raise_()
        self.label_176.raise_()
        self.numberRunsTimeChange.raise_()
        self.label_111.raise_()
        self.label_112.raise_()
        self.label_177.raise_()
        self.delayTimeFlexure.raise_()
        self.label_115.raise_()
        self.label_118.raise_()
        self.line_37.raise_()
        self.btn_runHysterese.raise_()
        self.label_182.raise_()
        self.numberRunsHysterese.raise_()
        self.label_183.raise_()
        self.label_184.raise_()
        self.delayTimeHysterese.raise_()
        self.azimuthHysterese2.raise_()
        self.label_117.raise_()
        self.label_114.raise_()
        self.altitudeHysterese2.raise_()
        self.label_119.raise_()
        self.label_185.raise_()
        self.label_120.raise_()
        self.label_121.raise_()
        self.azimuthHysterese1.raise_()
        self.altitudeHysterese1.raise_()
        self.label_186.raise_()
        self.label_187.raise_()
        self.btn_split.raise_()
        self.mainTabWidget.addTab(self.Analyse, "")
        self.Relay = QtWidgets.QWidget()
        self.Relay.setObjectName("Relay")
        self.line_31 = QtWidgets.QFrame(self.Relay)
        self.line_31.setGeometry(QtCore.QRect(20, 40, 161, 1))
        self.line_31.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_31.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_31.setObjectName("line_31")
        self.btn_relay1 = QtWidgets.QPushButton(self.Relay)
        self.btn_relay1.setGeometry(QtCore.QRect(20, 50, 161, 31))
        self.btn_relay1.setObjectName("btn_relay1")
        self.btn_relay3 = QtWidgets.QPushButton(self.Relay)
        self.btn_relay3.setGeometry(QtCore.QRect(20, 130, 161, 31))
        self.btn_relay3.setObjectName("btn_relay3")
        self.btn_relay2 = QtWidgets.QPushButton(self.Relay)
        self.btn_relay2.setGeometry(QtCore.QRect(20, 90, 161, 31))
        self.btn_relay2.setObjectName("btn_relay2")
        self.label_96 = QtWidgets.QLabel(self.Relay)
        self.label_96.setGeometry(QtCore.QRect(20, 20, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_96.setFont(font)
        self.label_96.setObjectName("label_96")
        self.btn_relay4 = QtWidgets.QPushButton(self.Relay)
        self.btn_relay4.setGeometry(QtCore.QRect(20, 170, 161, 31))
        self.btn_relay4.setObjectName("btn_relay4")
        self.btn_relay5 = QtWidgets.QPushButton(self.Relay)
        self.btn_relay5.setGeometry(QtCore.QRect(20, 210, 161, 31))
        self.btn_relay5.setObjectName("btn_relay5")
        self.btn_relay6 = QtWidgets.QPushButton(self.Relay)
        self.btn_relay6.setGeometry(QtCore.QRect(20, 250, 161, 31))
        self.btn_relay6.setObjectName("btn_relay6")
        self.btn_relay7 = QtWidgets.QPushButton(self.Relay)
        self.btn_relay7.setGeometry(QtCore.QRect(20, 290, 161, 31))
        self.btn_relay7.setObjectName("btn_relay7")
        self.btn_relay8 = QtWidgets.QPushButton(self.Relay)
        self.btn_relay8.setGeometry(QtCore.QRect(20, 330, 161, 31))
        self.btn_relay8.setObjectName("btn_relay8")
        self.line_59 = QtWidgets.QFrame(self.Relay)
        self.line_59.setGeometry(QtCore.QRect(190, 10, 21, 431))
        self.line_59.setStyleSheet("")
        self.line_59.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_59.setLineWidth(2)
        self.line_59.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_59.setObjectName("line_59")
        self.mainTabWidget.addTab(self.Relay, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.settingsTabWidget = QtWidgets.QTabWidget(self.tab_3)
        self.settingsTabWidget.setGeometry(QtCore.QRect(5, 10, 766, 436))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.settingsTabWidget.sizePolicy().hasHeightForWidth())
        self.settingsTabWidget.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.settingsTabWidget.setFont(font)
        self.settingsTabWidget.setAutoFillBackground(True)
        self.settingsTabWidget.setStyleSheet("")
        self.settingsTabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.settingsTabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.settingsTabWidget.setElideMode(QtCore.Qt.ElideLeft)
        self.settingsTabWidget.setUsesScrollButtons(True)
        self.settingsTabWidget.setObjectName("settingsTabWidget")
        self.tab_12 = QtWidgets.QWidget()
        self.tab_12.setObjectName("tab_12")
        self.line_56 = QtWidgets.QFrame(self.tab_12)
        self.line_56.setGeometry(QtCore.QRect(370, 10, 21, 381))
        self.line_56.setStyleSheet("")
        self.line_56.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_56.setLineWidth(2)
        self.line_56.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_56.setObjectName("line_56")
        self.label_139 = QtWidgets.QLabel(self.tab_12)
        self.label_139.setGeometry(QtCore.QRect(20, 180, 201, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_139.setFont(font)
        self.label_139.setObjectName("label_139")
        self.line_23 = QtWidgets.QFrame(self.tab_12)
        self.line_23.setGeometry(QtCore.QRect(20, 200, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_23.setFont(font)
        self.line_23.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_23.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_23.setObjectName("line_23")
        self.pd_chooseEnvironment = QtWidgets.QComboBox(self.tab_12)
        self.pd_chooseEnvironment.setGeometry(QtCore.QRect(20, 210, 331, 31))
        self.pd_chooseEnvironment.setObjectName("pd_chooseEnvironment")
        self.label_140 = QtWidgets.QLabel(self.tab_12)
        self.label_140.setGeometry(QtCore.QRect(20, 260, 201, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_140.setFont(font)
        self.label_140.setObjectName("label_140")
        self.line_60 = QtWidgets.QFrame(self.tab_12)
        self.line_60.setGeometry(QtCore.QRect(20, 280, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_60.setFont(font)
        self.line_60.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_60.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_60.setObjectName("line_60")
        self.pd_chooseDome = QtWidgets.QComboBox(self.tab_12)
        self.pd_chooseDome.setGeometry(QtCore.QRect(20, 290, 331, 31))
        self.pd_chooseDome.setObjectName("pd_chooseDome")
        self.pd_chooseImaging = QtWidgets.QComboBox(self.tab_12)
        self.pd_chooseImaging.setGeometry(QtCore.QRect(20, 50, 331, 31))
        self.pd_chooseImaging.setObjectName("pd_chooseImaging")
        self.label_143 = QtWidgets.QLabel(self.tab_12)
        self.label_143.setGeometry(QtCore.QRect(20, 20, 241, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_143.setFont(font)
        self.label_143.setObjectName("label_143")
        self.line_50 = QtWidgets.QFrame(self.tab_12)
        self.line_50.setGeometry(QtCore.QRect(20, 40, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_50.setFont(font)
        self.line_50.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_50.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_50.setObjectName("line_50")
        self.pd_chooseAstrometry = QtWidgets.QComboBox(self.tab_12)
        self.pd_chooseAstrometry.setGeometry(QtCore.QRect(20, 130, 331, 31))
        self.pd_chooseAstrometry.setObjectName("pd_chooseAstrometry")
        self.line_55 = QtWidgets.QFrame(self.tab_12)
        self.line_55.setGeometry(QtCore.QRect(20, 120, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_55.setFont(font)
        self.line_55.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_55.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_55.setObjectName("line_55")
        self.label_149 = QtWidgets.QLabel(self.tab_12)
        self.label_149.setGeometry(QtCore.QRect(20, 100, 241, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_149.setFont(font)
        self.label_149.setObjectName("label_149")
        self.settingsTabWidget.addTab(self.tab_12, "")
        self.ASCOM = QtWidgets.QWidget()
        self.ASCOM.setObjectName("ASCOM")
        self.btn_setupAscomDomeDriver = QtWidgets.QPushButton(self.ASCOM)
        self.btn_setupAscomDomeDriver.setGeometry(QtCore.QRect(20, 95, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_setupAscomDomeDriver.setFont(font)
        self.btn_setupAscomDomeDriver.setObjectName("btn_setupAscomDomeDriver")
        self.line_74 = QtWidgets.QFrame(self.ASCOM)
        self.line_74.setGeometry(QtCore.QRect(370, 10, 21, 381))
        self.line_74.setStyleSheet("")
        self.line_74.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_74.setLineWidth(2)
        self.line_74.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_74.setObjectName("line_74")
        self.label_258 = QtWidgets.QLabel(self.ASCOM)
        self.label_258.setGeometry(QtCore.QRect(20, 150, 231, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_258.setFont(font)
        self.label_258.setObjectName("label_258")
        self.btn_setupAscomEnvironmentDriver = QtWidgets.QPushButton(self.ASCOM)
        self.btn_setupAscomEnvironmentDriver.setGeometry(QtCore.QRect(20, 225, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_setupAscomEnvironmentDriver.setFont(font)
        self.btn_setupAscomEnvironmentDriver.setObjectName("btn_setupAscomEnvironmentDriver")
        self.label_261 = QtWidgets.QLabel(self.ASCOM)
        self.label_261.setGeometry(QtCore.QRect(20, 20, 231, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_261.setFont(font)
        self.label_261.setObjectName("label_261")
        self.le_ascomDomeDriverName = QtWidgets.QLineEdit(self.ASCOM)
        self.le_ascomDomeDriverName.setEnabled(False)
        self.le_ascomDomeDriverName.setGeometry(QtCore.QRect(20, 70, 331, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.le_ascomDomeDriverName.setFont(font)
        self.le_ascomDomeDriverName.setMouseTracking(False)
        self.le_ascomDomeDriverName.setAcceptDrops(False)
        self.le_ascomDomeDriverName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_ascomDomeDriverName.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_ascomDomeDriverName.setReadOnly(True)
        self.le_ascomDomeDriverName.setObjectName("le_ascomDomeDriverName")
        self.label_263 = QtWidgets.QLabel(self.ASCOM)
        self.label_263.setGeometry(QtCore.QRect(20, 45, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_263.setFont(font)
        self.label_263.setObjectName("label_263")
        self.le_ascomEnvironmentDriverName = QtWidgets.QLineEdit(self.ASCOM)
        self.le_ascomEnvironmentDriverName.setEnabled(False)
        self.le_ascomEnvironmentDriverName.setGeometry(QtCore.QRect(20, 200, 331, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.le_ascomEnvironmentDriverName.setFont(font)
        self.le_ascomEnvironmentDriverName.setMouseTracking(False)
        self.le_ascomEnvironmentDriverName.setAcceptDrops(False)
        self.le_ascomEnvironmentDriverName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_ascomEnvironmentDriverName.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_ascomEnvironmentDriverName.setReadOnly(True)
        self.le_ascomEnvironmentDriverName.setObjectName("le_ascomEnvironmentDriverName")
        self.label_264 = QtWidgets.QLabel(self.ASCOM)
        self.label_264.setGeometry(QtCore.QRect(20, 175, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_264.setFont(font)
        self.label_264.setObjectName("label_264")
        self.line_81 = QtWidgets.QFrame(self.ASCOM)
        self.line_81.setGeometry(QtCore.QRect(20, 40, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_81.setFont(font)
        self.line_81.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_81.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_81.setObjectName("line_81")
        self.line_82 = QtWidgets.QFrame(self.ASCOM)
        self.line_82.setGeometry(QtCore.QRect(20, 170, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_82.setFont(font)
        self.line_82.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_82.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_82.setObjectName("line_82")
        self.label_262 = QtWidgets.QLabel(self.ASCOM)
        self.label_262.setGeometry(QtCore.QRect(405, 20, 231, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_262.setFont(font)
        self.label_262.setObjectName("label_262")
        self.line_85 = QtWidgets.QFrame(self.ASCOM)
        self.line_85.setGeometry(QtCore.QRect(405, 40, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_85.setFont(font)
        self.line_85.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_85.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_85.setObjectName("line_85")
        self.label_265 = QtWidgets.QLabel(self.ASCOM)
        self.label_265.setGeometry(QtCore.QRect(405, 45, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_265.setFont(font)
        self.label_265.setObjectName("label_265")
        self.le_pinpointCatalogue = QtWidgets.QLineEdit(self.ASCOM)
        self.le_pinpointCatalogue.setEnabled(False)
        self.le_pinpointCatalogue.setGeometry(QtCore.QRect(405, 70, 331, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.le_pinpointCatalogue.setFont(font)
        self.le_pinpointCatalogue.setMouseTracking(False)
        self.le_pinpointCatalogue.setAcceptDrops(False)
        self.le_pinpointCatalogue.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_pinpointCatalogue.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_pinpointCatalogue.setReadOnly(True)
        self.le_pinpointCatalogue.setObjectName("le_pinpointCatalogue")
        self.btn_selectCatalogue = QtWidgets.QPushButton(self.ASCOM)
        self.btn_selectCatalogue.setGeometry(QtCore.QRect(405, 95, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_selectCatalogue.setFont(font)
        self.btn_selectCatalogue.setObjectName("btn_selectCatalogue")
        self.settingsTabWidget.addTab(self.ASCOM, "")
        self.tab_14 = QtWidgets.QWidget()
        self.tab_14.setObjectName("tab_14")
        self.label_mountMAC = QtWidgets.QLabel(self.tab_14)
        self.label_mountMAC.setGeometry(QtCore.QRect(20, 65, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountMAC.setFont(font)
        self.label_mountMAC.setObjectName("label_mountMAC")
        self.label_mountIP = QtWidgets.QLabel(self.tab_14)
        self.label_mountIP.setGeometry(QtCore.QRect(20, 40, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP.setFont(font)
        self.label_mountIP.setObjectName("label_mountIP")
        self.le_mountIP = QtWidgets.QLineEdit(self.tab_14)
        self.le_mountIP.setGeometry(QtCore.QRect(100, 40, 251, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_mountIP.setFont(font)
        self.le_mountIP.setMouseTracking(False)
        self.le_mountIP.setAcceptDrops(False)
        self.le_mountIP.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_mountIP.setMaxLength(50)
        self.le_mountIP.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_mountIP.setReadOnly(False)
        self.le_mountIP.setObjectName("le_mountIP")
        self.le_mountMAC = QtWidgets.QLineEdit(self.tab_14)
        self.le_mountMAC.setGeometry(QtCore.QRect(200, 65, 151, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_mountMAC.setFont(font)
        self.le_mountMAC.setMouseTracking(False)
        self.le_mountMAC.setAcceptDrops(False)
        self.le_mountMAC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_mountMAC.setMaxLength(18)
        self.le_mountMAC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_mountMAC.setReadOnly(False)
        self.le_mountMAC.setObjectName("le_mountMAC")
        self.le_remotePort = QtWidgets.QLineEdit(self.tab_14)
        self.le_remotePort.setGeometry(QtCore.QRect(685, 220, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_remotePort.setFont(font)
        self.le_remotePort.setMouseTracking(False)
        self.le_remotePort.setAcceptDrops(False)
        self.le_remotePort.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_remotePort.setInputMask("")
        self.le_remotePort.setMaxLength(5)
        self.le_remotePort.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_remotePort.setReadOnly(False)
        self.le_remotePort.setCursorMoveStyle(QtCore.Qt.VisualMoveStyle)
        self.le_remotePort.setObjectName("le_remotePort")
        self.label_193 = QtWidgets.QLabel(self.tab_14)
        self.label_193.setGeometry(QtCore.QRect(400, 160, 171, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_193.setFont(font)
        self.label_193.setObjectName("label_193")
        self.line_49 = QtWidgets.QFrame(self.tab_14)
        self.line_49.setGeometry(QtCore.QRect(400, 180, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_49.setFont(font)
        self.line_49.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_49.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_49.setObjectName("line_49")
        self.checkEnableRemoteAccess = QtWidgets.QCheckBox(self.tab_14)
        self.checkEnableRemoteAccess.setGeometry(QtCore.QRect(400, 190, 201, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEnableRemoteAccess.setFont(font)
        self.checkEnableRemoteAccess.setChecked(False)
        self.checkEnableRemoteAccess.setObjectName("checkEnableRemoteAccess")
        self.line_80 = QtWidgets.QFrame(self.tab_14)
        self.line_80.setGeometry(QtCore.QRect(370, 10, 21, 381))
        self.line_80.setStyleSheet("")
        self.line_80.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_80.setLineWidth(2)
        self.line_80.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_80.setObjectName("line_80")
        self.label_200 = QtWidgets.QLabel(self.tab_14)
        self.label_200.setGeometry(QtCore.QRect(20, 10, 291, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_200.setFont(font)
        self.label_200.setObjectName("label_200")
        self.line_61 = QtWidgets.QFrame(self.tab_14)
        self.line_61.setGeometry(QtCore.QRect(20, 30, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_61.setFont(font)
        self.line_61.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_61.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_61.setObjectName("line_61")
        self.label_mountIP_5 = QtWidgets.QLabel(self.tab_14)
        self.label_mountIP_5.setGeometry(QtCore.QRect(400, 220, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_5.setFont(font)
        self.label_mountIP_5.setObjectName("label_mountIP_5")
        self.le_INDIStatus = QtWidgets.QLineEdit(self.tab_14)
        self.le_INDIStatus.setEnabled(False)
        self.le_INDIStatus.setGeometry(QtCore.QRect(100, 240, 251, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_INDIStatus.setFont(font)
        self.le_INDIStatus.setMouseTracking(False)
        self.le_INDIStatus.setAcceptDrops(False)
        self.le_INDIStatus.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_INDIStatus.setText("")
        self.le_INDIStatus.setMaxLength(15)
        self.le_INDIStatus.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_INDIStatus.setReadOnly(False)
        self.le_INDIStatus.setObjectName("le_INDIStatus")
        self.label_146 = QtWidgets.QLabel(self.tab_14)
        self.label_146.setGeometry(QtCore.QRect(20, 240, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_146.setFont(font)
        self.label_146.setObjectName("label_146")
        self.label_198 = QtWidgets.QLabel(self.tab_14)
        self.label_198.setGeometry(QtCore.QRect(20, 265, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_198.setFont(font)
        self.label_198.setObjectName("label_198")
        self.le_INDIEnvironment = QtWidgets.QLineEdit(self.tab_14)
        self.le_INDIEnvironment.setEnabled(False)
        self.le_INDIEnvironment.setGeometry(QtCore.QRect(100, 315, 251, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_INDIEnvironment.setFont(font)
        self.le_INDIEnvironment.setMouseTracking(False)
        self.le_INDIEnvironment.setAcceptDrops(False)
        self.le_INDIEnvironment.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_INDIEnvironment.setText("")
        self.le_INDIEnvironment.setMaxLength(15)
        self.le_INDIEnvironment.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_INDIEnvironment.setReadOnly(False)
        self.le_INDIEnvironment.setObjectName("le_INDIEnvironment")
        self.checkEnableINDI = QtWidgets.QCheckBox(self.tab_14)
        self.checkEnableINDI.setGeometry(QtCore.QRect(20, 135, 201, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEnableINDI.setFont(font)
        self.checkEnableINDI.setChecked(False)
        self.checkEnableINDI.setObjectName("checkEnableINDI")
        self.le_INDIServerPort = QtWidgets.QLineEdit(self.tab_14)
        self.le_INDIServerPort.setGeometry(QtCore.QRect(305, 215, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_INDIServerPort.setFont(font)
        self.le_INDIServerPort.setCursor(QtGui.QCursor(QtCore.Qt.SizeHorCursor))
        self.le_INDIServerPort.setMouseTracking(False)
        self.le_INDIServerPort.setAcceptDrops(False)
        self.le_INDIServerPort.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_INDIServerPort.setInputMask("")
        self.le_INDIServerPort.setMaxLength(5)
        self.le_INDIServerPort.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_INDIServerPort.setReadOnly(False)
        self.le_INDIServerPort.setObjectName("le_INDIServerPort")
        self.le_INDIServerIP = QtWidgets.QLineEdit(self.tab_14)
        self.le_INDIServerIP.setGeometry(QtCore.QRect(100, 215, 191, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_INDIServerIP.setFont(font)
        self.le_INDIServerIP.setMouseTracking(False)
        self.le_INDIServerIP.setAcceptDrops(False)
        self.le_INDIServerIP.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_INDIServerIP.setMaxLength(50)
        self.le_INDIServerIP.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_INDIServerIP.setReadOnly(False)
        self.le_INDIServerIP.setObjectName("le_INDIServerIP")
        self.label_199 = QtWidgets.QLabel(self.tab_14)
        self.label_199.setGeometry(QtCore.QRect(20, 315, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_199.setFont(font)
        self.label_199.setObjectName("label_199")
        self.label_259 = QtWidgets.QLabel(self.tab_14)
        self.label_259.setGeometry(QtCore.QRect(20, 105, 211, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_259.setFont(font)
        self.label_259.setObjectName("label_259")
        self.le_INDICCD = QtWidgets.QLineEdit(self.tab_14)
        self.le_INDICCD.setEnabled(False)
        self.le_INDICCD.setGeometry(QtCore.QRect(100, 265, 251, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_INDICCD.setFont(font)
        self.le_INDICCD.setMouseTracking(False)
        self.le_INDICCD.setAcceptDrops(False)
        self.le_INDICCD.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_INDICCD.setText("")
        self.le_INDICCD.setMaxLength(15)
        self.le_INDICCD.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_INDICCD.setReadOnly(False)
        self.le_INDICCD.setObjectName("le_INDICCD")
        self.line_78 = QtWidgets.QFrame(self.tab_14)
        self.line_78.setGeometry(QtCore.QRect(20, 125, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_78.setFont(font)
        self.line_78.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_78.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_78.setObjectName("line_78")
        self.label_mountIP_3 = QtWidgets.QLabel(self.tab_14)
        self.label_mountIP_3.setGeometry(QtCore.QRect(20, 215, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_3.setFont(font)
        self.label_mountIP_3.setObjectName("label_mountIP_3")
        self.label_211 = QtWidgets.QLabel(self.tab_14)
        self.label_211.setGeometry(QtCore.QRect(20, 340, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_211.setFont(font)
        self.label_211.setObjectName("label_211")
        self.le_INDIDome = QtWidgets.QLineEdit(self.tab_14)
        self.le_INDIDome.setEnabled(False)
        self.le_INDIDome.setGeometry(QtCore.QRect(100, 340, 251, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_INDIDome.setFont(font)
        self.le_INDIDome.setMouseTracking(False)
        self.le_INDIDome.setAcceptDrops(False)
        self.le_INDIDome.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_INDIDome.setText("")
        self.le_INDIDome.setMaxLength(15)
        self.le_INDIDome.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_INDIDome.setReadOnly(False)
        self.le_INDIDome.setObjectName("le_INDIDome")
        self.label_260 = QtWidgets.QLabel(self.tab_14)
        self.label_260.setGeometry(QtCore.QRect(400, 260, 281, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_260.setFont(font)
        self.label_260.setObjectName("label_260")
        self.line_79 = QtWidgets.QFrame(self.tab_14)
        self.line_79.setGeometry(QtCore.QRect(400, 280, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_79.setFont(font)
        self.line_79.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_79.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_79.setObjectName("line_79")
        self.label_mountIP_10 = QtWidgets.QLabel(self.tab_14)
        self.label_mountIP_10.setGeometry(QtCore.QRect(295, 215, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_10.setFont(font)
        self.label_mountIP_10.setObjectName("label_mountIP_10")
        self.line_48 = QtWidgets.QFrame(self.tab_14)
        self.line_48.setGeometry(QtCore.QRect(405, 30, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_48.setFont(font)
        self.line_48.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_48.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_48.setObjectName("line_48")
        self.label_mountIP_2 = QtWidgets.QLabel(self.tab_14)
        self.label_mountIP_2.setGeometry(QtCore.QRect(405, 70, 71, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_2.setFont(font)
        self.label_mountIP_2.setObjectName("label_mountIP_2")
        self.le_relayPassword = QtWidgets.QLineEdit(self.tab_14)
        self.le_relayPassword.setGeometry(QtCore.QRect(585, 120, 151, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_relayPassword.setFont(font)
        self.le_relayPassword.setMouseTracking(False)
        self.le_relayPassword.setAcceptDrops(False)
        self.le_relayPassword.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_relayPassword.setMaxLength(15)
        self.le_relayPassword.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_relayPassword.setReadOnly(False)
        self.le_relayPassword.setObjectName("le_relayPassword")
        self.le_relayUsername = QtWidgets.QLineEdit(self.tab_14)
        self.le_relayUsername.setGeometry(QtCore.QRect(585, 95, 151, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_relayUsername.setFont(font)
        self.le_relayUsername.setMouseTracking(False)
        self.le_relayUsername.setAcceptDrops(False)
        self.le_relayUsername.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_relayUsername.setMaxLength(15)
        self.le_relayUsername.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_relayUsername.setReadOnly(False)
        self.le_relayUsername.setObjectName("le_relayUsername")
        self.label_196 = QtWidgets.QLabel(self.tab_14)
        self.label_196.setGeometry(QtCore.QRect(405, 95, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_196.setFont(font)
        self.label_196.setObjectName("label_196")
        self.checkEnableRelay = QtWidgets.QCheckBox(self.tab_14)
        self.checkEnableRelay.setGeometry(QtCore.QRect(405, 40, 201, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEnableRelay.setFont(font)
        self.checkEnableRelay.setChecked(False)
        self.checkEnableRelay.setObjectName("checkEnableRelay")
        self.label_197 = QtWidgets.QLabel(self.tab_14)
        self.label_197.setGeometry(QtCore.QRect(405, 120, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_197.setFont(font)
        self.label_197.setObjectName("label_197")
        self.le_relayIP = QtWidgets.QLineEdit(self.tab_14)
        self.le_relayIP.setGeometry(QtCore.QRect(485, 70, 251, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_relayIP.setFont(font)
        self.le_relayIP.setMouseTracking(False)
        self.le_relayIP.setAcceptDrops(False)
        self.le_relayIP.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_relayIP.setMaxLength(50)
        self.le_relayIP.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_relayIP.setReadOnly(False)
        self.le_relayIP.setObjectName("le_relayIP")
        self.label_141 = QtWidgets.QLabel(self.tab_14)
        self.label_141.setGeometry(QtCore.QRect(405, 10, 306, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_141.setFont(font)
        self.label_141.setObjectName("label_141")
        self.le_AstrometryAPIKey = QtWidgets.QLineEdit(self.tab_14)
        self.le_AstrometryAPIKey.setGeometry(QtCore.QRect(480, 315, 161, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_AstrometryAPIKey.setFont(font)
        self.le_AstrometryAPIKey.setMouseTracking(False)
        self.le_AstrometryAPIKey.setAcceptDrops(False)
        self.le_AstrometryAPIKey.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_AstrometryAPIKey.setMaxLength(25)
        self.le_AstrometryAPIKey.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_AstrometryAPIKey.setReadOnly(False)
        self.le_AstrometryAPIKey.setObjectName("le_AstrometryAPIKey")
        self.label_mountIP_8 = QtWidgets.QLabel(self.tab_14)
        self.label_mountIP_8.setGeometry(QtCore.QRect(650, 315, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_8.setFont(font)
        self.label_mountIP_8.setObjectName("label_mountIP_8")
        self.label_mountIP_4 = QtWidgets.QLabel(self.tab_14)
        self.label_mountIP_4.setGeometry(QtCore.QRect(405, 290, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_4.setFont(font)
        self.label_mountIP_4.setObjectName("label_mountIP_4")
        self.label_mountIP_9 = QtWidgets.QLabel(self.tab_14)
        self.label_mountIP_9.setGeometry(QtCore.QRect(675, 290, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_9.setFont(font)
        self.label_mountIP_9.setObjectName("label_mountIP_9")
        self.le_astrometryTimeout = QtWidgets.QLineEdit(self.tab_14)
        self.le_astrometryTimeout.setGeometry(QtCore.QRect(700, 315, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_astrometryTimeout.setFont(font)
        self.le_astrometryTimeout.setMouseTracking(False)
        self.le_astrometryTimeout.setAcceptDrops(False)
        self.le_astrometryTimeout.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_astrometryTimeout.setMaxLength(25)
        self.le_astrometryTimeout.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_astrometryTimeout.setReadOnly(False)
        self.le_astrometryTimeout.setObjectName("le_astrometryTimeout")
        self.le_AstrometryPort = QtWidgets.QLineEdit(self.tab_14)
        self.le_AstrometryPort.setGeometry(QtCore.QRect(685, 290, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_AstrometryPort.setFont(font)
        self.le_AstrometryPort.setMouseTracking(False)
        self.le_AstrometryPort.setAcceptDrops(False)
        self.le_AstrometryPort.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_AstrometryPort.setInputMask("")
        self.le_AstrometryPort.setMaxLength(5)
        self.le_AstrometryPort.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_AstrometryPort.setReadOnly(False)
        self.le_AstrometryPort.setObjectName("le_AstrometryPort")
        self.label_mountIP_13 = QtWidgets.QLabel(self.tab_14)
        self.label_mountIP_13.setGeometry(QtCore.QRect(405, 315, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_13.setFont(font)
        self.label_mountIP_13.setObjectName("label_mountIP_13")
        self.le_AstrometryHost = QtWidgets.QLineEdit(self.tab_14)
        self.le_AstrometryHost.setGeometry(QtCore.QRect(480, 290, 191, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_AstrometryHost.setFont(font)
        self.le_AstrometryHost.setMouseTracking(False)
        self.le_AstrometryHost.setAcceptDrops(False)
        self.le_AstrometryHost.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_AstrometryHost.setMaxLength(50)
        self.le_AstrometryHost.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_AstrometryHost.setReadOnly(False)
        self.le_AstrometryHost.setObjectName("le_AstrometryHost")
        self.label_220 = QtWidgets.QLabel(self.tab_14)
        self.label_220.setGeometry(QtCore.QRect(20, 290, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_220.setFont(font)
        self.label_220.setObjectName("label_220")
        self.le_INDITelescope = QtWidgets.QLineEdit(self.tab_14)
        self.le_INDITelescope.setEnabled(False)
        self.le_INDITelescope.setGeometry(QtCore.QRect(100, 290, 251, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_INDITelescope.setFont(font)
        self.le_INDITelescope.setMouseTracking(False)
        self.le_INDITelescope.setAcceptDrops(False)
        self.le_INDITelescope.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_INDITelescope.setText("")
        self.le_INDITelescope.setMaxLength(15)
        self.le_INDITelescope.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_INDITelescope.setReadOnly(False)
        self.le_INDITelescope.setObjectName("le_INDITelescope")
        self.checkEnableINDIListening = QtWidgets.QCheckBox(self.tab_14)
        self.checkEnableINDIListening.setGeometry(QtCore.QRect(20, 160, 201, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEnableINDIListening.setFont(font)
        self.checkEnableINDIListening.setChecked(False)
        self.checkEnableINDIListening.setObjectName("checkEnableINDIListening")
        self.checkEnableINDISolving = QtWidgets.QCheckBox(self.tab_14)
        self.checkEnableINDISolving.setGeometry(QtCore.QRect(20, 185, 256, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEnableINDISolving.setFont(font)
        self.checkEnableINDISolving.setChecked(False)
        self.checkEnableINDISolving.setObjectName("checkEnableINDISolving")
        self.settingsTabWidget.addTab(self.tab_14, "")
        self.Uploads = QtWidgets.QWidget()
        self.Uploads.setObjectName("Uploads")
        self.btn_downloadEarthrotation = QtWidgets.QPushButton(self.Uploads)
        self.btn_downloadEarthrotation.setGeometry(QtCore.QRect(20, 40, 341, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_downloadEarthrotation.setFont(font)
        self.btn_downloadEarthrotation.setObjectName("btn_downloadEarthrotation")
        self.label_123 = QtWidgets.QLabel(self.Uploads)
        self.label_123.setGeometry(QtCore.QRect(20, 10, 201, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_123.setFont(font)
        self.label_123.setObjectName("label_123")
        self.line_17 = QtWidgets.QFrame(self.Uploads)
        self.line_17.setGeometry(QtCore.QRect(20, 30, 341, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_17.setFont(font)
        self.line_17.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_17.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_17.setObjectName("line_17")
        self.btn_downloadSpacestations = QtWidgets.QPushButton(self.Uploads)
        self.btn_downloadSpacestations.setGeometry(QtCore.QRect(20, 90, 341, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_downloadSpacestations.setFont(font)
        self.btn_downloadSpacestations.setObjectName("btn_downloadSpacestations")
        self.btn_downloadSatbrighest = QtWidgets.QPushButton(self.Uploads)
        self.btn_downloadSatbrighest.setGeometry(QtCore.QRect(20, 130, 341, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_downloadSatbrighest.setFont(font)
        self.btn_downloadSatbrighest.setObjectName("btn_downloadSatbrighest")
        self.btn_downloadAsteroidsMPC5000 = QtWidgets.QPushButton(self.Uploads)
        self.btn_downloadAsteroidsMPC5000.setGeometry(QtCore.QRect(20, 245, 171, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_downloadAsteroidsMPC5000.setFont(font)
        self.btn_downloadAsteroidsMPC5000.setObjectName("btn_downloadAsteroidsMPC5000")
        self.btn_downloadComets = QtWidgets.QPushButton(self.Uploads)
        self.btn_downloadComets.setGeometry(QtCore.QRect(20, 190, 341, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_downloadComets.setFont(font)
        self.btn_downloadComets.setObjectName("btn_downloadComets")
        self.checkEarthrotation = QtWidgets.QCheckBox(self.Uploads)
        self.checkEarthrotation.setGeometry(QtCore.QRect(410, 40, 301, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEarthrotation.setFont(font)
        self.checkEarthrotation.setChecked(False)
        self.checkEarthrotation.setObjectName("checkEarthrotation")
        self.checkTLE = QtWidgets.QCheckBox(self.Uploads)
        self.checkTLE.setGeometry(QtCore.QRect(410, 110, 301, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkTLE.setFont(font)
        self.checkTLE.setChecked(False)
        self.checkTLE.setObjectName("checkTLE")
        self.checkAsteroids = QtWidgets.QCheckBox(self.Uploads)
        self.checkAsteroids.setGeometry(QtCore.QRect(410, 245, 301, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkAsteroids.setFont(font)
        self.checkAsteroids.setChecked(False)
        self.checkAsteroids.setObjectName("checkAsteroids")
        self.checkComets = QtWidgets.QCheckBox(self.Uploads)
        self.checkComets.setGeometry(QtCore.QRect(410, 190, 301, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkComets.setFont(font)
        self.checkComets.setChecked(False)
        self.checkComets.setObjectName("checkComets")
        self.btn_uploadMount = QtWidgets.QPushButton(self.Uploads)
        self.btn_uploadMount.setGeometry(QtCore.QRect(400, 350, 341, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_uploadMount.setFont(font)
        self.btn_uploadMount.setObjectName("btn_uploadMount")
        self.btn_downloadAll = QtWidgets.QPushButton(self.Uploads)
        self.btn_downloadAll.setGeometry(QtCore.QRect(20, 350, 341, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_downloadAll.setFont(font)
        self.btn_downloadAll.setObjectName("btn_downloadAll")
        self.line_47 = QtWidgets.QFrame(self.Uploads)
        self.line_47.setGeometry(QtCore.QRect(370, 10, 21, 381))
        self.line_47.setStyleSheet("")
        self.line_47.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_47.setLineWidth(2)
        self.line_47.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_47.setObjectName("line_47")
        self.label_134 = QtWidgets.QLabel(self.Uploads)
        self.label_134.setGeometry(QtCore.QRect(400, 10, 171, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_134.setFont(font)
        self.label_134.setObjectName("label_134")
        self.line_19 = QtWidgets.QFrame(self.Uploads)
        self.line_19.setGeometry(QtCore.QRect(400, 30, 341, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_19.setFont(font)
        self.line_19.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_19.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_19.setObjectName("line_19")
        self.checkFilterMPC = QtWidgets.QCheckBox(self.Uploads)
        self.checkFilterMPC.setGeometry(QtCore.QRect(430, 275, 271, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkFilterMPC.setFont(font)
        self.checkFilterMPC.setChecked(False)
        self.checkFilterMPC.setObjectName("checkFilterMPC")
        self.le_filterExpressionMPC = QtWidgets.QLineEdit(self.Uploads)
        self.le_filterExpressionMPC.setGeometry(QtCore.QRect(430, 305, 281, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_filterExpressionMPC.setFont(font)
        self.le_filterExpressionMPC.setMouseTracking(False)
        self.le_filterExpressionMPC.setAcceptDrops(False)
        self.le_filterExpressionMPC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_filterExpressionMPC.setReadOnly(False)
        self.le_filterExpressionMPC.setObjectName("le_filterExpressionMPC")
        self.btn_downloadAsteroidsNEA = QtWidgets.QPushButton(self.Uploads)
        self.btn_downloadAsteroidsNEA.setGeometry(QtCore.QRect(200, 245, 161, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_downloadAsteroidsNEA.setFont(font)
        self.btn_downloadAsteroidsNEA.setObjectName("btn_downloadAsteroidsNEA")
        self.btn_downloadAsteroidsTNO = QtWidgets.QPushButton(self.Uploads)
        self.btn_downloadAsteroidsTNO.setGeometry(QtCore.QRect(200, 285, 161, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_downloadAsteroidsTNO.setFont(font)
        self.btn_downloadAsteroidsTNO.setObjectName("btn_downloadAsteroidsTNO")
        self.btn_downloadAsteroidsPHA = QtWidgets.QPushButton(self.Uploads)
        self.btn_downloadAsteroidsPHA.setGeometry(QtCore.QRect(20, 285, 171, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_downloadAsteroidsPHA.setFont(font)
        self.btn_downloadAsteroidsPHA.setObjectName("btn_downloadAsteroidsPHA")
        self.line_17.raise_()
        self.line_19.raise_()
        self.btn_downloadEarthrotation.raise_()
        self.label_123.raise_()
        self.btn_downloadSpacestations.raise_()
        self.btn_downloadSatbrighest.raise_()
        self.btn_downloadAsteroidsMPC5000.raise_()
        self.btn_downloadComets.raise_()
        self.checkEarthrotation.raise_()
        self.checkTLE.raise_()
        self.checkAsteroids.raise_()
        self.checkComets.raise_()
        self.btn_uploadMount.raise_()
        self.btn_downloadAll.raise_()
        self.line_47.raise_()
        self.label_134.raise_()
        self.checkFilterMPC.raise_()
        self.le_filterExpressionMPC.raise_()
        self.btn_downloadAsteroidsNEA.raise_()
        self.btn_downloadAsteroidsTNO.raise_()
        self.btn_downloadAsteroidsPHA.raise_()
        self.settingsTabWidget.addTab(self.Uploads, "")
        self.tab_10 = QtWidgets.QWidget()
        self.tab_10.setObjectName("tab_10")
        self.le_altParkPos3 = QtWidgets.QLineEdit(self.tab_10)
        self.le_altParkPos3.setGeometry(QtCore.QRect(315, 130, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_altParkPos3.setFont(font)
        self.le_altParkPos3.setMouseTracking(False)
        self.le_altParkPos3.setAcceptDrops(False)
        self.le_altParkPos3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_altParkPos3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_altParkPos3.setReadOnly(False)
        self.le_altParkPos3.setObjectName("le_altParkPos3")
        self.le_parkPos3Text = QtWidgets.QLineEdit(self.tab_10)
        self.le_parkPos3Text.setGeometry(QtCore.QRect(80, 130, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_parkPos3Text.setFont(font)
        self.le_parkPos3Text.setMouseTracking(False)
        self.le_parkPos3Text.setAcceptDrops(False)
        self.le_parkPos3Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_parkPos3Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_parkPos3Text.setReadOnly(False)
        self.le_parkPos3Text.setObjectName("le_parkPos3Text")
        self.le_parkPos1Text = QtWidgets.QLineEdit(self.tab_10)
        self.le_parkPos1Text.setGeometry(QtCore.QRect(80, 70, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_parkPos1Text.setFont(font)
        self.le_parkPos1Text.setMouseTracking(False)
        self.le_parkPos1Text.setAcceptDrops(False)
        self.le_parkPos1Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_parkPos1Text.setReadOnly(False)
        self.le_parkPos1Text.setObjectName("le_parkPos1Text")
        self.label_19 = QtWidgets.QLabel(self.tab_10)
        self.label_19.setGeometry(QtCore.QRect(220, 70, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_19.setFont(font)
        self.label_19.setObjectName("label_19")
        self.label_43 = QtWidgets.QLabel(self.tab_10)
        self.label_43.setGeometry(QtCore.QRect(220, 100, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_43.setFont(font)
        self.label_43.setObjectName("label_43")
        self.label_54 = QtWidgets.QLabel(self.tab_10)
        self.label_54.setGeometry(QtCore.QRect(20, 20, 256, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_54.setFont(font)
        self.label_54.setObjectName("label_54")
        self.le_azParkPos3 = QtWidgets.QLineEdit(self.tab_10)
        self.le_azParkPos3.setGeometry(QtCore.QRect(240, 130, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_azParkPos3.setFont(font)
        self.le_azParkPos3.setMouseTracking(False)
        self.le_azParkPos3.setAcceptDrops(False)
        self.le_azParkPos3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_azParkPos3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_azParkPos3.setReadOnly(False)
        self.le_azParkPos3.setObjectName("le_azParkPos3")
        self.le_parkPos2Text = QtWidgets.QLineEdit(self.tab_10)
        self.le_parkPos2Text.setGeometry(QtCore.QRect(80, 100, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_parkPos2Text.setFont(font)
        self.le_parkPos2Text.setMouseTracking(False)
        self.le_parkPos2Text.setAcceptDrops(False)
        self.le_parkPos2Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_parkPos2Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_parkPos2Text.setReadOnly(False)
        self.le_parkPos2Text.setObjectName("le_parkPos2Text")
        self.le_azParkPos1 = QtWidgets.QLineEdit(self.tab_10)
        self.le_azParkPos1.setGeometry(QtCore.QRect(240, 70, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_azParkPos1.setFont(font)
        self.le_azParkPos1.setMouseTracking(False)
        self.le_azParkPos1.setAcceptDrops(False)
        self.le_azParkPos1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_azParkPos1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_azParkPos1.setReadOnly(False)
        self.le_azParkPos1.setObjectName("le_azParkPos1")
        self.label_45 = QtWidgets.QLabel(self.tab_10)
        self.label_45.setGeometry(QtCore.QRect(290, 100, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_45.setFont(font)
        self.label_45.setObjectName("label_45")
        self.le_altParkPos4 = QtWidgets.QLineEdit(self.tab_10)
        self.le_altParkPos4.setGeometry(QtCore.QRect(315, 160, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_altParkPos4.setFont(font)
        self.le_altParkPos4.setMouseTracking(False)
        self.le_altParkPos4.setAcceptDrops(False)
        self.le_altParkPos4.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_altParkPos4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_altParkPos4.setReadOnly(False)
        self.le_altParkPos4.setObjectName("le_altParkPos4")
        self.le_altParkPos1 = QtWidgets.QLineEdit(self.tab_10)
        self.le_altParkPos1.setGeometry(QtCore.QRect(315, 70, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_altParkPos1.setFont(font)
        self.le_altParkPos1.setMouseTracking(False)
        self.le_altParkPos1.setAcceptDrops(False)
        self.le_altParkPos1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_altParkPos1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_altParkPos1.setReadOnly(False)
        self.le_altParkPos1.setObjectName("le_altParkPos1")
        self.label_2 = QtWidgets.QLabel(self.tab_10)
        self.label_2.setGeometry(QtCore.QRect(20, 70, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.le_altParkPos2 = QtWidgets.QLineEdit(self.tab_10)
        self.le_altParkPos2.setGeometry(QtCore.QRect(315, 100, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_altParkPos2.setFont(font)
        self.le_altParkPos2.setMouseTracking(False)
        self.le_altParkPos2.setAcceptDrops(False)
        self.le_altParkPos2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_altParkPos2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_altParkPos2.setReadOnly(False)
        self.le_altParkPos2.setObjectName("le_altParkPos2")
        self.label_51 = QtWidgets.QLabel(self.tab_10)
        self.label_51.setGeometry(QtCore.QRect(290, 160, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_51.setFont(font)
        self.label_51.setObjectName("label_51")
        self.label_50 = QtWidgets.QLabel(self.tab_10)
        self.label_50.setGeometry(QtCore.QRect(20, 160, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_50.setFont(font)
        self.label_50.setObjectName("label_50")
        self.label_44 = QtWidgets.QLabel(self.tab_10)
        self.label_44.setGeometry(QtCore.QRect(20, 100, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_44.setFont(font)
        self.label_44.setObjectName("label_44")
        self.le_azParkPos2 = QtWidgets.QLineEdit(self.tab_10)
        self.le_azParkPos2.setGeometry(QtCore.QRect(240, 100, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_azParkPos2.setFont(font)
        self.le_azParkPos2.setMouseTracking(False)
        self.le_azParkPos2.setAcceptDrops(False)
        self.le_azParkPos2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_azParkPos2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_azParkPos2.setReadOnly(False)
        self.le_azParkPos2.setObjectName("le_azParkPos2")
        self.label_49 = QtWidgets.QLabel(self.tab_10)
        self.label_49.setGeometry(QtCore.QRect(220, 160, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_49.setFont(font)
        self.label_49.setObjectName("label_49")
        self.label_3 = QtWidgets.QLabel(self.tab_10)
        self.label_3.setGeometry(QtCore.QRect(290, 70, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.line_4 = QtWidgets.QFrame(self.tab_10)
        self.line_4.setGeometry(QtCore.QRect(20, 40, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_4.setFont(font)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setObjectName("line_4")
        self.le_azParkPos4 = QtWidgets.QLineEdit(self.tab_10)
        self.le_azParkPos4.setGeometry(QtCore.QRect(240, 160, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_azParkPos4.setFont(font)
        self.le_azParkPos4.setMouseTracking(False)
        self.le_azParkPos4.setAcceptDrops(False)
        self.le_azParkPos4.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_azParkPos4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_azParkPos4.setReadOnly(False)
        self.le_azParkPos4.setObjectName("le_azParkPos4")
        self.label_46 = QtWidgets.QLabel(self.tab_10)
        self.label_46.setGeometry(QtCore.QRect(220, 130, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_46.setFont(font)
        self.label_46.setObjectName("label_46")
        self.label_48 = QtWidgets.QLabel(self.tab_10)
        self.label_48.setGeometry(QtCore.QRect(290, 130, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_48.setFont(font)
        self.label_48.setObjectName("label_48")
        self.le_parkPos4Text = QtWidgets.QLineEdit(self.tab_10)
        self.le_parkPos4Text.setGeometry(QtCore.QRect(80, 160, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_parkPos4Text.setFont(font)
        self.le_parkPos4Text.setMouseTracking(False)
        self.le_parkPos4Text.setAcceptDrops(False)
        self.le_parkPos4Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_parkPos4Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_parkPos4Text.setReadOnly(False)
        self.le_parkPos4Text.setObjectName("le_parkPos4Text")
        self.label_47 = QtWidgets.QLabel(self.tab_10)
        self.label_47.setGeometry(QtCore.QRect(20, 130, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_47.setFont(font)
        self.label_47.setObjectName("label_47")
        self.le_azParkPos5 = QtWidgets.QLineEdit(self.tab_10)
        self.le_azParkPos5.setGeometry(QtCore.QRect(240, 190, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_azParkPos5.setFont(font)
        self.le_azParkPos5.setMouseTracking(False)
        self.le_azParkPos5.setAcceptDrops(False)
        self.le_azParkPos5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_azParkPos5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_azParkPos5.setReadOnly(False)
        self.le_azParkPos5.setObjectName("le_azParkPos5")
        self.label_92 = QtWidgets.QLabel(self.tab_10)
        self.label_92.setGeometry(QtCore.QRect(20, 220, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_92.setFont(font)
        self.label_92.setObjectName("label_92")
        self.le_altParkPos6 = QtWidgets.QLineEdit(self.tab_10)
        self.le_altParkPos6.setGeometry(QtCore.QRect(315, 220, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_altParkPos6.setFont(font)
        self.le_altParkPos6.setMouseTracking(False)
        self.le_altParkPos6.setAcceptDrops(False)
        self.le_altParkPos6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_altParkPos6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_altParkPos6.setReadOnly(False)
        self.le_altParkPos6.setObjectName("le_altParkPos6")
        self.label_93 = QtWidgets.QLabel(self.tab_10)
        self.label_93.setGeometry(QtCore.QRect(220, 190, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_93.setFont(font)
        self.label_93.setObjectName("label_93")
        self.label_94 = QtWidgets.QLabel(self.tab_10)
        self.label_94.setGeometry(QtCore.QRect(290, 190, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_94.setFont(font)
        self.label_94.setObjectName("label_94")
        self.label_95 = QtWidgets.QLabel(self.tab_10)
        self.label_95.setGeometry(QtCore.QRect(20, 190, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_95.setFont(font)
        self.label_95.setObjectName("label_95")
        self.le_altParkPos5 = QtWidgets.QLineEdit(self.tab_10)
        self.le_altParkPos5.setGeometry(QtCore.QRect(315, 190, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_altParkPos5.setFont(font)
        self.le_altParkPos5.setMouseTracking(False)
        self.le_altParkPos5.setAcceptDrops(False)
        self.le_altParkPos5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_altParkPos5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_altParkPos5.setReadOnly(False)
        self.le_altParkPos5.setObjectName("le_altParkPos5")
        self.le_parkPos6Text = QtWidgets.QLineEdit(self.tab_10)
        self.le_parkPos6Text.setGeometry(QtCore.QRect(80, 220, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_parkPos6Text.setFont(font)
        self.le_parkPos6Text.setMouseTracking(False)
        self.le_parkPos6Text.setAcceptDrops(False)
        self.le_parkPos6Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_parkPos6Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_parkPos6Text.setReadOnly(False)
        self.le_parkPos6Text.setObjectName("le_parkPos6Text")
        self.le_parkPos5Text = QtWidgets.QLineEdit(self.tab_10)
        self.le_parkPos5Text.setGeometry(QtCore.QRect(80, 190, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_parkPos5Text.setFont(font)
        self.le_parkPos5Text.setMouseTracking(False)
        self.le_parkPos5Text.setAcceptDrops(False)
        self.le_parkPos5Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_parkPos5Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_parkPos5Text.setReadOnly(False)
        self.le_parkPos5Text.setObjectName("le_parkPos5Text")
        self.label_131 = QtWidgets.QLabel(self.tab_10)
        self.label_131.setGeometry(QtCore.QRect(220, 220, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_131.setFont(font)
        self.label_131.setObjectName("label_131")
        self.label_132 = QtWidgets.QLabel(self.tab_10)
        self.label_132.setGeometry(QtCore.QRect(290, 220, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_132.setFont(font)
        self.label_132.setObjectName("label_132")
        self.le_azParkPos6 = QtWidgets.QLineEdit(self.tab_10)
        self.le_azParkPos6.setGeometry(QtCore.QRect(240, 220, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_azParkPos6.setFont(font)
        self.le_azParkPos6.setMouseTracking(False)
        self.le_azParkPos6.setAcceptDrops(False)
        self.le_azParkPos6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_azParkPos6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_azParkPos6.setReadOnly(False)
        self.le_azParkPos6.setObjectName("le_azParkPos6")
        self.line_83 = QtWidgets.QFrame(self.tab_10)
        self.line_83.setGeometry(QtCore.QRect(370, 10, 21, 381))
        self.line_83.setStyleSheet("")
        self.line_83.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_83.setLineWidth(2)
        self.line_83.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_83.setObjectName("line_83")
        self.label_97 = QtWidgets.QLabel(self.tab_10)
        self.label_97.setGeometry(QtCore.QRect(80, 50, 51, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_97.setFont(font)
        self.label_97.setObjectName("label_97")
        self.relay2Text = QtWidgets.QLineEdit(self.tab_10)
        self.relay2Text.setGeometry(QtCore.QRect(450, 100, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relay2Text.setFont(font)
        self.relay2Text.setMouseTracking(False)
        self.relay2Text.setAcceptDrops(False)
        self.relay2Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relay2Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relay2Text.setReadOnly(False)
        self.relay2Text.setObjectName("relay2Text")
        self.relay2Function = QtWidgets.QComboBox(self.tab_10)
        self.relay2Function.setGeometry(QtCore.QRect(570, 100, 161, 21))
        self.relay2Function.setObjectName("relay2Function")
        self.relay7Function = QtWidgets.QComboBox(self.tab_10)
        self.relay7Function.setGeometry(QtCore.QRect(570, 250, 161, 21))
        self.relay7Function.setObjectName("relay7Function")
        self.label_192 = QtWidgets.QLabel(self.tab_10)
        self.label_192.setGeometry(QtCore.QRect(410, 280, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_192.setFont(font)
        self.label_192.setObjectName("label_192")
        self.relay1Text = QtWidgets.QLineEdit(self.tab_10)
        self.relay1Text.setGeometry(QtCore.QRect(450, 70, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relay1Text.setFont(font)
        self.relay1Text.setMouseTracking(False)
        self.relay1Text.setAcceptDrops(False)
        self.relay1Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relay1Text.setReadOnly(False)
        self.relay1Text.setObjectName("relay1Text")
        self.label_161 = QtWidgets.QLabel(self.tab_10)
        self.label_161.setGeometry(QtCore.QRect(410, 220, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_161.setFont(font)
        self.label_161.setObjectName("label_161")
        self.label_195 = QtWidgets.QLabel(self.tab_10)
        self.label_195.setGeometry(QtCore.QRect(570, 50, 51, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_195.setFont(font)
        self.label_195.setObjectName("label_195")
        self.relay6Text = QtWidgets.QLineEdit(self.tab_10)
        self.relay6Text.setGeometry(QtCore.QRect(450, 220, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relay6Text.setFont(font)
        self.relay6Text.setMouseTracking(False)
        self.relay6Text.setAcceptDrops(False)
        self.relay6Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relay6Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relay6Text.setReadOnly(False)
        self.relay6Text.setObjectName("relay6Text")
        self.relay1Function = QtWidgets.QComboBox(self.tab_10)
        self.relay1Function.setGeometry(QtCore.QRect(570, 70, 161, 21))
        self.relay1Function.setObjectName("relay1Function")
        self.relay6Function = QtWidgets.QComboBox(self.tab_10)
        self.relay6Function.setGeometry(QtCore.QRect(570, 220, 161, 21))
        self.relay6Function.setObjectName("relay6Function")
        self.relay5Function = QtWidgets.QComboBox(self.tab_10)
        self.relay5Function.setGeometry(QtCore.QRect(570, 190, 161, 21))
        self.relay5Function.setObjectName("relay5Function")
        self.relay4Text = QtWidgets.QLineEdit(self.tab_10)
        self.relay4Text.setGeometry(QtCore.QRect(450, 160, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relay4Text.setFont(font)
        self.relay4Text.setMouseTracking(False)
        self.relay4Text.setAcceptDrops(False)
        self.relay4Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relay4Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relay4Text.setReadOnly(False)
        self.relay4Text.setObjectName("relay4Text")
        self.label_53 = QtWidgets.QLabel(self.tab_10)
        self.label_53.setGeometry(QtCore.QRect(400, 50, 51, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_53.setFont(font)
        self.label_53.setObjectName("label_53")
        self.label_36 = QtWidgets.QLabel(self.tab_10)
        self.label_36.setGeometry(QtCore.QRect(410, 70, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_36.setFont(font)
        self.label_36.setObjectName("label_36")
        self.relay4Function = QtWidgets.QComboBox(self.tab_10)
        self.relay4Function.setGeometry(QtCore.QRect(570, 160, 161, 21))
        self.relay4Function.setObjectName("relay4Function")
        self.relay3Function = QtWidgets.QComboBox(self.tab_10)
        self.relay3Function.setGeometry(QtCore.QRect(570, 130, 161, 21))
        self.relay3Function.setObjectName("relay3Function")
        self.label_191 = QtWidgets.QLabel(self.tab_10)
        self.label_191.setGeometry(QtCore.QRect(410, 250, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_191.setFont(font)
        self.label_191.setObjectName("label_191")
        self.line_22 = QtWidgets.QFrame(self.tab_10)
        self.line_22.setGeometry(QtCore.QRect(400, 40, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_22.setFont(font)
        self.line_22.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_22.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_22.setObjectName("line_22")
        self.relay8Text = QtWidgets.QLineEdit(self.tab_10)
        self.relay8Text.setGeometry(QtCore.QRect(450, 280, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relay8Text.setFont(font)
        self.relay8Text.setMouseTracking(False)
        self.relay8Text.setAcceptDrops(False)
        self.relay8Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relay8Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relay8Text.setReadOnly(False)
        self.relay8Text.setObjectName("relay8Text")
        self.relay3Text = QtWidgets.QLineEdit(self.tab_10)
        self.relay3Text.setGeometry(QtCore.QRect(450, 130, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relay3Text.setFont(font)
        self.relay3Text.setMouseTracking(False)
        self.relay3Text.setAcceptDrops(False)
        self.relay3Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relay3Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relay3Text.setReadOnly(False)
        self.relay3Text.setObjectName("relay3Text")
        self.label_150 = QtWidgets.QLabel(self.tab_10)
        self.label_150.setGeometry(QtCore.QRect(410, 160, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_150.setFont(font)
        self.label_150.setObjectName("label_150")
        self.label_162 = QtWidgets.QLabel(self.tab_10)
        self.label_162.setGeometry(QtCore.QRect(410, 100, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_162.setFont(font)
        self.label_162.setObjectName("label_162")
        self.label_41 = QtWidgets.QLabel(self.tab_10)
        self.label_41.setGeometry(QtCore.QRect(450, 50, 51, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_41.setFont(font)
        self.label_41.setObjectName("label_41")
        self.relay7Text = QtWidgets.QLineEdit(self.tab_10)
        self.relay7Text.setGeometry(QtCore.QRect(450, 250, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relay7Text.setFont(font)
        self.relay7Text.setMouseTracking(False)
        self.relay7Text.setAcceptDrops(False)
        self.relay7Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relay7Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relay7Text.setReadOnly(False)
        self.relay7Text.setObjectName("relay7Text")
        self.label_101 = QtWidgets.QLabel(self.tab_10)
        self.label_101.setGeometry(QtCore.QRect(410, 130, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_101.setFont(font)
        self.label_101.setObjectName("label_101")
        self.label_148 = QtWidgets.QLabel(self.tab_10)
        self.label_148.setGeometry(QtCore.QRect(410, 190, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_148.setFont(font)
        self.label_148.setObjectName("label_148")
        self.relay5Text = QtWidgets.QLineEdit(self.tab_10)
        self.relay5Text.setGeometry(QtCore.QRect(450, 190, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relay5Text.setFont(font)
        self.relay5Text.setMouseTracking(False)
        self.relay5Text.setAcceptDrops(False)
        self.relay5Text.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relay5Text.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relay5Text.setReadOnly(False)
        self.relay5Text.setObjectName("relay5Text")
        self.relay8Function = QtWidgets.QComboBox(self.tab_10)
        self.relay8Function.setGeometry(QtCore.QRect(570, 280, 161, 21))
        self.relay8Function.setObjectName("relay8Function")
        self.label_103 = QtWidgets.QLabel(self.tab_10)
        self.label_103.setGeometry(QtCore.QRect(400, 20, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_103.setFont(font)
        self.label_103.setObjectName("label_103")
        self.label_103.raise_()
        self.le_altParkPos3.raise_()
        self.le_parkPos3Text.raise_()
        self.le_parkPos1Text.raise_()
        self.label_19.raise_()
        self.label_43.raise_()
        self.label_54.raise_()
        self.le_azParkPos3.raise_()
        self.le_parkPos2Text.raise_()
        self.le_azParkPos1.raise_()
        self.label_45.raise_()
        self.le_altParkPos4.raise_()
        self.le_altParkPos1.raise_()
        self.label_2.raise_()
        self.le_altParkPos2.raise_()
        self.label_51.raise_()
        self.label_50.raise_()
        self.label_44.raise_()
        self.le_azParkPos2.raise_()
        self.label_49.raise_()
        self.label_3.raise_()
        self.le_azParkPos4.raise_()
        self.label_46.raise_()
        self.label_48.raise_()
        self.le_parkPos4Text.raise_()
        self.label_47.raise_()
        self.le_azParkPos5.raise_()
        self.label_92.raise_()
        self.le_altParkPos6.raise_()
        self.label_93.raise_()
        self.label_94.raise_()
        self.label_95.raise_()
        self.le_altParkPos5.raise_()
        self.le_parkPos6Text.raise_()
        self.le_parkPos5Text.raise_()
        self.label_131.raise_()
        self.label_132.raise_()
        self.le_azParkPos6.raise_()
        self.line_83.raise_()
        self.label_97.raise_()
        self.relay2Text.raise_()
        self.relay2Function.raise_()
        self.relay7Function.raise_()
        self.label_192.raise_()
        self.relay1Text.raise_()
        self.label_161.raise_()
        self.label_195.raise_()
        self.relay6Text.raise_()
        self.relay1Function.raise_()
        self.relay6Function.raise_()
        self.relay5Function.raise_()
        self.relay4Text.raise_()
        self.label_53.raise_()
        self.label_36.raise_()
        self.relay4Function.raise_()
        self.relay3Function.raise_()
        self.label_191.raise_()
        self.relay8Text.raise_()
        self.relay3Text.raise_()
        self.label_150.raise_()
        self.label_162.raise_()
        self.label_41.raise_()
        self.relay7Text.raise_()
        self.label_101.raise_()
        self.label_148.raise_()
        self.relay5Text.raise_()
        self.relay8Function.raise_()
        self.line_4.raise_()
        self.line_22.raise_()
        self.settingsTabWidget.addTab(self.tab_10, "")
        self.tab_13 = QtWidgets.QWidget()
        self.tab_13.setObjectName("tab_13")
        self.soundMountSlewFinished = QtWidgets.QComboBox(self.tab_13)
        self.soundMountSlewFinished.setGeometry(QtCore.QRect(190, 50, 161, 21))
        self.soundMountSlewFinished.setObjectName("soundMountSlewFinished")
        self.label_210 = QtWidgets.QLabel(self.tab_13)
        self.label_210.setGeometry(QtCore.QRect(20, 20, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_210.setFont(font)
        self.label_210.setObjectName("label_210")
        self.line_24 = QtWidgets.QFrame(self.tab_13)
        self.line_24.setGeometry(QtCore.QRect(20, 40, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_24.setFont(font)
        self.line_24.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_24.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_24.setObjectName("line_24")
        self.line_84 = QtWidgets.QFrame(self.tab_13)
        self.line_84.setGeometry(QtCore.QRect(370, 10, 21, 381))
        self.line_84.setStyleSheet("")
        self.line_84.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_84.setLineWidth(2)
        self.line_84.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_84.setObjectName("line_84")
        self.soundMountAlert = QtWidgets.QComboBox(self.tab_13)
        self.soundMountAlert.setGeometry(QtCore.QRect(190, 110, 161, 21))
        self.soundMountAlert.setObjectName("soundMountAlert")
        self.soundDomeSlewFinished = QtWidgets.QComboBox(self.tab_13)
        self.soundDomeSlewFinished.setGeometry(QtCore.QRect(190, 80, 161, 21))
        self.soundDomeSlewFinished.setObjectName("soundDomeSlewFinished")
        self.label_212 = QtWidgets.QLabel(self.tab_13)
        self.label_212.setGeometry(QtCore.QRect(20, 50, 136, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_212.setFont(font)
        self.label_212.setObjectName("label_212")
        self.label_214 = QtWidgets.QLabel(self.tab_13)
        self.label_214.setGeometry(QtCore.QRect(20, 80, 136, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_214.setFont(font)
        self.label_214.setObjectName("label_214")
        self.label_215 = QtWidgets.QLabel(self.tab_13)
        self.label_215.setGeometry(QtCore.QRect(20, 110, 136, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_215.setFont(font)
        self.label_215.setObjectName("label_215")
        self.label_217 = QtWidgets.QLabel(self.tab_13)
        self.label_217.setGeometry(QtCore.QRect(20, 140, 136, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_217.setFont(font)
        self.label_217.setObjectName("label_217")
        self.soundModelingFinished = QtWidgets.QComboBox(self.tab_13)
        self.soundModelingFinished.setGeometry(QtCore.QRect(190, 140, 161, 21))
        self.soundModelingFinished.setObjectName("soundModelingFinished")
        self.settingsTabWidget.addTab(self.tab_13, "")
        self.tab_11 = QtWidgets.QWidget()
        self.tab_11.setObjectName("tab_11")
        self.le_firmwareTime = QtWidgets.QLabel(self.tab_11)
        self.le_firmwareTime.setGeometry(QtCore.QRect(130, 140, 191, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.le_firmwareTime.setFont(font)
        self.le_firmwareTime.setObjectName("le_firmwareTime")
        self.le_firmwareProductName = QtWidgets.QLabel(self.tab_11)
        self.le_firmwareProductName.setGeometry(QtCore.QRect(130, 50, 191, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.le_firmwareProductName.setFont(font)
        self.le_firmwareProductName.setObjectName("le_firmwareProductName")
        self.le_firmwareDate = QtWidgets.QLabel(self.tab_11)
        self.le_firmwareDate.setGeometry(QtCore.QRect(130, 110, 191, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.le_firmwareDate.setFont(font)
        self.le_firmwareDate.setObjectName("le_firmwareDate")
        self.firmwareProductName_2 = QtWidgets.QLabel(self.tab_11)
        self.firmwareProductName_2.setGeometry(QtCore.QRect(20, 50, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.firmwareProductName_2.setFont(font)
        self.firmwareProductName_2.setObjectName("firmwareProductName_2")
        self.le_firmwareProductName_2 = QtWidgets.QLabel(self.tab_11)
        self.le_firmwareProductName_2.setGeometry(QtCore.QRect(460, 370, 271, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.le_firmwareProductName_2.setFont(font)
        self.le_firmwareProductName_2.setObjectName("le_firmwareProductName_2")
        self.firmwareProductName_6 = QtWidgets.QLabel(self.tab_11)
        self.firmwareProductName_6.setGeometry(QtCore.QRect(20, 170, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.firmwareProductName_6.setFont(font)
        self.firmwareProductName_6.setObjectName("firmwareProductName_6")
        self.label_56 = QtWidgets.QLabel(self.tab_11)
        self.label_56.setGeometry(QtCore.QRect(20, 20, 161, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_56.setFont(font)
        self.label_56.setObjectName("label_56")
        self.le_hardwareVersion = QtWidgets.QLabel(self.tab_11)
        self.le_hardwareVersion.setGeometry(QtCore.QRect(130, 170, 201, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.le_hardwareVersion.setFont(font)
        self.le_hardwareVersion.setObjectName("le_hardwareVersion")
        self.le_firmwareNumber = QtWidgets.QLabel(self.tab_11)
        self.le_firmwareNumber.setGeometry(QtCore.QRect(130, 80, 191, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.le_firmwareNumber.setFont(font)
        self.le_firmwareNumber.setObjectName("le_firmwareNumber")
        self.line_6 = QtWidgets.QFrame(self.tab_11)
        self.line_6.setGeometry(QtCore.QRect(20, 40, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_6.setFont(font)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setObjectName("line_6")
        self.firmwareProductName_4 = QtWidgets.QLabel(self.tab_11)
        self.firmwareProductName_4.setGeometry(QtCore.QRect(20, 110, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.firmwareProductName_4.setFont(font)
        self.firmwareProductName_4.setObjectName("firmwareProductName_4")
        self.firmwareProductName_5 = QtWidgets.QLabel(self.tab_11)
        self.firmwareProductName_5.setGeometry(QtCore.QRect(20, 140, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.firmwareProductName_5.setFont(font)
        self.firmwareProductName_5.setObjectName("firmwareProductName_5")
        self.firmwareProductName_3 = QtWidgets.QLabel(self.tab_11)
        self.firmwareProductName_3.setGeometry(QtCore.QRect(20, 80, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.firmwareProductName_3.setFont(font)
        self.firmwareProductName_3.setObjectName("firmwareProductName_3")
        self.logginLevel = QtWidgets.QGroupBox(self.tab_11)
        self.logginLevel.setGeometry(QtCore.QRect(400, 60, 176, 151))
        self.logginLevel.setObjectName("logginLevel")
        self.loglevelDebug = QtWidgets.QRadioButton(self.logginLevel)
        self.loglevelDebug.setGeometry(QtCore.QRect(15, 25, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loglevelDebug.setFont(font)
        self.loglevelDebug.setChecked(True)
        self.loglevelDebug.setObjectName("loglevelDebug")
        self.loglevelInfo = QtWidgets.QRadioButton(self.logginLevel)
        self.loglevelInfo.setGeometry(QtCore.QRect(15, 55, 76, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loglevelInfo.setFont(font)
        self.loglevelInfo.setChecked(False)
        self.loglevelInfo.setObjectName("loglevelInfo")
        self.loglevelWarning = QtWidgets.QRadioButton(self.logginLevel)
        self.loglevelWarning.setGeometry(QtCore.QRect(15, 85, 106, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loglevelWarning.setFont(font)
        self.loglevelWarning.setChecked(False)
        self.loglevelWarning.setObjectName("loglevelWarning")
        self.loglevelError = QtWidgets.QRadioButton(self.logginLevel)
        self.loglevelError.setGeometry(QtCore.QRect(15, 115, 106, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loglevelError.setFont(font)
        self.loglevelError.setChecked(False)
        self.loglevelError.setObjectName("loglevelError")
        self.line_57 = QtWidgets.QFrame(self.tab_11)
        self.line_57.setGeometry(QtCore.QRect(370, 10, 21, 381))
        self.line_57.setStyleSheet("")
        self.line_57.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_57.setLineWidth(2)
        self.line_57.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_57.setObjectName("line_57")
        self.line_9 = QtWidgets.QFrame(self.tab_11)
        self.line_9.setGeometry(QtCore.QRect(400, 40, 331, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_9.setFont(font)
        self.line_9.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_9.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_9.setObjectName("line_9")
        self.label_59 = QtWidgets.QLabel(self.tab_11)
        self.label_59.setGeometry(QtCore.QRect(400, 20, 161, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_59.setFont(font)
        self.label_59.setObjectName("label_59")
        self.btn_resetWindowSizes = QtWidgets.QPushButton(self.tab_11)
        self.btn_resetWindowSizes.setGeometry(QtCore.QRect(410, 225, 176, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.btn_resetWindowSizes.setFont(font)
        self.btn_resetWindowSizes.setStyleSheet("")
        self.btn_resetWindowSizes.setObjectName("btn_resetWindowSizes")
        self.settingsTabWidget.addTab(self.tab_11, "")
        self.mainTabWidget.addTab(self.tab_3, "")
        self.btn_saveConfigQuit = QtWidgets.QPushButton(MainWindow)
        self.btn_saveConfigQuit.setGeometry(QtCore.QRect(680, 65, 91, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_saveConfigQuit.setFont(font)
        self.btn_saveConfigQuit.setStyleSheet("")
        self.btn_saveConfigQuit.setObjectName("btn_saveConfigQuit")
        self.btn_saveConfig = QtWidgets.QPushButton(MainWindow)
        self.btn_saveConfig.setGeometry(QtCore.QRect(590, 95, 86, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_saveConfig.setFont(font)
        self.btn_saveConfig.setStyleSheet("")
        self.btn_saveConfig.setObjectName("btn_saveConfig")
        self.btn_cameraConnected = QtWidgets.QPushButton(MainWindow)
        self.btn_cameraConnected.setEnabled(False)
        self.btn_cameraConnected.setGeometry(QtCore.QRect(325, 65, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_cameraConnected.setFont(font)
        self.btn_cameraConnected.setObjectName("btn_cameraConnected")
        self.btn_domeConnected = QtWidgets.QPushButton(MainWindow)
        self.btn_domeConnected.setEnabled(False)
        self.btn_domeConnected.setGeometry(QtCore.QRect(245, 65, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_domeConnected.setFont(font)
        self.btn_domeConnected.setObjectName("btn_domeConnected")
        self.btn_driverMountConnected = QtWidgets.QPushButton(MainWindow)
        self.btn_driverMountConnected.setEnabled(False)
        self.btn_driverMountConnected.setGeometry(QtCore.QRect(20, 65, 66, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_driverMountConnected.setFont(font)
        self.btn_driverMountConnected.setFlat(False)
        self.btn_driverMountConnected.setObjectName("btn_driverMountConnected")
        self.btn_environmentConnected = QtWidgets.QPushButton(MainWindow)
        self.btn_environmentConnected.setEnabled(False)
        self.btn_environmentConnected.setGeometry(QtCore.QRect(165, 65, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_environmentConnected.setFont(font)
        self.btn_environmentConnected.setObjectName("btn_environmentConnected")
        self.btn_openMessageWindow = QtWidgets.QPushButton(MainWindow)
        self.btn_openMessageWindow.setGeometry(QtCore.QRect(20, 95, 86, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_openMessageWindow.setFont(font)
        self.btn_openMessageWindow.setStyleSheet("")
        self.btn_openMessageWindow.setObjectName("btn_openMessageWindow")
        self.btn_astrometryConnected = QtWidgets.QPushButton(MainWindow)
        self.btn_astrometryConnected.setEnabled(False)
        self.btn_astrometryConnected.setGeometry(QtCore.QRect(410, 65, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_astrometryConnected.setFont(font)
        self.btn_astrometryConnected.setObjectName("btn_astrometryConnected")
        self.btn_loadFrom = QtWidgets.QPushButton(MainWindow)
        self.btn_loadFrom.setGeometry(QtCore.QRect(590, 65, 86, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_loadFrom.setFont(font)
        self.btn_loadFrom.setStyleSheet("")
        self.btn_loadFrom.setObjectName("btn_loadFrom")
        self.btn_saveConfigAs = QtWidgets.QPushButton(MainWindow)
        self.btn_saveConfigAs.setGeometry(QtCore.QRect(680, 95, 91, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_saveConfigAs.setFont(font)
        self.btn_saveConfigAs.setStyleSheet("")
        self.btn_saveConfigAs.setObjectName("btn_saveConfigAs")
        self.le_configName = QtWidgets.QLineEdit(MainWindow)
        self.le_configName.setEnabled(False)
        self.le_configName.setGeometry(QtCore.QRect(590, 40, 181, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_configName.setFont(font)
        self.le_configName.setMouseTracking(False)
        self.le_configName.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_configName.setAcceptDrops(False)
        self.le_configName.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_configName.setReadOnly(True)
        self.le_configName.setObjectName("le_configName")
        self.mainicon = QtWidgets.QLabel(MainWindow)
        self.mainicon.setGeometry(QtCore.QRect(495, 30, 91, 91))
        self.mainicon.setProperty("iconpicture", True)
        self.mainicon.setObjectName("mainicon")
        self.label_102 = QtWidgets.QLabel(MainWindow)
        self.label_102.setGeometry(QtCore.QRect(590, 15, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_102.setFont(font)
        self.label_102.setObjectName("label_102")
        self.btn_mountShutdown = QtWidgets.QPushButton(MainWindow)
        self.btn_mountShutdown.setGeometry(QtCore.QRect(435, 95, 56, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_mountShutdown.setFont(font)
        self.btn_mountShutdown.setObjectName("btn_mountShutdown")
        self.btn_mountBoot = QtWidgets.QPushButton(MainWindow)
        self.btn_mountBoot.setGeometry(QtCore.QRect(380, 95, 51, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_mountBoot.setFont(font)
        self.btn_mountBoot.setObjectName("btn_mountBoot")
        self.mainBackground = QtWidgets.QLabel(MainWindow)
        self.mainBackground.setGeometry(QtCore.QRect(0, 0, 790, 141))
        self.mainBackground.setText("")
        self.mainBackground.setObjectName("mainBackground")
        self.btn_INDIConnected = QtWidgets.QPushButton(MainWindow)
        self.btn_INDIConnected.setEnabled(False)
        self.btn_INDIConnected.setGeometry(QtCore.QRect(90, 65, 71, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_INDIConnected.setFont(font)
        self.btn_INDIConnected.setObjectName("btn_INDIConnected")
        self.le_cameraStatusText = QtWidgets.QLineEdit(MainWindow)
        self.le_cameraStatusText.setEnabled(False)
        self.le_cameraStatusText.setGeometry(QtCore.QRect(325, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_cameraStatusText.setFont(font)
        self.le_cameraStatusText.setMouseTracking(False)
        self.le_cameraStatusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_cameraStatusText.setAcceptDrops(False)
        self.le_cameraStatusText.setText("")
        self.le_cameraStatusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_cameraStatusText.setReadOnly(True)
        self.le_cameraStatusText.setObjectName("le_cameraStatusText")
        self.le_astrometryStatusText = QtWidgets.QLineEdit(MainWindow)
        self.le_astrometryStatusText.setEnabled(False)
        self.le_astrometryStatusText.setGeometry(QtCore.QRect(410, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_astrometryStatusText.setFont(font)
        self.le_astrometryStatusText.setMouseTracking(False)
        self.le_astrometryStatusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_astrometryStatusText.setAcceptDrops(False)
        self.le_astrometryStatusText.setText("")
        self.le_astrometryStatusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_astrometryStatusText.setReadOnly(True)
        self.le_astrometryStatusText.setObjectName("le_astrometryStatusText")
        self.le_domeStatusText = QtWidgets.QLineEdit(MainWindow)
        self.le_domeStatusText.setEnabled(False)
        self.le_domeStatusText.setGeometry(QtCore.QRect(245, 40, 76, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_domeStatusText.setFont(font)
        self.le_domeStatusText.setMouseTracking(False)
        self.le_domeStatusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_domeStatusText.setAcceptDrops(False)
        self.le_domeStatusText.setText("")
        self.le_domeStatusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_domeStatusText.setReadOnly(True)
        self.le_domeStatusText.setObjectName("le_domeStatusText")
        self.line_30 = QtWidgets.QFrame(MainWindow)
        self.line_30.setGeometry(QtCore.QRect(590, 35, 181, 1))
        self.line_30.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_30.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_30.setObjectName("line_30")
        self.label_110 = QtWidgets.QLabel(MainWindow)
        self.label_110.setGeometry(QtCore.QRect(20, 15, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_110.setFont(font)
        self.label_110.setObjectName("label_110")
        self.line_32 = QtWidgets.QFrame(MainWindow)
        self.line_32.setGeometry(QtCore.QRect(20, 35, 471, 1))
        self.line_32.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_32.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_32.setObjectName("line_32")
        self.btn_openHemisphereWindow = QtWidgets.QPushButton(MainWindow)
        self.btn_openHemisphereWindow.setGeometry(QtCore.QRect(110, 95, 86, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.btn_openHemisphereWindow.setFont(font)
        self.btn_openHemisphereWindow.setStyleSheet("")
        self.btn_openHemisphereWindow.setObjectName("btn_openHemisphereWindow")
        self.btn_openImageWindow = QtWidgets.QPushButton(MainWindow)
        self.btn_openImageWindow.setGeometry(QtCore.QRect(200, 95, 86, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.btn_openImageWindow.setFont(font)
        self.btn_openImageWindow.setStyleSheet("")
        self.btn_openImageWindow.setObjectName("btn_openImageWindow")
        self.btn_openAnalyseWindow = QtWidgets.QPushButton(MainWindow)
        self.btn_openAnalyseWindow.setGeometry(QtCore.QRect(290, 95, 86, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_openAnalyseWindow.setFont(font)
        self.btn_openAnalyseWindow.setStyleSheet("")
        self.btn_openAnalyseWindow.setObjectName("btn_openAnalyseWindow")
        self.mainBackground.raise_()
        self.le_mountStatus.raise_()
        self.mainTabWidget.raise_()
        self.btn_saveConfigQuit.raise_()
        self.btn_saveConfig.raise_()
        self.btn_cameraConnected.raise_()
        self.btn_domeConnected.raise_()
        self.btn_driverMountConnected.raise_()
        self.btn_environmentConnected.raise_()
        self.btn_openMessageWindow.raise_()
        self.btn_astrometryConnected.raise_()
        self.btn_loadFrom.raise_()
        self.btn_saveConfigAs.raise_()
        self.le_configName.raise_()
        self.mainicon.raise_()
        self.label_102.raise_()
        self.btn_mountShutdown.raise_()
        self.btn_mountBoot.raise_()
        self.btn_INDIConnected.raise_()
        self.le_cameraStatusText.raise_()
        self.le_astrometryStatusText.raise_()
        self.le_domeStatusText.raise_()
        self.line_30.raise_()
        self.label_110.raise_()
        self.line_32.raise_()
        self.btn_openHemisphereWindow.raise_()
        self.btn_openImageWindow.raise_()
        self.btn_openAnalyseWindow.raise_()

        self.retranslateUi(MainWindow)
        self.mainTabWidget.setCurrentIndex(0)
        self.settingsTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.btn_openMessageWindow, self.btn_mountBoot)
        MainWindow.setTabOrder(self.btn_mountBoot, self.btn_mountShutdown)
        MainWindow.setTabOrder(self.btn_mountShutdown, self.btn_loadFrom)
        MainWindow.setTabOrder(self.btn_loadFrom, self.btn_saveConfigQuit)
        MainWindow.setTabOrder(self.btn_saveConfigQuit, self.btn_saveConfig)
        MainWindow.setTabOrder(self.btn_saveConfig, self.btn_saveConfigAs)
        MainWindow.setTabOrder(self.btn_saveConfigAs, self.le_slewRate)
        MainWindow.setTabOrder(self.le_slewRate, self.le_telescopeDualTrack)
        MainWindow.setTabOrder(self.le_telescopeDualTrack, self.btn_setDualTracking)
        MainWindow.setTabOrder(self.btn_setDualTracking, self.le_telescopeUnattendedFlip)
        MainWindow.setTabOrder(self.le_telescopeUnattendedFlip, self.btn_setUnattendedFlip)
        MainWindow.setTabOrder(self.btn_setUnattendedFlip, self.le_meridianLimitGuide)
        MainWindow.setTabOrder(self.le_meridianLimitGuide, self.le_meridianLimitSlew)
        MainWindow.setTabOrder(self.le_meridianLimitSlew, self.le_horizonLimitLow)
        MainWindow.setTabOrder(self.le_horizonLimitLow, self.btn_flipMount)
        MainWindow.setTabOrder(self.btn_flipMount, self.le_horizonLimitHigh)
        MainWindow.setTabOrder(self.le_horizonLimitHigh, self.btn_mountPos1)
        MainWindow.setTabOrder(self.btn_mountPos1, self.btn_mountPos2)
        MainWindow.setTabOrder(self.btn_mountPos2, self.btn_mountPos3)
        MainWindow.setTabOrder(self.btn_mountPos3, self.btn_mountPos4)
        MainWindow.setTabOrder(self.btn_mountPos4, self.btn_mountPos5)
        MainWindow.setTabOrder(self.btn_mountPos5, self.btn_mountPos6)
        MainWindow.setTabOrder(self.btn_mountPos6, self.btn_startTracking)
        MainWindow.setTabOrder(self.btn_startTracking, self.btn_stopTracking)
        MainWindow.setTabOrder(self.btn_stopTracking, self.btn_setTrackingLunar)
        MainWindow.setTabOrder(self.btn_setTrackingLunar, self.btn_setTrackingSolar)
        MainWindow.setTabOrder(self.btn_setTrackingSolar, self.btn_setTrackingSideral)
        MainWindow.setTabOrder(self.btn_setTrackingSideral, self.btn_mountPark)
        MainWindow.setTabOrder(self.btn_mountPark, self.btn_mountUnpark)
        MainWindow.setTabOrder(self.btn_mountUnpark, self.btn_stop)
        MainWindow.setTabOrder(self.btn_stop, self.le_UTCDataValid)
        MainWindow.setTabOrder(self.le_UTCDataValid, self.le_UTCDataExpirationDate)
        MainWindow.setTabOrder(self.le_UTCDataExpirationDate, self.btn_setRefractionCorrection)
        MainWindow.setTabOrder(self.btn_setRefractionCorrection, self.checkAutoRefractionNone)
        MainWindow.setTabOrder(self.checkAutoRefractionNone, self.checkAutoRefractionContinous)
        MainWindow.setTabOrder(self.checkAutoRefractionContinous, self.checkAutoRefractionNotTracking)
        MainWindow.setTabOrder(self.checkAutoRefractionNotTracking, self.btn_setRefractionParameters)
        MainWindow.setTabOrder(self.btn_setRefractionParameters, self.cameraExposure)
        MainWindow.setTabOrder(self.cameraExposure, self.cameraBin)
        MainWindow.setTabOrder(self.cameraBin, self.checkDoSubframe)
        MainWindow.setTabOrder(self.checkDoSubframe, self.scaleSubframe)
        MainWindow.setTabOrder(self.scaleSubframe, self.checkFastDownload)
        MainWindow.setTabOrder(self.checkFastDownload, self.isoSetting)
        MainWindow.setTabOrder(self.isoSetting, self.settlingTime)
        MainWindow.setTabOrder(self.settlingTime, self.checkKeepImages)
        MainWindow.setTabOrder(self.checkKeepImages, self.pixelSize)
        MainWindow.setTabOrder(self.pixelSize, self.focalLength)
        MainWindow.setTabOrder(self.focalLength, self.astrometryDownsampling)
        MainWindow.setTabOrder(self.astrometryDownsampling, self.btn_openAnalyseWindow)
        MainWindow.setTabOrder(self.btn_openAnalyseWindow, self.btn_loadHorizonMask)
        MainWindow.setTabOrder(self.btn_loadHorizonMask, self.btn_saveHorizonMask)
        MainWindow.setTabOrder(self.btn_saveHorizonMask, self.btn_saveHorizonMaskAs)
        MainWindow.setTabOrder(self.btn_saveHorizonMaskAs, self.checkUseFileHorizonLine)
        MainWindow.setTabOrder(self.checkUseFileHorizonLine, self.checkUseMinimumHorizonLine)
        MainWindow.setTabOrder(self.checkUseMinimumHorizonLine, self.altitudeMinimumHorizon)
        MainWindow.setTabOrder(self.altitudeMinimumHorizon, self.checkDeletePointsHorizonMask)
        MainWindow.setTabOrder(self.checkDeletePointsHorizonMask, self.checkSortPoints)
        MainWindow.setTabOrder(self.checkSortPoints, self.btn_loadAnalyseData)
        MainWindow.setTabOrder(self.btn_loadAnalyseData, self.btn_showInitialModelPoints)
        MainWindow.setTabOrder(self.btn_showInitialModelPoints, self.btn_loadInitialModelPoints)
        MainWindow.setTabOrder(self.btn_loadInitialModelPoints, self.btn_saveInitialModelPoints)
        MainWindow.setTabOrder(self.btn_saveInitialModelPoints, self.btn_saveInitialModelPointsAs)
        MainWindow.setTabOrder(self.btn_saveInitialModelPointsAs, self.btn_generateInitialPoints)
        MainWindow.setTabOrder(self.btn_generateInitialPoints, self.altitudeBase)
        MainWindow.setTabOrder(self.altitudeBase, self.azimuthBase)
        MainWindow.setTabOrder(self.azimuthBase, self.numberBase)
        MainWindow.setTabOrder(self.numberBase, self.btn_runInitialModel)
        MainWindow.setTabOrder(self.btn_runInitialModel, self.btn_cancelInitialModel)
        MainWindow.setTabOrder(self.btn_cancelInitialModel, self.btn_plateSolveSync)
        MainWindow.setTabOrder(self.btn_plateSolveSync, self.btn_showFullModelPoints)
        MainWindow.setTabOrder(self.btn_showFullModelPoints, self.btn_loadFullModelPoints)
        MainWindow.setTabOrder(self.btn_loadFullModelPoints, self.btn_saveFullModelPoints)
        MainWindow.setTabOrder(self.btn_saveFullModelPoints, self.btn_saveFullModelPointsAs)
        MainWindow.setTabOrder(self.btn_saveFullModelPointsAs, self.btn_generateGridPoints)
        MainWindow.setTabOrder(self.btn_generateGridPoints, self.numberGridPointsRow)
        MainWindow.setTabOrder(self.numberGridPointsRow, self.numberGridPointsCol)
        MainWindow.setTabOrder(self.numberGridPointsCol, self.altitudeMin)
        MainWindow.setTabOrder(self.altitudeMin, self.altitudeMax)
        MainWindow.setTabOrder(self.altitudeMax, self.btn_generateMaxPoints)
        MainWindow.setTabOrder(self.btn_generateMaxPoints, self.btn_generateNormalPoints)
        MainWindow.setTabOrder(self.btn_generateNormalPoints, self.btn_generateMinPoints)
        MainWindow.setTabOrder(self.btn_generateMinPoints, self.btn_generateDSOPoints)
        MainWindow.setTabOrder(self.btn_generateDSOPoints, self.numberHoursDSO)
        MainWindow.setTabOrder(self.numberHoursDSO, self.numberPointsDSO)
        MainWindow.setTabOrder(self.numberPointsDSO, self.numberHoursPreview)
        MainWindow.setTabOrder(self.numberHoursPreview, self.btn_runFullModel)
        MainWindow.setTabOrder(self.btn_runFullModel, self.btn_cancelFullModel)
        MainWindow.setTabOrder(self.btn_cancelFullModel, self.btn_deleteWorstPoint)
        MainWindow.setTabOrder(self.btn_deleteWorstPoint, self.btn_reloadAlignmentModel)
        MainWindow.setTabOrder(self.btn_reloadAlignmentModel, self.btn_runTargetRMSAlignment)
        MainWindow.setTabOrder(self.btn_runTargetRMSAlignment, self.targetRMS)
        MainWindow.setTabOrder(self.targetRMS, self.btn_cancelRunTargetRMSAlignment)
        MainWindow.setTabOrder(self.btn_cancelRunTargetRMSAlignment, self.btn_clearModel)
        MainWindow.setTabOrder(self.btn_clearModel, self.altitudeTimeChange)
        MainWindow.setTabOrder(self.altitudeTimeChange, self.azimuthTimeChange)
        MainWindow.setTabOrder(self.azimuthTimeChange, self.numberRunsTimeChange)
        MainWindow.setTabOrder(self.numberRunsTimeChange, self.delayTimeFlexure)
        MainWindow.setTabOrder(self.delayTimeFlexure, self.btn_runFlexure)
        MainWindow.setTabOrder(self.btn_runFlexure, self.altitudeHysterese1)
        MainWindow.setTabOrder(self.altitudeHysterese1, self.azimuthHysterese1)
        MainWindow.setTabOrder(self.azimuthHysterese1, self.altitudeHysterese2)
        MainWindow.setTabOrder(self.altitudeHysterese2, self.azimuthHysterese2)
        MainWindow.setTabOrder(self.azimuthHysterese2, self.numberRunsHysterese)
        MainWindow.setTabOrder(self.numberRunsHysterese, self.delayTimeHysterese)
        MainWindow.setTabOrder(self.delayTimeHysterese, self.btn_runHysterese)
        MainWindow.setTabOrder(self.btn_runHysterese, self.btn_cancelAnalyse)
        MainWindow.setTabOrder(self.btn_cancelAnalyse, self.btn_relay1)
        MainWindow.setTabOrder(self.btn_relay1, self.btn_relay2)
        MainWindow.setTabOrder(self.btn_relay2, self.btn_relay3)
        MainWindow.setTabOrder(self.btn_relay3, self.btn_relay4)
        MainWindow.setTabOrder(self.btn_relay4, self.btn_relay5)
        MainWindow.setTabOrder(self.btn_relay5, self.btn_relay6)
        MainWindow.setTabOrder(self.btn_relay6, self.btn_relay7)
        MainWindow.setTabOrder(self.btn_relay7, self.btn_relay8)
        MainWindow.setTabOrder(self.btn_relay8, self.le_mountIP)
        MainWindow.setTabOrder(self.le_mountIP, self.le_mountMAC)
        MainWindow.setTabOrder(self.le_mountMAC, self.checkEnableINDI)
        MainWindow.setTabOrder(self.checkEnableINDI, self.le_INDIServerIP)
        MainWindow.setTabOrder(self.le_INDIServerIP, self.le_INDIServerPort)
        MainWindow.setTabOrder(self.le_INDIServerPort, self.le_INDIStatus)
        MainWindow.setTabOrder(self.le_INDIStatus, self.le_INDICCD)
        MainWindow.setTabOrder(self.le_INDICCD, self.le_INDIEnvironment)
        MainWindow.setTabOrder(self.le_INDIEnvironment, self.le_INDIDome)
        MainWindow.setTabOrder(self.le_INDIDome, self.le_AstrometryHost)
        MainWindow.setTabOrder(self.le_AstrometryHost, self.le_AstrometryPort)
        MainWindow.setTabOrder(self.le_AstrometryPort, self.le_AstrometryAPIKey)
        MainWindow.setTabOrder(self.le_AstrometryAPIKey, self.le_astrometryTimeout)
        MainWindow.setTabOrder(self.le_astrometryTimeout, self.checkEnableRelay)
        MainWindow.setTabOrder(self.checkEnableRelay, self.le_relayIP)
        MainWindow.setTabOrder(self.le_relayIP, self.le_relayUsername)
        MainWindow.setTabOrder(self.le_relayUsername, self.le_relayPassword)
        MainWindow.setTabOrder(self.le_relayPassword, self.checkEnableRemoteAccess)
        MainWindow.setTabOrder(self.checkEnableRemoteAccess, self.le_remotePort)
        MainWindow.setTabOrder(self.le_remotePort, self.pd_chooseImaging)
        MainWindow.setTabOrder(self.pd_chooseImaging, self.pd_chooseAstrometry)
        MainWindow.setTabOrder(self.pd_chooseAstrometry, self.pd_chooseEnvironment)
        MainWindow.setTabOrder(self.pd_chooseEnvironment, self.pd_chooseDome)
        MainWindow.setTabOrder(self.pd_chooseDome, self.btn_setupAscomDomeDriver)
        MainWindow.setTabOrder(self.btn_setupAscomDomeDriver, self.btn_setupAscomEnvironmentDriver)
        MainWindow.setTabOrder(self.btn_setupAscomEnvironmentDriver, self.btn_selectCatalogue)
        MainWindow.setTabOrder(self.btn_selectCatalogue, self.btn_downloadEarthrotation)
        MainWindow.setTabOrder(self.btn_downloadEarthrotation, self.btn_downloadSpacestations)
        MainWindow.setTabOrder(self.btn_downloadSpacestations, self.btn_downloadSatbrighest)
        MainWindow.setTabOrder(self.btn_downloadSatbrighest, self.btn_downloadComets)
        MainWindow.setTabOrder(self.btn_downloadComets, self.btn_downloadAsteroidsMPC5000)
        MainWindow.setTabOrder(self.btn_downloadAsteroidsMPC5000, self.btn_downloadAsteroidsNEA)
        MainWindow.setTabOrder(self.btn_downloadAsteroidsNEA, self.btn_downloadAsteroidsPHA)
        MainWindow.setTabOrder(self.btn_downloadAsteroidsPHA, self.btn_downloadAsteroidsTNO)
        MainWindow.setTabOrder(self.btn_downloadAsteroidsTNO, self.btn_downloadAll)
        MainWindow.setTabOrder(self.btn_downloadAll, self.checkEarthrotation)
        MainWindow.setTabOrder(self.checkEarthrotation, self.checkTLE)
        MainWindow.setTabOrder(self.checkTLE, self.checkComets)
        MainWindow.setTabOrder(self.checkComets, self.checkAsteroids)
        MainWindow.setTabOrder(self.checkAsteroids, self.checkFilterMPC)
        MainWindow.setTabOrder(self.checkFilterMPC, self.le_filterExpressionMPC)
        MainWindow.setTabOrder(self.le_filterExpressionMPC, self.btn_uploadMount)
        MainWindow.setTabOrder(self.btn_uploadMount, self.le_parkPos1Text)
        MainWindow.setTabOrder(self.le_parkPos1Text, self.le_azParkPos1)
        MainWindow.setTabOrder(self.le_azParkPos1, self.le_altParkPos1)
        MainWindow.setTabOrder(self.le_altParkPos1, self.le_parkPos2Text)
        MainWindow.setTabOrder(self.le_parkPos2Text, self.le_azParkPos2)
        MainWindow.setTabOrder(self.le_azParkPos2, self.le_altParkPos2)
        MainWindow.setTabOrder(self.le_altParkPos2, self.le_parkPos3Text)
        MainWindow.setTabOrder(self.le_parkPos3Text, self.le_azParkPos3)
        MainWindow.setTabOrder(self.le_azParkPos3, self.le_altParkPos3)
        MainWindow.setTabOrder(self.le_altParkPos3, self.le_parkPos4Text)
        MainWindow.setTabOrder(self.le_parkPos4Text, self.le_azParkPos4)
        MainWindow.setTabOrder(self.le_azParkPos4, self.le_altParkPos4)
        MainWindow.setTabOrder(self.le_altParkPos4, self.le_parkPos5Text)
        MainWindow.setTabOrder(self.le_parkPos5Text, self.le_azParkPos5)
        MainWindow.setTabOrder(self.le_azParkPos5, self.le_altParkPos5)
        MainWindow.setTabOrder(self.le_altParkPos5, self.le_parkPos6Text)
        MainWindow.setTabOrder(self.le_parkPos6Text, self.le_azParkPos6)
        MainWindow.setTabOrder(self.le_azParkPos6, self.le_altParkPos6)
        MainWindow.setTabOrder(self.le_altParkPos6, self.relay1Text)
        MainWindow.setTabOrder(self.relay1Text, self.relay1Function)
        MainWindow.setTabOrder(self.relay1Function, self.relay2Text)
        MainWindow.setTabOrder(self.relay2Text, self.relay2Function)
        MainWindow.setTabOrder(self.relay2Function, self.relay3Text)
        MainWindow.setTabOrder(self.relay3Text, self.relay3Function)
        MainWindow.setTabOrder(self.relay3Function, self.relay4Text)
        MainWindow.setTabOrder(self.relay4Text, self.relay4Function)
        MainWindow.setTabOrder(self.relay4Function, self.relay5Text)
        MainWindow.setTabOrder(self.relay5Text, self.relay5Function)
        MainWindow.setTabOrder(self.relay5Function, self.relay6Text)
        MainWindow.setTabOrder(self.relay6Text, self.relay6Function)
        MainWindow.setTabOrder(self.relay6Function, self.relay7Text)
        MainWindow.setTabOrder(self.relay7Text, self.relay7Function)
        MainWindow.setTabOrder(self.relay7Function, self.relay8Text)
        MainWindow.setTabOrder(self.relay8Text, self.relay8Function)
        MainWindow.setTabOrder(self.relay8Function, self.soundMountSlewFinished)
        MainWindow.setTabOrder(self.soundMountSlewFinished, self.soundDomeSlewFinished)
        MainWindow.setTabOrder(self.soundDomeSlewFinished, self.soundMountAlert)
        MainWindow.setTabOrder(self.soundMountAlert, self.soundModelingFinished)
        MainWindow.setTabOrder(self.soundModelingFinished, self.loglevelDebug)
        MainWindow.setTabOrder(self.loglevelDebug, self.loglevelInfo)
        MainWindow.setTabOrder(self.loglevelInfo, self.loglevelWarning)
        MainWindow.setTabOrder(self.loglevelWarning, self.loglevelError)
        MainWindow.setTabOrder(self.loglevelError, self.btn_INDIConnected)
        MainWindow.setTabOrder(self.btn_INDIConnected, self.le_pinpointCatalogue)
        MainWindow.setTabOrder(self.le_pinpointCatalogue, self.btn_cameraConnected)
        MainWindow.setTabOrder(self.btn_cameraConnected, self.btn_environmentConnected)
        MainWindow.setTabOrder(self.btn_environmentConnected, self.le_modelFullPointsFileName)
        MainWindow.setTabOrder(self.le_modelFullPointsFileName, self.btn_driverMountConnected)
        MainWindow.setTabOrder(self.btn_driverMountConnected, self.le_ascomEnvironmentDriverName)
        MainWindow.setTabOrder(self.le_ascomEnvironmentDriverName, self.le_modelInitialPointsFileName)
        MainWindow.setTabOrder(self.le_modelInitialPointsFileName, self.btn_astrometryConnected)
        MainWindow.setTabOrder(self.btn_astrometryConnected, self.settingsTabWidget)
        MainWindow.setTabOrder(self.settingsTabWidget, self.btn_domeConnected)
        MainWindow.setTabOrder(self.btn_domeConnected, self.le_horizonPointsFileName)
        MainWindow.setTabOrder(self.le_horizonPointsFileName, self.le_analyseFileName)
        MainWindow.setTabOrder(self.le_analyseFileName, self.le_ascomDomeDriverName)
        MainWindow.setTabOrder(self.le_ascomDomeDriverName, self.mainTabWidget)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MountWizzard3 (C) MW 2016-18"))
        self.le_mountStatus.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.le_mountStatus.setText(_translate("MainWindow", "Mount Status"))
        self.mainTabWidget.setToolTip(_translate("MainWindow", "<html><head/><body><p>Tabs for the main functions of model making</p></body></html>"))
        self.le_telescopeRA.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt; font-weight:400;\">Right Ascension in J2000</span></p></body></html>"))
        self.le_telescopeRA.setText(_translate("MainWindow", "RA"))
        self.le_telescopeAzimut.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt; font-weight:400;\">Azimuth</span></p></body></html>"))
        self.le_telescopeAzimut.setText(_translate("MainWindow", "AZ"))
        self.le_telescopeDEC.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt; font-weight:400;\">Declination in J2000</span></p></body></html>"))
        self.le_telescopeDEC.setText(_translate("MainWindow", "DEC"))
        self.le_telescopeAltitude.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt; font-weight:400;\">Altitude</span></p></body></html>"))
        self.le_telescopeAltitude.setText(_translate("MainWindow", "ALT"))
        self.label_9.setText(_translate("MainWindow", "Telescope AZ (Deg)"))
        self.label_10.setText(_translate("MainWindow", "Telescope DEC (DMS)"))
        self.label_8.setText(_translate("MainWindow", "Telescope ALT (Deg)"))
        self.label_11.setText(_translate("MainWindow", "Telescope RA (HMS)"))
        self.label_17.setText(_translate("MainWindow", "Slew Rate"))
        self.le_meridianLimitGuide.setText(_translate("MainWindow", "--"))
        self.le_telescopePierSide.setToolTip(_translate("MainWindow", "Pier side"))
        self.le_telescopePierSide.setText(_translate("MainWindow", "----"))
        self.label_18.setText(_translate("MainWindow", "Pier side is"))
        self.label_21.setText(_translate("MainWindow", "Dual Tracking"))
        self.le_telescopeDualTrack.setToolTip(_translate("MainWindow", "Status of dual tracking"))
        self.le_telescopeDualTrack.setText(_translate("MainWindow", "--"))
        self.btn_setDualTracking.setToolTip(_translate("MainWindow", "Sets dual tracking on / off"))
        self.btn_setDualTracking.setText(_translate("MainWindow", "I/O"))
        self.label_52.setText(_translate("MainWindow", "Time to Flip"))
        self.le_timeToFlip.setToolTip(_translate("MainWindow", "Time until tracking ends because of flip / horizon limit"))
        self.le_timeToFlip.setText(_translate("MainWindow", "----"))
        self.btn_setUnattendedFlip.setToolTip(_translate("MainWindow", "Sets unattended flip on / off"))
        self.btn_setUnattendedFlip.setText(_translate("MainWindow", "I/O"))
        self.le_telescopeUnattendedFlip.setToolTip(_translate("MainWindow", "Status of unattended Flip"))
        self.le_telescopeUnattendedFlip.setText(_translate("MainWindow", "--"))
        self.label_23.setText(_translate("MainWindow", "Unattended Flip"))
        self.label_61.setText(_translate("MainWindow", "min"))
        self.label_62.setText(_translate("MainWindow", "°/s"))
        self.label_80.setText(_translate("MainWindow", "Tracking"))
        self.btn_startTracking.setToolTip(_translate("MainWindow", "Start tracking the mount"))
        self.btn_startTracking.setText(_translate("MainWindow", "Start Tracking"))
        self.btn_mountPark.setToolTip(_translate("MainWindow", "<html><head/><body><p>Park mount in default position</p></body></html>"))
        self.btn_mountPark.setText(_translate("MainWindow", "Park"))
        self.btn_mountUnpark.setToolTip(_translate("MainWindow", "Unpark the mount"))
        self.btn_mountUnpark.setText(_translate("MainWindow", "Unpark"))
        self.label_83.setText(_translate("MainWindow", "Defined Positions"))
        self.label_84.setText(_translate("MainWindow", "Tracking"))
        self.btn_mountPos4.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.btn_mountPos4.setText(_translate("MainWindow", "Park Pos4"))
        self.label_85.setText(_translate("MainWindow", "Parking"))
        self.btn_mountPos2.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.btn_mountPos2.setText(_translate("MainWindow", "Park Pos2"))
        self.btn_mountPos3.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.btn_mountPos3.setText(_translate("MainWindow", "Park Pos3"))
        self.btn_mountPos1.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.btn_mountPos1.setText(_translate("MainWindow", "Park Pos1"))
        self.btn_stopTracking.setToolTip(_translate("MainWindow", "Stop tracking mount"))
        self.btn_stopTracking.setText(_translate("MainWindow", "Stop Tracking"))
        self.btn_stop.setToolTip(_translate("MainWindow", "Stops all mount movements immediatly"))
        self.btn_stop.setText(_translate("MainWindow", "STOP MOUNT"))
        self.btn_flipMount.setToolTip(_translate("MainWindow", "Flips the mount if possible"))
        self.btn_flipMount.setText(_translate("MainWindow", "Meridian Flip Now"))
        self.btn_setTrackingLunar.setToolTip(_translate("MainWindow", "Set tracking speed to lunar"))
        self.btn_setTrackingLunar.setText(_translate("MainWindow", "Lunar"))
        self.btn_setTrackingSolar.setToolTip(_translate("MainWindow", "Set tracking speed to solar"))
        self.btn_setTrackingSolar.setText(_translate("MainWindow", "Solar"))
        self.btn_setTrackingSideral.setToolTip(_translate("MainWindow", "Set tracking speed to sideral"))
        self.btn_setTrackingSideral.setText(_translate("MainWindow", "Sideral"))
        self.label_34.setText(_translate("MainWindow", "Meridian Lim Guide"))
        self.label_65.setText(_translate("MainWindow", "Meridian Lim Slew"))
        self.le_meridianLimitSlew.setText(_translate("MainWindow", "--"))
        self.label_88.setText(_translate("MainWindow", "°"))
        self.label_89.setText(_translate("MainWindow", "°"))
        self.label_90.setText(_translate("MainWindow", "Time to Meridian"))
        self.le_timeToMeridian.setToolTip(_translate("MainWindow", "Time until tracking ends because of flip / horizon limit"))
        self.le_timeToMeridian.setText(_translate("MainWindow", "----"))
        self.label_91.setText(_translate("MainWindow", "min"))
        self.btn_mountPos6.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.btn_mountPos6.setText(_translate("MainWindow", "Park Pos6"))
        self.btn_mountPos5.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.btn_mountPos5.setText(_translate("MainWindow", "Park Pos5"))
        self.label_20.setText(_translate("MainWindow", "Horizon Lim high"))
        self.le_horizonLimitHigh.setToolTip(_translate("MainWindow", "Actual horizon limit for slewing higher bound"))
        self.label_4.setText(_translate("MainWindow", "Horion Lim low"))
        self.le_horizonLimitLow.setToolTip(_translate("MainWindow", "Actual horizon limit for slewing lower bound"))
        self.label_98.setText(_translate("MainWindow", "Pointing Coordinates"))
        self.label_104.setText(_translate("MainWindow", "°"))
        self.label_109.setText(_translate("MainWindow", "°"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab_2), _translate("MainWindow", " Mount Data"))
        self.le_siteElevation.setText(_translate("MainWindow", "0000.0"))
        self.label_5.setText(_translate("MainWindow", "Site Longitude"))
        self.le_siteLongitude.setText(_translate("MainWindow", "0000000000"))
        self.label_6.setText(_translate("MainWindow", "Site Latitude"))
        self.le_localSiderealTime.setText(_translate("MainWindow", "00:00:00.00"))
        self.label_7.setText(_translate("MainWindow", "Local Sidereal Time "))
        self.le_siteLatitude.setText(_translate("MainWindow", "0000000000"))
        self.label_12.setText(_translate("MainWindow", "Site Elevation"))
        self.label_58.setText(_translate("MainWindow", "Coordinates from Mount"))
        self.label_16.setText(_translate("MainWindow", "measurement (for both axes)"))
        self.le_telescopeTempDECMotor.setToolTip(_translate("MainWindow", "Temperatur of DEC motor in mount"))
        self.le_telescopeTempDECMotor.setText(_translate("MainWindow", "00.0"))
        self.label_55.setText(_translate("MainWindow", "Temperature Motor Driver Circuit"))
        self.label_64.setText(_translate("MainWindow", "°C"))
        self.le_windSpeed.setToolTip(_translate("MainWindow", "shows the refraction correction status on / off"))
        self.le_windSpeed.setText(_translate("MainWindow", "---"))
        self.label_32.setText(_translate("MainWindow", "WindSpeed"))
        self.label_73.setText(_translate("MainWindow", "hPa"))
        self.le_temperature.setText(_translate("MainWindow", "---"))
        self.label_25.setText(_translate("MainWindow", "DewPoint"))
        self.label_31.setText(_translate("MainWindow", "Rain Rate"))
        self.label_70.setText(_translate("MainWindow", "°C"))
        self.label_68.setText(_translate("MainWindow", "%"))
        self.label_26.setText(_translate("MainWindow", "Humidity"))
        self.le_refractionTemperature.setToolTip(_translate("MainWindow", "shows the refraction temperature used in mount"))
        self.le_refractionTemperature.setText(_translate("MainWindow", "---"))
        self.le_humidity.setText(_translate("MainWindow", "---"))
        self.le_cloudCover.setText(_translate("MainWindow", "---"))
        self.label_67.setText(_translate("MainWindow", "m/s"))
        self.btn_setRefractionParameters.setToolTip(_translate("MainWindow", "<html><head/><body><p>Pushes the refraction temperature and pressure to mount.</p></body></html>"))
        self.btn_setRefractionParameters.setText(_translate("MainWindow", "Push\n"
"Parameters\n"
"to Mount\n"
"manually"))
        self.le_dewPoint.setText(_translate("MainWindow", "---"))
        self.label_71.setText(_translate("MainWindow", "°C"))
        self.le_windDirection.setToolTip(_translate("MainWindow", "shows the refraction correction status on / off"))
        self.le_windDirection.setText(_translate("MainWindow", "---"))
        self.label_33.setText(_translate("MainWindow", "WindDirect."))
        self.label_30.setText(_translate("MainWindow", "CloudCover"))
        self.label_24.setText(_translate("MainWindow", "Airtemp."))
        self.label_22.setText(_translate("MainWindow", "Refraction correction in mount computer is"))
        self.label_27.setText(_translate("MainWindow", "Pressure"))
        self.label_57.setText(_translate("MainWindow", "Environment Conditions"))
        self.le_refractionPressure.setToolTip(_translate("MainWindow", "shows the refraction pressure used in mount"))
        self.le_refractionPressure.setText(_translate("MainWindow", "---"))
        self.le_rainRate.setText(_translate("MainWindow", "---"))
        self.label_66.setText(_translate("MainWindow", "°"))
        self.le_pressure.setText(_translate("MainWindow", "---"))
        self.le_refractionStatus.setToolTip(_translate("MainWindow", "shows the refraction correction status on / off"))
        self.le_refractionStatus.setText(_translate("MainWindow", "ON"))
        self.label_13.setText(_translate("MainWindow", "Data in mount"))
        self.label_72.setText(_translate("MainWindow", "%"))
        self.label_69.setText(_translate("MainWindow", "%"))
        self.label_78.setText(_translate("MainWindow", "hPa"))
        self.label_82.setText(_translate("MainWindow", "m"))
        self.label_86.setText(_translate("MainWindow", "Refraction Correction in Mount"))
        self.btn_setRefractionCorrection.setToolTip(_translate("MainWindow", "Sets refraction correction in mount on / off"))
        self.btn_setRefractionCorrection.setText(_translate("MainWindow", "I/O"))
        self.label_105.setText(_translate("MainWindow", "DMS"))
        self.label_106.setText(_translate("MainWindow", "DMS"))
        self.label_79.setText(_translate("MainWindow", "°C"))
        self.label_142.setText(_translate("MainWindow", "SQR"))
        self.le_SQR.setToolTip(_translate("MainWindow", "shows the refraction correction status on / off"))
        self.le_SQR.setText(_translate("MainWindow", "---"))
        self.label_87.setText(_translate("MainWindow", "mpas"))
        self.label_63.setText(_translate("MainWindow", "Julian Date"))
        self.le_JulianDate.setText(_translate("MainWindow", "0000000.00000"))
        self.label_151.setText(_translate("MainWindow", "UTC update is"))
        self.le_UTCDataExpirationDate.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Expiration date of UTC data</span></p></body></html>"))
        self.le_UTCDataExpirationDate.setText(_translate("MainWindow", "-----"))
        self.label_152.setText(_translate("MainWindow", "and expires on "))
        self.le_UTCDataValid.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of UTC data</p></body></html>"))
        self.le_UTCDataValid.setText(_translate("MainWindow", "-----"))
        self.label_74.setText(_translate("MainWindow", "Actuality of Time Data in Mount Computer"))
        self.label_135.setText(_translate("MainWindow", "Periodic refraction data update"))
        self.groupBox_3.setTitle(_translate("MainWindow", "Update scheme"))
        self.checkAutoRefractionContinous.setToolTip(_translate("MainWindow", "<html><head/><body><p>Update the refraction parameters to the mount when camera is connected and IDLE.</p></body></html>"))
        self.checkAutoRefractionContinous.setText(_translate("MainWindow", "Continously with smooth filter"))
        self.checkAutoRefractionNotTracking.setToolTip(_translate("MainWindow", "<html><head/><body><p>Update the refraction parameters to the mount when the mount is not tracking.</p></body></html>"))
        self.checkAutoRefractionNotTracking.setText(_translate("MainWindow", "When mount is not in tracking state"))
        self.checkAutoRefractionNone.setToolTip(_translate("MainWindow", "<html><head/><body><p>Update the refraction parameters to the mount when camera is connected and IDLE.</p></body></html>"))
        self.checkAutoRefractionNone.setText(_translate("MainWindow", "No update of parameters"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab_4), _translate("MainWindow", "Site Data"))
        self.checkSortPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sorts the modeling points EAST / WEST side of pier.</p></body></html>"))
        self.checkSortPoints.setText(_translate("MainWindow", "Auto sort points East / West"))
        self.checkDeletePointsHorizonMask.setToolTip(_translate("MainWindow", "<html><head/><body><p>Removes modeling points, which are below the horizon mask.</p></body></html>"))
        self.checkDeletePointsHorizonMask.setText(_translate("MainWindow", "Auto delete points below horizon"))
        self.label_122.setText(_translate("MainWindow", "deg"))
        self.checkUseMinimumHorizonLine.setToolTip(_translate("MainWindow", "<html><head/><body><p>Use minimum altitude for horizon mask in hemisphere window.</p></body></html>"))
        self.checkUseMinimumHorizonLine.setText(_translate("MainWindow", "Use Minumum Altitude"))
        self.checkUseFileHorizonLine.setToolTip(_translate("MainWindow", "<html><head/><body><p>Use horizon mask file in hemisphere window.</p></body></html>"))
        self.checkUseFileHorizonLine.setText(_translate("MainWindow", "Use Horizon Mask File"))
        self.le_horizonPointsFileName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.label_201.setText(_translate("MainWindow", "Set altitude minimum to"))
        self.label_194.setText(_translate("MainWindow", "Horizon Mask File"))
        self.altitudeMinimumHorizon.setToolTip(_translate("MainWindow", "<html><head/><body><p><br/></p></body></html>"))
        self.label_164.setText(_translate("MainWindow", "s"))
        self.label_165.setText(_translate("MainWindow", "s"))
        self.settlingTime.setToolTip(_translate("MainWindow", "Sets the time to wait after mount stop before imaging"))
        self.label_172.setText(_translate("MainWindow", "ISO Setting"))
        self.checkFastDownload.setToolTip(_translate("MainWindow", "Use Highspeed download of CCD camera if availabe"))
        self.checkFastDownload.setText(_translate("MainWindow", "Use fast download mode if camera supports it"))
        self.label_40.setText(_translate("MainWindow", "Camera Binning (X/Y)"))
        self.label_163.setText(_translate("MainWindow", "Imaging Configuration for Modeling"))
        self.label_38.setText(_translate("MainWindow", "Mount Settle Time"))
        self.checkDoSubframe.setToolTip(_translate("MainWindow", "<html><head/><body><p>If checked, subframes were used if possible</p></body></html>"))
        self.checkDoSubframe.setText(_translate("MainWindow", "Use Subframe"))
        self.checkKeepImages.setToolTip(_translate("MainWindow", "Checked if you would like to keep the images of modeling session"))
        self.checkKeepImages.setText(_translate("MainWindow", "Keep images from model building"))
        self.cameraBin.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the binning for the camera</p></body></html>"))
        self.scaleSubframe.setToolTip(_translate("MainWindow", "<html><head/><body><p>sets the percentage of frame size for subframe </p></body></html>"))
        self.cameraExposure.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the exposure time</p></body></html>"))
        self.label_173.setText(_translate("MainWindow", "%"))
        self.isoSetting.setToolTip(_translate("MainWindow", "Sets the iso setting"))
        self.label_39.setText(_translate("MainWindow", "Exposure Time of Camera"))
        self.btn_saveHorizonMaskAs.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the horizon mask in new file</p></body></html>"))
        self.btn_saveHorizonMaskAs.setText(_translate("MainWindow", "Save as"))
        self.btn_loadHorizonMask.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the horizon mask from given file</p></body></html>"))
        self.btn_loadHorizonMask.setText(_translate("MainWindow", "Load"))
        self.btn_saveHorizonMask.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the horizon mask in file</p></body></html>"))
        self.btn_saveHorizonMask.setText(_translate("MainWindow", "Save"))
        self.label_99.setText(_translate("MainWindow", "Analyse File"))
        self.le_analyseFileName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Analysis file, where the data ist stored.</p><p>Please Double Click to select.</p></body></html>"))
        self.btn_loadAnalyseData.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the modeling data from file.</p></body></html>"))
        self.btn_loadAnalyseData.setText(_translate("MainWindow", "Load"))
        self.label_166.setText(_translate("MainWindow", "Pixel size x/y for camera"))
        self.pixelSize.setToolTip(_translate("MainWindow", "Pixel size of camera"))
        self.label_167.setText(_translate("MainWindow", "um"))
        self.label_168.setText(_translate("MainWindow", "Focal length of telescope"))
        self.focalLength.setToolTip(_translate("MainWindow", "resulting focal length - please think of reducers etc."))
        self.label_169.setText(_translate("MainWindow", "mm"))
        self.label_218.setText(_translate("MainWindow", "Configuration for solving"))
        self.astrometryDownsampling.setToolTip(_translate("MainWindow", "<html><head/><body><p>downsample_factor: float, >1. Downsample (bin) your image by this factor before performing source detection. This often helps with saturated images, noisy images, and large images. 2 and 4 are commonly-useful values.</p></body></html>"))
        self.label_219.setText(_translate("MainWindow", "Astrometry downsample factor"))
        self.label_221.setText(_translate("MainWindow", "Astrometry radius"))
        self.astrometryRadius.setToolTip(_translate("MainWindow", "<html><head/><body><p>Radius: float, in degrees. Used with center_ra, center_dec to specify that you know roughly where your image is on the sky. Please set to 0 if you don\'t know.</p></body></html>"))
        self.label_222.setText(_translate("MainWindow", "deg"))
        self.checkUseBlindSolving.setToolTip(_translate("MainWindow", "<html><head/><body><p>Checked if you would like to enable blind solver function in SGPro</p></body></html>"))
        self.checkUseBlindSolving.setText(_translate("MainWindow", "Use blind solving (SGPro)"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab), _translate("MainWindow", "Config Model Build"))
        self.label_116.setText(_translate("MainWindow", "Generate Initial Points"))
        self.label_171.setText(_translate("MainWindow", "Azimuth"))
        self.label_170.setText(_translate("MainWindow", "Altitude"))
        self.btn_runInitialModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Runs the base model.</p></body></html>"))
        self.btn_runInitialModel.setText(_translate("MainWindow", "Run Initial Model"))
        self.btn_generateInitialPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates base points based on number, altitude and azimuth settings.</p></body></html>"))
        self.btn_generateInitialPoints.setText(_translate("MainWindow", "Generate Initial Points"))
        self.btn_cancelInitialModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels model run.</p></body></html>"))
        self.btn_cancelInitialModel.setText(_translate("MainWindow", "Cancel Model Run"))
        self.label.setText(_translate("MainWindow", "Number Stars"))
        self.le_alignErrorAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual Polar Error</p></body></html>"))
        self.le_alignErrorAz.setText(_translate("MainWindow", "--:--:--"))
        self.le_alignErrorPosAngle.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual RMS Error in arc sec</p><p><br/></p></body></html>"))
        self.le_alignErrorPosAngle.setText(_translate("MainWindow", "--.-"))
        self.le_alignKnobTurnAz.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of Turns azimuth ajustment knobs for correction (left if polar axis has to be moved to the left) </p></body></html>"))
        self.le_alignKnobTurnAz.setText(_translate("MainWindow", "--- left/right"))
        self.le_alignErrorPolar.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual Polar Error</p></body></html>"))
        self.le_alignErrorPolar.setText(_translate("MainWindow", "--:--:--"))
        self.label_14.setText(_translate("MainWindow", "RMS"))
        self.label_153.setText(_translate("MainWindow", "Ortho Align Error"))
        self.le_alignNumberTerms.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of Terms in actual used pointing model</p></body></html>"))
        self.le_alignNumberTerms.setText(_translate("MainWindow", "0"))
        self.le_alignKnobTurnAlt.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of Turns altitude ajustment knobs for correction (down if the polar axis has to be moved downwards) </p></body></html>"))
        self.le_alignKnobTurnAlt.setText(_translate("MainWindow", "--- left/right"))
        self.label_156.setText(_translate("MainWindow", "Turn Knob AZ"))
        self.label_158.setText(_translate("MainWindow", "AZ Pointing"))
        self.le_alignNumberStars.setToolTip(_translate("MainWindow", "Number of alignment stars in actual used pointing model"))
        self.le_alignNumberStars.setText(_translate("MainWindow", "0"))
        self.le_alignErrorRMS.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual RMS Error in arc sec</p><p><br/></p></body></html>"))
        self.le_alignErrorRMS.setText(_translate("MainWindow", "--.-"))
        self.label_155.setText(_translate("MainWindow", "Alignment Info from Mount Computer"))
        self.le_alignErrorOrtho.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual Ortho Error</p></body></html>"))
        self.le_alignErrorOrtho.setText(_translate("MainWindow", "--:--:--"))
        self.label_159.setText(_translate("MainWindow", "ALT Pointing"))
        self.label_15.setText(_translate("MainWindow", "Polar Align Error "))
        self.le_alignErrorAlt.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual Ortho Error</p></body></html>"))
        self.le_alignErrorAlt.setText(_translate("MainWindow", "--:--:--"))
        self.label_154.setText(_translate("MainWindow", "Number Terms"))
        self.label_157.setText(_translate("MainWindow", "Turn Knob ALT"))
        self.label_160.setText(_translate("MainWindow", "PosAngle"))
        self.btn_plateSolveSync.setToolTip(_translate("MainWindow", "<html><head/><body><p>Image, Platesolve and Sync the alignment model of the mount</p></body></html>"))
        self.btn_plateSolveSync.setText(_translate("MainWindow", "Plate Solve and Sync Model"))
        self.label_206.setText(_translate("MainWindow", "Number of points"))
        self.picAZ.setText(_translate("MainWindow", "TextLabel"))
        self.label_145.setText(_translate("MainWindow", "Align Model"))
        self.label_207.setText(_translate("MainWindow", "Adjust Polar Alignment"))
        self.picALT.setText(_translate("MainWindow", "TextLabel"))
        self.label_208.setText(_translate("MainWindow", "Fast Align"))
        self.btn_showInitialModelPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Show the initial model points from given file</p></body></html>"))
        self.btn_showInitialModelPoints.setText(_translate("MainWindow", "Reload"))
        self.btn_saveInitialModelPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the initial model points in file</p></body></html>"))
        self.btn_saveInitialModelPoints.setText(_translate("MainWindow", "Save"))
        self.btn_saveInitialModelPointsAs.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the initial model points in new file</p></body></html>"))
        self.btn_saveInitialModelPointsAs.setText(_translate("MainWindow", "Save as"))
        self.le_modelInitialPointsFileName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Model Points file, where the data is stored.</p><p>Please Double Click to Select</p></body></html>"))
        self.label_147.setText(_translate("MainWindow", "Load Initial Model Points File"))
        self.btn_loadInitialModelPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the initial model points from file</p></body></html>"))
        self.btn_loadInitialModelPoints.setText(_translate("MainWindow", "Load"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab_15), _translate("MainWindow", "Initial Model"))
        self.btn_generateGridPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates a regular pattern with the given parameters.</p></body></html>"))
        self.btn_generateGridPoints.setText(_translate("MainWindow", "Generate a Grid of Model Points"))
        self.label_108.setText(_translate("MainWindow", "Generate Model Points"))
        self.label_179.setText(_translate("MainWindow", "Columns"))
        self.label_178.setText(_translate("MainWindow", "Rows"))
        self.btn_generateNormalPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates a pointcloud with greater circles: medium number of stars.</p></body></html>"))
        self.btn_generateNormalPoints.setText(_translate("MainWindow", "Medium"))
        self.btn_generateMaxPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates a pointcloud with greater circles: high number of stars.</p></body></html>"))
        self.btn_generateMaxPoints.setText(_translate("MainWindow", "Maximal"))
        self.numberGridPointsRow.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of rows in the grid build</p></body></html>"))
        self.numberGridPointsCol.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of coloms in the grid build</p></body></html>"))
        self.btn_runFullModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Run full alignment model.</p></body></html>"))
        self.btn_runFullModel.setText(_translate("MainWindow", "Run Full Model"))
        self.btn_cancelFullModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels model run.</p></body></html>"))
        self.btn_cancelFullModel.setText(_translate("MainWindow", "Cancel Model Run"))
        self.btn_generateDSOPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates model points along DSO track for a give time and other parameters.</p></body></html>"))
        self.btn_generateDSOPoints.setText(_translate("MainWindow", "Generate DSO Track for Model Points"))
        self.numberHoursDSO.setToolTip(_translate("MainWindow", "<html><head/><body><p>Length of the DSO track in hours.</p></body></html>"))
        self.label_181.setText(_translate("MainWindow", "Hours"))
        self.label_188.setText(_translate("MainWindow", "Points"))
        self.numberPointsDSO.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of points in the DSO track for the given time in hours</p></body></html>"))
        self.label_189.setText(_translate("MainWindow", "TimeShift"))
        self.numberHoursPreview.setToolTip(_translate("MainWindow", "<html><head/><body><p>Length of the DSO track in hours.</p></body></html>"))
        self.label_180.setText(_translate("MainWindow", "Altitude Max"))
        self.altitudeMax.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of coloms in the grid build</p></body></html>"))
        self.altitudeMin.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of rows in the grid build</p></body></html>"))
        self.label_190.setText(_translate("MainWindow", "Altitude Min"))
        self.label_137.setText(_translate("MainWindow", "°"))
        self.label_144.setText(_translate("MainWindow", "°"))
        self.label_202.setText(_translate("MainWindow", "°"))
        self.label_203.setText(_translate("MainWindow", "°"))
        self.btn_generateMinPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates a pointcloud with greater circles: low number of stars.</p></body></html>"))
        self.btn_generateMinPoints.setText(_translate("MainWindow", "Minimal"))
        self.label_204.setText(_translate("MainWindow", "h"))
        self.label_138.setText(_translate("MainWindow", "Full Model"))
        self.label_113.setText(_translate("MainWindow", "Generate Model Points based on greater circles"))
        self.label_136.setText(_translate("MainWindow", "Generate Model Points based on tracks"))
        self.label_213.setText(_translate("MainWindow", "Load Full Model Points File"))
        self.le_modelFullPointsFileName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Model Points file, where the data is stored.</p><p>Please Double Click to Select</p></body></html>"))
        self.btn_showFullModelPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Show the full model points from given file</p></body></html>"))
        self.btn_showFullModelPoints.setText(_translate("MainWindow", "Reload"))
        self.btn_saveFullModelPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the full model points in file</p></body></html>"))
        self.btn_saveFullModelPoints.setText(_translate("MainWindow", "Save"))
        self.btn_saveFullModelPointsAs.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the full model points in new file</p></body></html>"))
        self.btn_saveFullModelPointsAs.setText(_translate("MainWindow", "Save as"))
        self.btn_loadFullModelPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the full model points from file</p></body></html>"))
        self.btn_loadFullModelPoints.setText(_translate("MainWindow", "Load"))
        self.le_modelingTimeEstimated.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_modelingTimeEstimated.setText(_translate("MainWindow", "00:00"))
        self.label_35.setText(_translate("MainWindow", "Estimated time left:"))
        self.label_29.setText(_translate("MainWindow", "Elapsed time:"))
        self.label_216.setText(_translate("MainWindow", "Modeling Progress"))
        self.btn_runBatchModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Run model making from a data file stored as analyse file</p></body></html>"))
        self.btn_runBatchModel.setText(_translate("MainWindow", "Choose file and program model"))
        self.label_205.setText(_translate("MainWindow", "Build model from data"))
        self.le_modelingTimeElapsed.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_modelingTimeElapsed.setText(_translate("MainWindow", "00:00"))
        self.label_42.setText(_translate("MainWindow", "Finishing Time:"))
        self.le_modelingTimeFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_modelingTimeFinished.setText(_translate("MainWindow", "00:00:00"))
        self.le_modelingTimeActual.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_modelingTimeActual.setText(_translate("MainWindow", "00:00:00"))
        self.label_60.setText(_translate("MainWindow", "Actual Time:"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab_6), _translate("MainWindow", "Full Model"))
        self.label_37.setText(_translate("MainWindow", "Target RMS"))
        self.btn_runTargetRMSAlignment.setToolTip(_translate("MainWindow", "<html><head/><body><p>Deletes worst point of actual alignment model until error is below target RMS.</p></body></html>"))
        self.btn_runTargetRMSAlignment.setText(_translate("MainWindow", "Run Optimise"))
        self.btn_deleteWorstPoint.setToolTip(_translate("MainWindow", "<html><head/><body><p>Deletes worst point of actual alignment model in the mount. It could happen, that the overall RMS rises.</p></body></html>"))
        self.btn_deleteWorstPoint.setText(_translate("MainWindow", "Delete Worst Point"))
        self.label_124.setText(_translate("MainWindow", "Optimise actual model"))
        self.label_125.setText(_translate("MainWindow", "Models stored in mount computer:"))
        self.btn_cancelRunTargetRMSAlignment.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels the optimise run.</p></body></html>"))
        self.btn_cancelRunTargetRMSAlignment.setText(_translate("MainWindow", "Cancel Optimise"))
        self.le_alignErrorRMS2.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual RMS Error in arc sec</p><p><br/></p></body></html>"))
        self.le_alignErrorRMS2.setText(_translate("MainWindow", "0.0"))
        self.label_75.setText(_translate("MainWindow", "Actual RMS"))
        self.btn_clearModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Clear the alignment model in mount. Can\'t be revised.</p></body></html>"))
        self.btn_clearModel.setText(_translate("MainWindow", "Clear Model"))
        self.le_alignNumberStars2.setToolTip(_translate("MainWindow", "Number of alignment stars in actual used pointing model"))
        self.le_alignNumberStars2.setText(_translate("MainWindow", "0"))
        self.label_76.setText(_translate("MainWindow", "Stars"))
        self.le_alignNumberTerms2.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of Terms in actual used pointing model</p></body></html>"))
        self.le_alignNumberTerms2.setText(_translate("MainWindow", "0"))
        self.label_209.setText(_translate("MainWindow", "Terms"))
        self.btn_reloadAlignmentModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Reloads the refinement model from mount. Needed if you change things manually with handcontroller or other software</p></body></html>"))
        self.btn_reloadAlignmentModel.setText(_translate("MainWindow", "Reload Mount Model"))
        self.btn_saveModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves actual alignment model under selected name or edited name.</p></body></html>"))
        self.btn_saveModel.setText(_translate("MainWindow", "Save"))
        self.btn_loadModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the actual selected alignment model.</p></body></html>"))
        self.btn_loadModel.setText(_translate("MainWindow", "Load"))
        self.btn_deleteModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Deletes the actual selected alignment model.</p></body></html>"))
        self.btn_deleteModel.setText(_translate("MainWindow", "Delete"))
        self.checkShowErrorValues.setToolTip(_translate("MainWindow", "Use Highspeed download of CCD camera if availabe"))
        self.checkShowErrorValues.setText(_translate("MainWindow", "Show error values in polar plot"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab_8), _translate("MainWindow", "Manage Model"))
        self.label_107.setText(_translate("MainWindow", "Analyse - Flexure"))
        self.btn_cancelAnalyse.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels the analyse run.</p></body></html>"))
        self.btn_cancelAnalyse.setText(_translate("MainWindow", "Cancel Analyse Run"))
        self.btn_runFlexure.setToolTip(_translate("MainWindow", "<html><head/><body><p>Runs a analyse cycle by constant pointing to given alt/az and measuring the error each delay seconds.</p></body></html>"))
        self.btn_runFlexure.setText(_translate("MainWindow", "Run Flexure"))
        self.label_174.setText(_translate("MainWindow", "Altitude"))
        self.label_175.setText(_translate("MainWindow", "Azimuth"))
        self.label_176.setText(_translate("MainWindow", "Number Cycles"))
        self.label_111.setText(_translate("MainWindow", "°"))
        self.label_112.setText(_translate("MainWindow", "°"))
        self.label_177.setText(_translate("MainWindow", "Delay"))
        self.label_115.setText(_translate("MainWindow", "s"))
        self.label_118.setText(_translate("MainWindow", "Analyse - Hysterese"))
        self.btn_runHysterese.setToolTip(_translate("MainWindow", "<html><head/><body><p>Runs a analyse cycle by swapping two alt/az positions and measuring the error each delay seconds</p></body></html>"))
        self.btn_runHysterese.setText(_translate("MainWindow", "Run Hysteresis"))
        self.label_182.setText(_translate("MainWindow", "Azimuth"))
        self.label_183.setText(_translate("MainWindow", "Number Cycles"))
        self.label_184.setText(_translate("MainWindow", "Delay"))
        self.label_117.setText(_translate("MainWindow", "s"))
        self.label_114.setText(_translate("MainWindow", "°"))
        self.label_119.setText(_translate("MainWindow", "°"))
        self.label_185.setText(_translate("MainWindow", "Point 1    Altitude"))
        self.label_120.setText(_translate("MainWindow", "°"))
        self.label_121.setText(_translate("MainWindow", "°"))
        self.label_186.setText(_translate("MainWindow", "Azimuth"))
        self.label_187.setText(_translate("MainWindow", "Point 2    Altitude"))
        self.btn_split.setToolTip(_translate("MainWindow", "<html><head/><body><p>Runs a analyse cycle by swapping two alt/az positions and measuring the error each delay seconds</p></body></html>"))
        self.btn_split.setText(_translate("MainWindow", "Split"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Analyse), _translate("MainWindow", "Analyse"))
        self.btn_relay1.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.btn_relay1.setText(_translate("MainWindow", "Relay1"))
        self.btn_relay3.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.btn_relay3.setText(_translate("MainWindow", "Relay3"))
        self.btn_relay2.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.btn_relay2.setText(_translate("MainWindow", "Relay2"))
        self.label_96.setText(_translate("MainWindow", "Relays"))
        self.btn_relay4.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.btn_relay4.setText(_translate("MainWindow", "Relay4"))
        self.btn_relay5.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.btn_relay5.setText(_translate("MainWindow", "Relay5"))
        self.btn_relay6.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.btn_relay6.setText(_translate("MainWindow", "Relay6"))
        self.btn_relay7.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.btn_relay7.setText(_translate("MainWindow", "Relay7"))
        self.btn_relay8.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.btn_relay8.setText(_translate("MainWindow", "Relay8"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Relay), _translate("MainWindow", "Relay"))
        self.settingsTabWidget.setToolTip(_translate("MainWindow", "<html><head/><body><p>Configuration of settings</p></body></html>"))
        self.label_139.setText(_translate("MainWindow", "Environment Connection"))
        self.pd_chooseEnvironment.setToolTip(_translate("MainWindow", "<html><head/><body><p>Chooses the framework for getting environment data.</p></body></html>"))
        self.label_140.setText(_translate("MainWindow", "Dome Connection"))
        self.pd_chooseDome.setToolTip(_translate("MainWindow", "<html><head/><body><p>Chooses the framework for controlling a dome.</p></body></html>"))
        self.pd_chooseImaging.setToolTip(_translate("MainWindow", "<html><head/><body><p>Chooses which imaging application or framework should be used for modeling.</p></body></html>"))
        self.label_143.setText(_translate("MainWindow", "Imaging Connection"))
        self.pd_chooseAstrometry.setToolTip(_translate("MainWindow", "<html><head/><body><p>Chooses which astrometry / platesolving application or framework should be used for modeling.</p></body></html>"))
        self.label_149.setText(_translate("MainWindow", "Astrometry Connection"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.tab_12), _translate("MainWindow", "Connections"))
        self.btn_setupAscomDomeDriver.setToolTip(_translate("MainWindow", "<html><head/><body><p>Setup the ASCOM dome driver.</p></body></html>"))
        self.btn_setupAscomDomeDriver.setText(_translate("MainWindow", "Setup"))
        self.label_258.setText(_translate("MainWindow", "ASCOM Observing Conditions Hub"))
        self.btn_setupAscomEnvironmentDriver.setToolTip(_translate("MainWindow", "<html><head/><body><p>Setup the ASCOM Observing Conditions hub for environment data.</p></body></html>"))
        self.btn_setupAscomEnvironmentDriver.setText(_translate("MainWindow", "Setup"))
        self.label_261.setText(_translate("MainWindow", "ASCOM Dome Driver Setup"))
        self.le_ascomDomeDriverName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Button text for relay</p></body></html>"))
        self.le_ascomDomeDriverName.setText(_translate("MainWindow", "---"))
        self.label_263.setText(_translate("MainWindow", "Driver Name"))
        self.le_ascomEnvironmentDriverName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Button text for relay</p></body></html>"))
        self.le_ascomEnvironmentDriverName.setText(_translate("MainWindow", "---"))
        self.label_264.setText(_translate("MainWindow", "Driver Name"))
        self.label_262.setText(_translate("MainWindow", "PinPoint Settings"))
        self.label_265.setText(_translate("MainWindow", "Catalog:"))
        self.le_pinpointCatalogue.setToolTip(_translate("MainWindow", "<html><head/><body><p>Button text for relay</p></body></html>"))
        self.le_pinpointCatalogue.setText(_translate("MainWindow", "---"))
        self.btn_selectCatalogue.setToolTip(_translate("MainWindow", "<html><head/><body><p>Setup the ASCOM dome driver.</p></body></html>"))
        self.btn_selectCatalogue.setText(_translate("MainWindow", "Choose"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.ASCOM), _translate("MainWindow", "ASCOM / Win32 Drivers"))
        self.label_mountMAC.setText(_translate("MainWindow", "MAC Address"))
        self.label_mountIP.setText(_translate("MainWindow", "IP Address"))
        self.le_mountIP.setToolTip(_translate("MainWindow", "<html><head/><body><p>IP address of mount, ip address and host names!</p></body></html>"))
        self.le_mountIP.setText(_translate("MainWindow", "127.0.0.1"))
        self.le_mountMAC.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">MAC address of the mount</span></p></body></html>"))
        self.le_mountMAC.setText(_translate("MainWindow", "00:00:00:00:00:00"))
        self.le_remotePort.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Port number for remote access.</span></p></body></html>"))
        self.le_remotePort.setText(_translate("MainWindow", "3456"))
        self.label_193.setText(_translate("MainWindow", "Remote Access"))
        self.checkEnableRemoteAccess.setToolTip(_translate("MainWindow", "<html><head/><body><p>Enable / disable remote access for shutdown.</p></body></html>"))
        self.checkEnableRemoteAccess.setText(_translate("MainWindow", "Enable Remote Access"))
        self.label_200.setText(_translate("MainWindow", "Mount IP Direct Connection"))
        self.label_mountIP_5.setText(_translate("MainWindow", "Port Number"))
        self.le_INDIStatus.setToolTip(_translate("MainWindow", "<html><head/><body><p>Gives an indication if connection to indi server is established</p></body></html>"))
        self.label_146.setText(_translate("MainWindow", "INDI Status"))
        self.label_198.setText(_translate("MainWindow", "CCD"))
        self.le_INDIEnvironment.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the driver name of Environment device</p></body></html>"))
        self.checkEnableINDI.setToolTip(_translate("MainWindow", "<html><head/><body><p>Enable / disable the INDI subsystem.</p></body></html>"))
        self.checkEnableINDI.setText(_translate("MainWindow", "Enable INDI client running"))
        self.le_INDIServerPort.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Port address of INDI server</span></p></body></html>"))
        self.le_INDIServerPort.setText(_translate("MainWindow", "12345"))
        self.le_INDIServerIP.setToolTip(_translate("MainWindow", "<html><head/><body><p>IP address of INDI server, ip address and host names!</p></body></html>"))
        self.le_INDIServerIP.setText(_translate("MainWindow", "127.0.0.1"))
        self.label_199.setText(_translate("MainWindow", "Environment"))
        self.label_259.setText(_translate("MainWindow", "Configuration of INDI Subsystem"))
        self.le_INDICCD.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the driver name of CCD device</p></body></html>"))
        self.label_mountIP_3.setText(_translate("MainWindow", "IP / Port"))
        self.label_211.setText(_translate("MainWindow", "Dome"))
        self.le_INDIDome.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the name of the Dome device</p></body></html>"))
        self.label_260.setText(_translate("MainWindow", "Configuration of astrometry.net solver"))
        self.label_mountIP_10.setText(_translate("MainWindow", "/"))
        self.label_mountIP_2.setText(_translate("MainWindow", "IP Address"))
        self.le_relayPassword.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Password of relay switch. Is not saved encrypted !</span></p></body></html>"))
        self.le_relayPassword.setText(_translate("MainWindow", "empty"))
        self.le_relayUsername.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Username of relay switch. Is not saved encrypted !</span></p></body></html>"))
        self.le_relayUsername.setText(_translate("MainWindow", "empty"))
        self.label_196.setText(_translate("MainWindow", "Username"))
        self.checkEnableRelay.setToolTip(_translate("MainWindow", "<html><head/><body><p>Enable / disable relay functions.</p></body></html>"))
        self.checkEnableRelay.setText(_translate("MainWindow", "Enable Relais Box"))
        self.label_197.setText(_translate("MainWindow", "Password"))
        self.le_relayIP.setToolTip(_translate("MainWindow", "<html><head/><body><p>IP address of relay switch, ip address and host names!</p></body></html>"))
        self.le_relayIP.setText(_translate("MainWindow", "127.0.0.1"))
        self.label_141.setText(_translate("MainWindow", "Configuration of Relay Box (KMTronics Switch)"))
        self.le_AstrometryAPIKey.setToolTip(_translate("MainWindow", "<html><head/><body><p>API key from astrometry server.</p></body></html>"))
        self.le_AstrometryAPIKey.setText(_translate("MainWindow", "abcdefghighdfegr"))
        self.label_mountIP_8.setText(_translate("MainWindow", "Timeout"))
        self.label_mountIP_4.setText(_translate("MainWindow", "IP / Port"))
        self.label_mountIP_9.setText(_translate("MainWindow", "/"))
        self.le_astrometryTimeout.setToolTip(_translate("MainWindow", "<html><head/><body><p>Timeout for solving time.</p></body></html>"))
        self.le_astrometryTimeout.setText(_translate("MainWindow", "60"))
        self.le_AstrometryPort.setToolTip(_translate("MainWindow", "<html><head/><body><p>Port address of astrometry server</p></body></html>"))
        self.le_AstrometryPort.setText(_translate("MainWindow", "80"))
        self.label_mountIP_13.setText(_translate("MainWindow", "API Key"))
        self.le_AstrometryHost.setToolTip(_translate("MainWindow", "<html><head/><body><p>IP address of astrometry server, ip address and host names!</p></body></html>"))
        self.le_AstrometryHost.setText(_translate("MainWindow", "127.0.0.1"))
        self.label_220.setText(_translate("MainWindow", "Telescope"))
        self.le_INDITelescope.setToolTip(_translate("MainWindow", "<html><head/><body><p>Shows the name of the Dome device</p></body></html>"))
        self.checkEnableINDIListening.setToolTip(_translate("MainWindow", "<html><head/><body><p>Enable / disable the INDI subsystem.</p></body></html>"))
        self.checkEnableINDIListening.setText(_translate("MainWindow", "Enable INDI image listening"))
        self.checkEnableINDISolving.setToolTip(_translate("MainWindow", "<html><head/><body><p>Enable / disable the INDI subsystem.</p></body></html>"))
        self.checkEnableINDISolving.setText(_translate("MainWindow", "Enable INDI solving after image listening"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.tab_14), _translate("MainWindow", "INDI / IP Drivers"))
        self.btn_downloadEarthrotation.setToolTip(_translate("MainWindow", "<html><head/><body><p>Download earth rotation data</p></body></html>"))
        self.btn_downloadEarthrotation.setText(_translate("MainWindow", "Download Earth Rotation and UTC files"))
        self.label_123.setText(_translate("MainWindow", "Webservices for space data"))
        self.btn_downloadSpacestations.setToolTip(_translate("MainWindow", "<html><head/><body><p>Download space stations TLE (please note: alternatively to Satellite)</p></body></html>"))
        self.btn_downloadSpacestations.setText(_translate("MainWindow", "Download Space Stations TLE Elements"))
        self.btn_downloadSatbrighest.setToolTip(_translate("MainWindow", "<html><head/><body><p>Download Satellite TLE (please note: alternatively to space stations)</p></body></html>"))
        self.btn_downloadSatbrighest.setText(_translate("MainWindow", "Download Brightest Satellite TLE Elements"))
        self.btn_downloadAsteroidsMPC5000.setToolTip(_translate("MainWindow", "<html><head/><body><p>Download asteroids MPC file</p></body></html>"))
        self.btn_downloadAsteroidsMPC5000.setText(_translate("MainWindow", "Downl. Asteroids MPC5000"))
        self.btn_downloadComets.setToolTip(_translate("MainWindow", "<html><head/><body><p>Download comets MPC file</p></body></html>"))
        self.btn_downloadComets.setText(_translate("MainWindow", "Download Comets MPC File"))
        self.checkEarthrotation.setToolTip(_translate("MainWindow", "<html><head/><body><p>Upload earth rotation and UTC data</p></body></html>"))
        self.checkEarthrotation.setText(_translate("MainWindow", "Upload  Earth Rotation and UTC files"))
        self.checkTLE.setToolTip(_translate("MainWindow", "<html><head/><body><p>Upload TLE elements</p></body></html>"))
        self.checkTLE.setText(_translate("MainWindow", "Upload TLE Elements"))
        self.checkAsteroids.setToolTip(_translate("MainWindow", "<html><head/><body><p>Upload asteroids data</p></body></html>"))
        self.checkAsteroids.setText(_translate("MainWindow", "Upload Asteroids MPC File"))
        self.checkComets.setToolTip(_translate("MainWindow", "<html><head/><body><p>Upload comets data</p></body></html>"))
        self.checkComets.setText(_translate("MainWindow", "Upload Comets MPC File"))
        self.btn_uploadMount.setToolTip(_translate("MainWindow", "<html><head/><body><p>Start upload with 10micron updater</p></body></html>"))
        self.btn_uploadMount.setText(_translate("MainWindow", "Upload to Mount"))
        self.btn_downloadAll.setToolTip(_translate("MainWindow", "<html><head/><body><p>Download all files (please note satellite and space stations work alternatively)  </p></body></html>"))
        self.btn_downloadAll.setText(_translate("MainWindow", "Download All Files"))
        self.label_134.setText(_translate("MainWindow", "Mount Uploads"))
        self.checkFilterMPC.setToolTip(_translate("MainWindow", "<html><head/><body><p>Check to filter Comets and Asteroids MPC File</p></body></html>"))
        self.checkFilterMPC.setText(_translate("MainWindow", "Filter MPC Files with expression(s):"))
        self.le_filterExpressionMPC.setToolTip(_translate("MainWindow", "<html><head/><body><p>serach expression, separated with \',\'</p></body></html>"))
        self.btn_downloadAsteroidsNEA.setToolTip(_translate("MainWindow", "<html><head/><body><p>Download asteroids MPC file</p></body></html>"))
        self.btn_downloadAsteroidsNEA.setText(_translate("MainWindow", "Downl. Asteroids NEA"))
        self.btn_downloadAsteroidsTNO.setToolTip(_translate("MainWindow", "<html><head/><body><p>Download asteroids MPC file</p></body></html>"))
        self.btn_downloadAsteroidsTNO.setText(_translate("MainWindow", "Downl. Asteroids TNO"))
        self.btn_downloadAsteroidsPHA.setToolTip(_translate("MainWindow", "<html><head/><body><p>Download asteroids MPC file</p></body></html>"))
        self.btn_downloadAsteroidsPHA.setText(_translate("MainWindow", "Downl. Asteroids PHA"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.Uploads), _translate("MainWindow", "Uploads"))
        self.le_altParkPos3.setInputMask(_translate("MainWindow", "99"))
        self.le_altParkPos3.setText(_translate("MainWindow", "1"))
        self.le_parkPos3Text.setToolTip(_translate("MainWindow", "Button text park menu 3"))
        self.le_parkPos3Text.setText(_translate("MainWindow", "Dew"))
        self.le_parkPos1Text.setToolTip(_translate("MainWindow", "Button text park menu 1"))
        self.le_parkPos1Text.setText(_translate("MainWindow", "Flat West"))
        self.label_19.setText(_translate("MainWindow", "Az"))
        self.label_43.setText(_translate("MainWindow", "Az"))
        self.label_54.setText(_translate("MainWindow", "Configuration of Fixed Park Positions "))
        self.le_azParkPos3.setInputMask(_translate("MainWindow", "999"))
        self.le_azParkPos3.setText(_translate("MainWindow", "270"))
        self.le_parkPos2Text.setToolTip(_translate("MainWindow", "Button text park menu 2"))
        self.le_parkPos2Text.setText(_translate("MainWindow", "Flat East"))
        self.le_azParkPos1.setInputMask(_translate("MainWindow", "999"))
        self.le_azParkPos1.setText(_translate("MainWindow", "1"))
        self.label_45.setText(_translate("MainWindow", "Alt"))
        self.le_altParkPos4.setInputMask(_translate("MainWindow", "99"))
        self.le_altParkPos4.setText(_translate("MainWindow", "1"))
        self.le_altParkPos1.setInputMask(_translate("MainWindow", "99"))
        self.le_altParkPos1.setText(_translate("MainWindow", "89"))
        self.label_2.setText(_translate("MainWindow", "Button 1"))
        self.le_altParkPos2.setInputMask(_translate("MainWindow", "99"))
        self.le_altParkPos2.setText(_translate("MainWindow", "89"))
        self.label_51.setText(_translate("MainWindow", "Alt"))
        self.label_50.setText(_translate("MainWindow", "Button 4"))
        self.label_44.setText(_translate("MainWindow", "Button 2"))
        self.le_azParkPos2.setInputMask(_translate("MainWindow", "999"))
        self.le_azParkPos2.setText(_translate("MainWindow", "359"))
        self.label_49.setText(_translate("MainWindow", "Az"))
        self.label_3.setText(_translate("MainWindow", "Alt"))
        self.le_azParkPos4.setInputMask(_translate("MainWindow", "999"))
        self.le_azParkPos4.setText(_translate("MainWindow", "90"))
        self.label_46.setText(_translate("MainWindow", "Az"))
        self.label_48.setText(_translate("MainWindow", "Alt"))
        self.le_parkPos4Text.setToolTip(_translate("MainWindow", "Button text park menu 4"))
        self.le_parkPos4Text.setText(_translate("MainWindow", "CCD"))
        self.label_47.setText(_translate("MainWindow", "Button 3"))
        self.le_azParkPos5.setInputMask(_translate("MainWindow", "999"))
        self.le_azParkPos5.setText(_translate("MainWindow", "270"))
        self.label_92.setText(_translate("MainWindow", "Button 6"))
        self.le_altParkPos6.setInputMask(_translate("MainWindow", "99"))
        self.le_altParkPos6.setText(_translate("MainWindow", "1"))
        self.label_93.setText(_translate("MainWindow", "Az"))
        self.label_94.setText(_translate("MainWindow", "Alt"))
        self.label_95.setText(_translate("MainWindow", "Button 5"))
        self.le_altParkPos5.setInputMask(_translate("MainWindow", "99"))
        self.le_altParkPos5.setText(_translate("MainWindow", "1"))
        self.le_parkPos6Text.setToolTip(_translate("MainWindow", "Button text park menu 4"))
        self.le_parkPos6Text.setText(_translate("MainWindow", "CCD"))
        self.le_parkPos5Text.setToolTip(_translate("MainWindow", "Button text park menu 3"))
        self.le_parkPos5Text.setText(_translate("MainWindow", "Dew"))
        self.label_131.setText(_translate("MainWindow", "Az"))
        self.label_132.setText(_translate("MainWindow", "Alt"))
        self.le_azParkPos6.setInputMask(_translate("MainWindow", "999"))
        self.le_azParkPos6.setText(_translate("MainWindow", "90"))
        self.label_97.setText(_translate("MainWindow", "Text"))
        self.relay2Text.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relay2Text.setText(_translate("MainWindow", "free"))
        self.label_192.setText(_translate("MainWindow", "8"))
        self.relay1Text.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relay1Text.setText(_translate("MainWindow", "free"))
        self.label_161.setText(_translate("MainWindow", "6"))
        self.label_195.setText(_translate("MainWindow", "Function"))
        self.relay6Text.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relay6Text.setText(_translate("MainWindow", "free"))
        self.relay4Text.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relay4Text.setText(_translate("MainWindow", "free"))
        self.label_53.setText(_translate("MainWindow", "Relay"))
        self.label_36.setText(_translate("MainWindow", "1"))
        self.label_191.setText(_translate("MainWindow", "7"))
        self.relay8Text.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relay8Text.setText(_translate("MainWindow", "free"))
        self.relay3Text.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relay3Text.setText(_translate("MainWindow", "free"))
        self.label_150.setText(_translate("MainWindow", "4"))
        self.label_162.setText(_translate("MainWindow", "2"))
        self.label_41.setText(_translate("MainWindow", "Text"))
        self.relay7Text.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relay7Text.setText(_translate("MainWindow", "free"))
        self.label_101.setText(_translate("MainWindow", "3"))
        self.label_148.setText(_translate("MainWindow", "5"))
        self.relay5Text.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relay5Text.setText(_translate("MainWindow", "free"))
        self.label_103.setText(_translate("MainWindow", "Relay Button Text Setup"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.tab_10), _translate("MainWindow", "Position / Relay"))
        self.soundMountSlewFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when mount finished slewing</p></body></html>"))
        self.label_210.setText(_translate("MainWindow", "Play audio signals"))
        self.soundMountAlert.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when mount stopped in emergency case</p></body></html>"))
        self.soundDomeSlewFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when dome finished slewing</p></body></html>"))
        self.label_212.setText(_translate("MainWindow", "Mount slew finished"))
        self.label_214.setText(_translate("MainWindow", "Dome slew finished"))
        self.label_215.setText(_translate("MainWindow", "Mount alert for STOP"))
        self.label_217.setText(_translate("MainWindow", "Modeling finished"))
        self.soundModelingFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when mount stopped in emergency case</p></body></html>"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.tab_13), _translate("MainWindow", "Audio Signals"))
        self.le_firmwareTime.setText(_translate("MainWindow", "Firmware Time"))
        self.le_firmwareProductName.setText(_translate("MainWindow", "Product Number"))
        self.le_firmwareDate.setText(_translate("MainWindow", "Firmware Date"))
        self.firmwareProductName_2.setText(_translate("MainWindow", "Product:"))
        self.le_firmwareProductName_2.setText(_translate("MainWindow", "MountWizzard3 (C) 2016-18 MWORION"))
        self.firmwareProductName_6.setText(_translate("MainWindow", "HW Version:"))
        self.label_56.setText(_translate("MainWindow", "Product Versions"))
        self.le_hardwareVersion.setText(_translate("MainWindow", "Hardware Version"))
        self.le_firmwareNumber.setText(_translate("MainWindow", "Firmware Number"))
        self.firmwareProductName_4.setText(_translate("MainWindow", "FW Date:"))
        self.firmwareProductName_5.setText(_translate("MainWindow", "FW Time:"))
        self.firmwareProductName_3.setText(_translate("MainWindow", "Firmware:"))
        self.logginLevel.setTitle(_translate("MainWindow", "Choose Log Level"))
        self.loglevelDebug.setToolTip(_translate("MainWindow", "<html><head/><body><p>Choose local solver</p></body></html>"))
        self.loglevelDebug.setText(_translate("MainWindow", "Debug"))
        self.loglevelInfo.setToolTip(_translate("MainWindow", "<html><head/><body><p>Choose online solver nova.astrometry.net as solver</p></body></html>"))
        self.loglevelInfo.setText(_translate("MainWindow", "Info"))
        self.loglevelWarning.setToolTip(_translate("MainWindow", "<html><head/><body><p>Choose online solver nova.astrometry.net as solver</p></body></html>"))
        self.loglevelWarning.setText(_translate("MainWindow", "Warning"))
        self.loglevelError.setToolTip(_translate("MainWindow", "<html><head/><body><p>Choose online solver nova.astrometry.net as solver</p></body></html>"))
        self.loglevelError.setText(_translate("MainWindow", "Error"))
        self.label_59.setText(_translate("MainWindow", "Misc Settings"))
        self.btn_resetWindowSizes.setToolTip(_translate("MainWindow", "<html><head/><body><p>Opens the image window</p></body></html>"))
        self.btn_resetWindowSizes.setText(_translate("MainWindow", "Reset window sizes"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.tab_11), _translate("MainWindow", "Versions"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab_3), _translate("MainWindow", "Settings"))
        self.btn_saveConfigQuit.setToolTip(_translate("MainWindow", "Quits the tool and saves the settings data in config.cfg"))
        self.btn_saveConfigQuit.setText(_translate("MainWindow", "Save/Quit"))
        self.btn_saveConfig.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the configuration under the current profile</p></body></html>"))
        self.btn_saveConfig.setText(_translate("MainWindow", "Save"))
        self.btn_cameraConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of camera:</p><p>Red: Error</p><p>Yellow: Connecting</p><p>Green: Connected</p><p>Gray: No Configured</p></body></html>"))
        self.btn_cameraConnected.setText(_translate("MainWindow", "Camera"))
        self.btn_domeConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of dome: </p><p>Red: Error</p><p>Gray: No Configured</p><p>Green: Connected</p></body></html>"))
        self.btn_domeConnected.setText(_translate("MainWindow", "Dome"))
        self.btn_driverMountConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of mount</p><p>Red: Error</p><p>Yellow: Connecting</p><p>Green: Connected</p></body></html>"))
        self.btn_driverMountConnected.setText(_translate("MainWindow", "Mount"))
        self.btn_environmentConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of environment: </p><p>Red: Error</p><p>Gray: No Configured</p><p>Green: Connected</p></body></html>"))
        self.btn_environmentConnected.setText(_translate("MainWindow", "Environ"))
        self.btn_openMessageWindow.setToolTip(_translate("MainWindow", "<html><head/><body><p>Open the message and log window. The window could be sized in height.</p></body></html>"))
        self.btn_openMessageWindow.setText(_translate("MainWindow", "Message"))
        self.btn_astrometryConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of plate solver:</p><p>Red: Error</p><p>Yellow: Connecting</p><p>Green: Connected</p><p>Gray: No Configured</p></body></html>"))
        self.btn_astrometryConnected.setText(_translate("MainWindow", "Solver"))
        self.btn_loadFrom.setToolTip(_translate("MainWindow", "<html><head/><body><p>Load an existing configuration profile.</p></body></html>"))
        self.btn_loadFrom.setText(_translate("MainWindow", "Load"))
        self.btn_saveConfigAs.setToolTip(_translate("MainWindow", "<html><head/><body><p>Save the actual settings under a new profile name.</p></body></html>"))
        self.btn_saveConfigAs.setText(_translate("MainWindow", "Save as"))
        self.le_configName.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Actual used profile</span></p></body></html>"))
        self.le_configName.setText(_translate("MainWindow", "profile"))
        self.mainicon.setText(_translate("MainWindow", "TextLabel"))
        self.label_102.setText(_translate("MainWindow", "Profile"))
        self.btn_mountShutdown.setToolTip(_translate("MainWindow", "<html><head/><body><p>Send shutdown command to the mount.</p></body></html>"))
        self.btn_mountShutdown.setText(_translate("MainWindow", "OFF"))
        self.btn_mountBoot.setToolTip(_translate("MainWindow", "<html><head/><body><p>If the mount is connected via ethernet and is in the same subnet, from firmware 2.15.8 on the mount could be started by sending a WOL packet to the moutn computer.</p></body></html>"))
        self.btn_mountBoot.setText(_translate("MainWindow", "ON"))
        self.mainBackground.setProperty("color", _translate("MainWindow", "blue"))
        self.btn_INDIConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of INDI subsystem: </p><p>Red: Error</p><p>Yellow: Connecting</p><p>Green: Connected</p><p>Gray: No Configured</p></body></html>"))
        self.btn_INDIConnected.setText(_translate("MainWindow", "INDI"))
        self.le_cameraStatusText.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.le_astrometryStatusText.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.le_domeStatusText.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.label_110.setText(_translate("MainWindow", "Status"))
        self.btn_openHemisphereWindow.setToolTip(_translate("MainWindow", "<html><head/><body><p>Opens the hemisphere window, where you can see all model points, the horizon mask and the modeling progress</p></body></html>"))
        self.btn_openHemisphereWindow.setText(_translate("MainWindow", "Hemisph."))
        self.btn_openImageWindow.setToolTip(_translate("MainWindow", "<html><head/><body><p>Opens the image window</p></body></html>"))
        self.btn_openImageWindow.setText(_translate("MainWindow", "Image"))
        self.btn_openAnalyseWindow.setToolTip(_translate("MainWindow", "<html><head/><body><p>Opens the analyse window for checking measurements and modlelling data.</p></body></html>"))
        self.btn_openAnalyseWindow.setText(_translate("MainWindow", "Analyse"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QWidget()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())

